# -*- coding: utf-8 -*-

import json
import datetime
from dateutil import tz

import autocomplete_light

from django import forms
from django.db.models import Q
from django.conf import settings
from django.contrib import admin
from django.utils import translation
from django.utils.translation import ugettext as _
from django.utils.html import strip_tags
from django.core.urlresolvers import reverse
from django.utils.http import urlquote
from django.utils.translation import ugettext as ugt
from django.template.defaultfilters import slugify
from django.conf.urls import patterns, url
from django.core import serializers
from django.http import HttpResponse
from django.shortcuts import redirect
from django.contrib.admin.filters import SimpleListFilter
from ftfy import fix_text

from jmb.i18n.models import Language
from jmb.core.middleware.thread_local import get_request
from jmb.core.admin import AjaxInline, register_inline, ConstrainedModelForm
from jmb.core.admin.options import ExtendibleModelAdmin
from jmb.core.forms.models import AdminModelForm
from jmb.core.forms.widgets import DateTimePicker

from jmb.tag.models import Tag
from jmb.tag.admin import TagAdminForm
from publieditor.content import models
from publieditor.content.admin_forms import (
    CarModelEquipmentAdminForm, ContentActionForm, CarModelUpdateStatusForm,
)
from publieditor.content.admin_views import (
    set_equipment, import_unrae, insert_equipments, update_sales_data, model_sync,
    get_photos, get_videos, get_tables, OutputView, OutputViewHtml, CarImportView
)
from publieditor.content.models import Layout, Issue, Stream
from publieditor.content.admin_actions import clone_car_items, refresh_car_items_slug


default_list_display = ('id', '__unicode__', 'get_edit_icon')
default_fieldsets = (
    _('logs admin'), {
        'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        'classes': ('collapse',)
    },
)


class TranslationContentInlineForm(AdminModelForm):
    class Meta:
        model = models.TranslationContent

    tags_new = forms.CharField(
        required=False,
        widget=forms.widgets.HiddenInput()
    )

    tags = forms.ModelMultipleChoiceField(
        Tag.objects,
        required=False,
        widget=autocomplete_light.MultipleChoiceWidget("NewTagAutocomplete")
    )

    def __init__(self, *args, **kwargs):
        super(TranslationContentInlineForm, self).__init__(*args, **kwargs)
        if not self.instance or not self.instance.id:
            try:
                language_ita = Language.objects.get(code="IT")
                self.fields['language'].initial = language_ita
            except Exception, ex:
                pass

    def clean(self):
        clean = super(TranslationContentInlineForm, self).clean()
        if self.is_valid() and self.cleaned_data.get('tags_new'):
            old_tags_obj = self.cleaned_data.get('tags')
            old_tags_ids = [x.pk for x in old_tags_obj]
            new_tags = self.cleaned_data.get('tags_new').split("-----")
            new_tags_ids = []
            for tag in new_tags:
                new_tag = TagAdminForm({'name': tag, 'slug': slugify(tag)})
                if new_tag.is_valid():
                    new_tag.save()
                    new_tags_ids.append(new_tag.instance.pk)
                else:
                    raise forms.ValidationError(str(new_tag.errors))
            new_list_ids = new_tags_ids + old_tags_ids
            new_list_queryset = Tag.objects.filter(pk__in=new_list_ids)
            self.cleaned_data['tags'] = new_list_queryset

        if (
            "description" in self.cleaned_data and self.cleaned_data["description"] == "" and
            "abstract" in self.cleaned_data and self.cleaned_data["abstract"] == "" and
            "body" in self.cleaned_data and self.cleaned_data["body"] == ""
        ):
            msg = _("Please compile at least one between 'Descrizione' 'Abstract' 'Corpo'")
            raise(forms.ValidationError(msg))
        return clean


class TranslationContentInline(admin.StackedInline):
    model = models.TranslationContent
    form = TranslationContentInlineForm
    extra = 1
    max_num = 1
    fieldsets = (
        (_('main'), {
            'fields': (
                'language',
                ('title', 'slug',),
                'description',
                'abstract',
                'body',
                'tags',
                'tags_new'
            ),
        }),
    )
    prepopulated_fields = {"slug": ("title",)}


class PhotoContentGalleryAjaxInline(AjaxInline):
    model = models.PhotoContentGallery
    extra = 0
    list_display = ('get_photo',)


class PhotoContentGalleryPrepopulatedInlineForm(forms.ModelForm):
    '''
    serve a gestire l'assenza dell'url nel campo photo 
    (attualmente e' presente solo il nome della foto, ed 
    il path viene applicato dal metodo 'get_photo_url' nella classe model)
    '''
    def __init__(self, *args, **kwargs):
        super(PhotoContentGalleryPrepopulatedInlineForm, self).__init__(*args, **kwargs)
        self.initial["photo"] = self.instance.get_photo_url()

    class Meta:
        model = models.PhotoContentGallery


class PhotoContentGalleryInline(admin.TabularInline):
    form = PhotoContentGalleryPrepopulatedInlineForm
    model = models.PhotoContentGallery
    extra = 0
    fields = ('photo', 'photo_image')
    readonly_fields = ('photo_image',)


class ContentInline(admin.TabularInline):
    model = models.Content
    extra = 0
    fields = ('get_edit_icon', 'get_id', 'get_date_content', 'status', 'stream', 'layout', 'get_title',)

    readonly_fields = ('get_id', 'get_edit_icon', 'get_date_content', 'status', 'stream', 'layout', 'get_title',)

    def get_id(self, obj):
        if obj.parent:
            return "<strong>%s</strong><br><span class='parent_id'>%s</span>" % (obj.id, obj.parent.id)
        return "<strong>%s</strong>" % obj.id
    get_id.short_description = _('ID')
    get_id.admin_order_field = 'id'
    get_id.allow_tags = True

    def get_title(self, obj):
        if obj.translations.all():
            translation = obj.translations.all()[0]
            title = translation.title
            sector = obj.sector
            return "<b>%s</b><br/>%s <span class='issue'>%s</span>" % (title, sector, obj.kind)
        return obj.name
    get_title.allow_tags = True
    get_title.short_description = _('Title')
    get_title.admin_order_field = 'title'

    def get_edit_icon(self, obj):
        ## per condividerla con TabularInline
        return get_edit_icon(self, obj)
    get_edit_icon.short_description = _("E")
    get_edit_icon.allow_tags = True

    def get_img_icon(self, obj):
        try:
            # get_first_photo_url_dict a volte rotorna None per ora
            a_content = '<img src="%s" alt="%s" width="100px"/>' % (
                obj.get_first_photo_url_dict()['thumb'],
                obj.title
            )
        except:
            a_content = obj.id

        txt = '<a href="%s">%s</a>' % (
            reverse('admin:content_content_change', args=(obj.id,)),
            a_content
        )
        return txt
    get_img_icon.short_description = 'link'
    get_img_icon.allow_tags = True

    def get_date_content(self, obj):
        return obj.date_content.strftime('%d/%m/%Y')
    get_date_content.short_description = _('Date')
    get_date_content.admin_order_field = 'date_content'


class MediaPhotoInline(admin.TabularInline):
    template = 'admin/content/media/edit_inline/tabular.html'
    model = models.MediaPhoto
    extra = 0
    fields = ('photo_id', 'get_thumb', 'title', 'description', 'width', 'height', 'weigth', 'mime_type')
    readonly_fields = ('photo_id', 'width', 'get_thumb', 'height', 'weigth', 'mime_type')

    class Media:
        css = {
            'all': ('/static/admin/css/tabular.css',)
        }

    def get_thumb(self, obj):
        return """
            <img src="%s" alt="%s" width="100px"/>
        """ % (
            obj.get_photo_url_dict()['thumb43'],
            obj.title
        )
    get_thumb.allow_tags = True


class MediaTableInline(admin.TabularInline):
    template = 'admin/content/media/edit_inline/tabular.html'
    model = models.MediaTable
    extra = 0
    fields = ('table_id', 'get_thumb', 'title', 'description', 'width', 'height', 'weigth', 'mime_type')
    readonly_fields = ('table_id', 'width', 'get_thumb', 'height', 'weigth', 'mime_type')

    def get_thumb(self, obj):
        return """
            <img src="%s" alt="%s" width="100px"/>
        """ % (
            obj.get_table_url_dict()['thumb'],
            obj.title
        )
    get_thumb.allow_tags = True


class MediaVideoInline(admin.TabularInline):
    template = 'admin/content/media/edit_inline/tabular.html'
    model = models.MediaVideo
    extra = 0
    fields = ('video_id', 'get_thumb', 'title', 'description', 'width', 'height', 'weigth', 'mime_type')
    readonly_fields = ('video_id', 'width', 'get_thumb', 'height', 'weigth', 'mime_type')

    def get_thumb(self, obj):
        return """
            <img src="%s" alt="%s" width="100px"/>
        """ % (
            obj.get_video_url_dict()['thumb'],
            obj.title
        )
    get_thumb.allow_tags = True


class PhotoContentGalleryInlineForm(ConstrainedModelForm):
    class Meta:
        model = models.PhotoContentGallery
        

class PhotoContentGalleryAdmin(ExtendibleModelAdmin):
    list_display = default_list_display + ('content', 'get_photo',)
    fieldsets = (
        ('Additional information', {
            'fields': (('date_create', 'date_last_modify', 'creator', 'last_modifier',)),
            'classes': ('collapse',)
        }),
        (None, {'fields': ('photo',)}),
    )
    form = PhotoContentGalleryInlineForm


class VideoContentGalleryAjaxInline(AjaxInline):
    model = models.VideoContentGallery
    extra = 0
    list_display = ('get_video',)
    fields = ('get_video',)
    readonly_fields = ('get_video',)


class VideoContentGalleryPrepopulatedInlineForm(forms.ModelForm):
    '''
    serve a gestire l'assenza dell'url nel campo video 
    (attualmente e' presente solo il nome del video, ed 
    il path viene applicato dal metodo 'get_video_url' nella classe model)
    '''
    def __init__(self, *args, **kwargs):
        super(VideoContentGalleryPrepopulatedInlineForm, self).__init__(*args, **kwargs)
        self.initial["video"] = self.instance.get_video_url()

    class Meta:
        model = models.VideoContentGallery


class VideoContentGalleryInline(admin.TabularInline):
    form = VideoContentGalleryPrepopulatedInlineForm
    model = models.VideoContentGallery
    extra = 0
    fields = ('video', 'video_image')
    readonly_fields = ('video_image',)


class VideoContentGalleryAdmin(ExtendibleModelAdmin):
    list_display = default_list_display + ('content', 'get_video',)
    

class PhotoCarModelGalleryInline(admin.TabularInline):    
    model = models.PhotoCarModelGallery
    extra = 0
    fields = ('photo', 'photo_image')
    readonly_fields = ('photo_image',)
    

class VideoCarModelGalleryInline(admin.TabularInline):    
    model = models.VideoCarModelGallery
    extra = 0
    fields = ('video', 'video_image')
    readonly_fields = ('video_image',)


class CarContentAjaxInline(AjaxInline):
    model = models.CarContent
    extra = 0
    list_display = (
        'corporate', 'brand', 'model', 'car'
    )


class CarContentInlineForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(CarContentInlineForm, self).__init__(*args, **kwargs)
        self.fields['car'] = forms.ModelChoiceField(
            models.Car.objects.all(),
            required=False,
            widget=autocomplete_light.ChoiceWidget('CarAutocomplete')
        )
        self.fields['model'] = forms.ModelChoiceField(
            models.CarModel.objects.all(),
            required=False,
            widget=autocomplete_light.ChoiceWidget('CarModelAutocomplete')
        )
        self.fields['brand'] = forms.ModelChoiceField(
            models.Brand.objects.all(),
            required=False,
            widget=autocomplete_light.ChoiceWidget('BrandAutocomplete')
        )
        self.fields['corporate'] = forms.ModelChoiceField(
            models.Corporate.objects.all(),
            required=False,
            widget=autocomplete_light.ChoiceWidget('CorporateAutocomplete')
        )

    class Meta:
        model = models.CarContent


class CarContentInline(admin.TabularInline):
    model = models.CarContent
    extra = 0
    fields = ('corporate', 'brand', 'model', 'car',)
    form = CarContentInlineForm


class CorporateAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'id', 'name', 'get_tooltip_note',
        'kind', 'status',
    )
    advanced_search_fields = (
        ('id', 'name',),
        ('kind__in', 'status__in'),
    )
    search_fields = ('name',)
    list_filter = ('status',)
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                ('name', 'nick',),
                'slug',
                ('status', 'kind',),
                ('alts', 'ticker'),
                ('url', 'wiki',),
                'note',
            )
        })
    )

    def get_actions(self, request,*args, **kw):
        actions = super(CorporateAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        return actions

    def get_tooltip_note(self, obj):
        img_pth = "%sadmin/images/view.gif" % settings.STATIC_URL
        code = "<img src='%s' title='%s' />" % (img_pth, obj.note) if obj.note else ""
        return code
    get_tooltip_note.allow_tags = True
    get_tooltip_note.short_description = _('Note')


class BrandAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'get_link_list_model', 'id',
        'get_logo_small', 'get_logo_big',
        'name', 'get_tooltip_note',
        'auto', 'moto', 'truck', 'model_count', 'car_count',
        'kind', 'status', 'market'
    )
    advanced_search_fields = (
        ('id', 'name',),
        ('auto', 'moto', 'truck',),
        ('kind__in', 'status__in'),
    )
    search_fields = ('name',)
    list_filter = ('status', 'auto', 'moto', 'truck')
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                ('name', 'nick', 'slug'),
                ('status', 'kind',),
                ('popolarity',),  # corporates
                ('alts', 'shorts'),
                ('url', 'logo'),
                ('logo_small', 'logo_big'),
                'auto', 'moto', 'truck',
                ('start_date', 'end_date',),
                'market',
                ('web', 'phone',),
                'note',
            )
        })
    )

    def get_actions(self, request, *args, **kw):
        actions = super(BrandAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        return actions

    def get_tooltip_note(self, obj):
        img_pth = "%sadmin/images/view.gif" % settings.STATIC_URL
        code = "<img src='%s' title='%s' />" % (img_pth, obj.note) if obj.note else ""
        return code
    get_tooltip_note.allow_tags = True
    get_tooltip_note.short_description = _('Note')

    def get_link_list_model(self, obj):
        return """
            <a href="%(url)s?brand__id=%(brand_id)s" title="%(title)s" target="_blank">
                %(link)s
            </a>
        """ % {
            'url': reverse('admin:content_carmodel_changelist'),
            'brand_id': obj.id,
            'title': _('list models'),
            'link': _('LM')
        }
    get_link_list_model.short_description = _('LM')
    get_link_list_model.allow_tags = True


class CarModelForm(AdminModelForm):
    def __init__(self, *args, **kwargs):
        super(CarModelForm, self).__init__(*args, **kwargs)
        self.fields['brand'] = forms.ModelChoiceField(
            models.Brand.objects.all(),
            required=False,
            widget=autocomplete_light.ChoiceWidget('BrandAutocompleteBase')
        )

    def clean_slug(self):
        slug = slugify(self.cleaned_data['slug'])
        qs = self._meta.model.objects.filter(slug__iexact=slug)
        if self.instance.id:
            qs = qs.exclude(id=self.instance.id)
        if qs.count():
            raise forms.ValidationError(_("Slug already exists"))
        return slug

    class Meta:
        model = models.CarModel


class CarModelAdmin(ExtendibleModelAdmin):
    form = CarModelForm
    list_display = (
        'get_edit_icon', 'id', 'get_link_set_equipment', 'get_link_update_versions',
        'brand', 'name', 'status', 'get_tooltip_note', 'new_entry',
    )
    advanced_search_fields = (
        ('id', 'name__icontains',),
        ('brand__in', 'status__in'),
    )
    search_fields = ('name',)
    list_filter = ('status',)
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                ('name', 'slug',),
                ('status', 'new_entry',),
                ('brand', 'brand_info',),
                'alts',
                'multi',
                'descr',
                ('new_date', 'mod_date',),
                'pmodels_id',
                'safety',
                'euroncap_name',
                'shapes_version',
                'driving',
                'phase',
                'note',
                'brochure_link',
                'listino_link'
            )
        })
    )
    inlines = (PhotoCarModelGalleryInline, VideoCarModelGalleryInline)
    action_form = CarModelUpdateStatusForm
    action_form_fields = {
        'update_status': ['status'],
    }
    ordering = ('-id',)
    readonly_fields = ('shapes_version',)

    def get_actions(self, request,*args, **kw):
        actions = super(CarModelAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        actions['update_status'] = (self.update_status, 'update_status', _('Update status'))
        return actions

    @staticmethod
    def update_status(modeladmin, request, queryset):
        status_to_change_to = request.POST.get("status")
        queryset.update(status=status_to_change_to)
        return redirect(request.META.get('HTTP_REFERER'))

    def get_tooltip_note(self, obj):
        img_pth = "%sadmin/images/view.gif" % settings.STATIC_URL
        code = "<img src='%s' title='%s' />" % (img_pth, obj.note) if obj.note else ""
        return code
    get_tooltip_note.allow_tags = True
    get_tooltip_note.short_description = _('Note')

    def get_link_set_equipment(self, obj):
        return """
            <a href="%(url)s" title="%(title)s">
                %(link)s
            </a>
        """ % {
            'url': reverse('admin:content_carmodel_set_equipment', args=(obj.id,)),
            'title': _('sets equipments'),
            'link': _('set')
        }
    get_link_set_equipment.short_description = _('Equipments')
    get_link_set_equipment.allow_tags = True

    def get_link_update_versions(self, obj):
        return """
            <a href="%(url)s" title="%(title)s">
                %(link)s
            </a>
        """ % {
            'url': reverse('admin:content_car_update', args=(obj.id,)),
            'title': _('update versions'),
            'link': _('update')
        }
    get_link_update_versions.short_description = _('Versions')
    get_link_update_versions.allow_tags = True

    def get_urls(self):
        urls = super(CarModelAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^equipment/(?P<carmodel_id>\d+)/$',
                self.admin_site.admin_view(set_equipment),
                name='content_carmodel_set_equipment'
            ),
            url(
                r'^insert_equipments/(?P<carmodel_id>\d+)/$',
                self.admin_site.admin_view(insert_equipments),
                name='content_carmodel_insert_equipments'
            ),         
        )

        return my_urls + urls

    def get_link_list_car(self, obj):
        return """
            <a href="%(url)s?model__id=%(model_id)s" title="%(title)s" target="_blank">
                %(link)s
            </a>
        """ % {
            'url': reverse('admin:content_car_changelist'),
            'model_id': obj.id,
            'title': _('list cars'),
            'link': _('LC')
        }
    get_link_list_car.short_description = _('LC')
    get_link_list_car.allow_tags = True


class CarAdminForm(AdminModelForm):
    def __init__(self, *args, **kwargs):
        super(CarAdminForm, self).__init__(*args, **kwargs)
        car_id = self.get_object_id('car')
        self.fields['corporate'] = forms.ModelChoiceField(
            models.Corporate.objects.all(),
            required=False,
            widget=autocomplete_light.ChoiceWidget('CorporateAutocomplete')
        )
        self.fields['brand'] = forms.ModelChoiceField(
            models.Brand.objects.all(),
            required=True,
            widget=autocomplete_light.ChoiceWidget('BrandAutocompleteDynamic')
        )
        self.fields['model'] = forms.ModelChoiceField(
            models.CarModel.objects.all(),
            required=True,
            widget=autocomplete_light.ChoiceWidget('CarModelAutocompleteDynamic')
        )
        self.fields['version'].required = True

    class Meta:
        model = models.Car

    def clean_slug(self):
        slug = slugify(self.cleaned_data['slug'])
        qs = self._meta.model.objects.filter(slug__iexact=slug)
        if self.instance.id:
            qs = qs.exclude(id=self.instance.id)
        if qs.count():
            raise forms.ValidationError(_("Slug already exists"))
        return slug


class CarChangeListAdminForm(AdminModelForm):

    class Meta:
        model = models.Car

    def clean_slug(self):
        slug = slugify(self.cleaned_data['slug'])
        qs = self._meta.model.objects.filter(slug__iexact=slug)
        if self.instance.id:
            qs = qs.exclude(id=self.instance.id)
        if qs.count():
            raise forms.ValidationError(_("Slug already exists"))
        return slug


class CarAdmin(ExtendibleModelAdmin):

    class Media:
        js = (
            "content/car/js/slug_refresh.js",
        )

    form = CarAdminForm
    ordering = ('-status', 'id', 'version')
    prepopulated_fields = {'slug': ('version',)}

    advanced_search_fields = (
        ('id', 'version__icontains',),
        ('brand__in', 'model__in',),
        ('status__in',),
        ('number_of_engines',),
        ('combustion_engine_power_cv__gte', 'combustion_engine_power_cv__lte',),
        ('combustion_engine_power_kw__gte', 'combustion_engine_power_kw__lte',),
        ('combustion_engine_torque_Nm__gte', 'combustion_engine_torque_Nm__lte',),
        ('electric_engine_power_cv__gte', 'electric_engine_power_cv__lte',),
        ('electric_engine_power_kw__gte', 'electric_engine_power_kw__lte',),
        ('electric_engine_torque_Nm__gte', 'electric_engine_torque_Nm__lte',),
        ('total_power_cv__gte', 'total_power_cv__lte',),
        ('total_power_kw__gte', 'total_power_kw__lte',),
        ('total_torque_Nm__gte', 'total_torque_Nm__lte',),
        ('batteries_capacity__gte', 'batteries_capacity__lte',),
        ('usable_capacity__gte', 'usable_capacity__lte',),
        ('on_board_battery_charger__gte', 'on_board_battery_charger__lte',),
        ('average_electric_consumption__gte', 'average_electric_consumption__lte',),
        ('ac_charging_time_at_3_7_kw__gte', 'ac_charging_time_at_3_7_kw__lte',),
        ('ac_charging_time_at_7_5_kw__gte', 'ac_charging_time_at_7_5_kw__lte',),
        ('ac_charging_time_at_11_kw__gte', 'ac_charging_time_at_11_kw__lte',),
        ('ac_charging_time_at_22_kw__gte', 'ac_charging_time_at_22_kw__lte',),
        ('dc_charging_time_at_150_kw__gte', 'dc_charging_time_at_150_kw__lte',),
        ('electric_autonomy_in_km__gte', 'electric_autonomy_in_km__lte',),
        ('total_autonomy_in_km__gte', 'total_autonomy_in_km__lte',),
        ('top_speed_in_EV__gte', 'top_speed_in_EV__lte',),
    )
    search_fields = ('version',)
    list_display = (
        'get_edit_icon', 'id', 'status', 'corporate', 'brand', 'model', 'version',
        'price', 'fuel', 'displ', 'power_cv', 'layout', 'date_last_modify'
    )
    # list_display_csv = (
    #     'status', 'infocar', 'get_corporate_id', 'get_corporate_name', 'get_brand_id', 'get_brand_name',
    #     'get_model_id', 'get_model_name', 'get_model_status', 'motor', 'version', 'price', 'shape',
    #     'target', 'seat', 'engine', 'layout', 'turbo', 'fuel', 'traction', 'gear', 'agear', 'pollution',
    #     'engine_loc', 'doors', 'luggage', 'luggage_max',
    #     'tank', 'weight', 'length', 'width', 'height', 'displ', 'power_cv', 'power_kw', 'power_rpm',
    #     'torque_max', 'torque_rpm', 'speed', 'acc', 'co2', 'tow', 'cons_mix',
    #     'turning_circle', 'warranty', 'wheelbase', 'kml_mix', 'get_neopatentati', 'get_low_emission',
    #     'get_area_c', 'power_spec', 'power_weight', 'proxymity', 'get_price_update', 'field_src',
    #     'get_new_date', 'get_mod_date', 'standard_num', 'slug'
    # )
    list_filter = ('status',)
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'id',
                'status',
                ('version', 'slug'),
                'infocar',
                'corporate',
                'brand',
                'model',
                'motor',
                'price',
                'shape',
                'target',
                'seat',
                'engine',
                'layout',
                'turbo',
                'fuel',
                'traction',
                'gear',
                'pollution',
                'engine_loc',
                'doors',
                'luggage',
                'luggage_max',
                'tank',
                'weight',
                'length',
                'width',
                'height',
                'displ',
                'power_cv',
                'power_kw',
                'power_rpm',
                'torque_max',
                'torque_rpm',
                'speed',
                'acc',
                'co2',
                'tow',
                'cons_mix',
                'turning_circle',
                'wheelbase',
                'warranty',
                'number_of_engines',
                'combustion_engine_power_cv',
                'combustion_engine_power_kw',
                'combustion_engine_torque_Nm',
                'electric_engine_power_cv',
                'electric_engine_power_kw',
                'electric_engine_torque_Nm',
                'total_power_cv',
                'total_power_kw',
                'total_torque_Nm',
                'batteries_capacity',
                'usable_capacity',
                'on_board_battery_charger',
                'average_electric_consumption',
                'ac_charging_time_at_3_7_kw',
                'ac_charging_time_at_7_5_kw',
                'ac_charging_time_at_11_kw',
                'ac_charging_time_at_22_kw',
                'dc_charging_time_at_150_kw',
                'electric_autonomy_in_km',
                'total_autonomy_in_km',
                'top_speed_in_EV',
            )
        }),
        (_('data'), {
            'fields': (
                'kml_mix',
                'driving',
                'eco',
                'area_c',
                'power_spec',
                'electric_average_mileage',
                'power_weight',
                'proxymity',
                'price_update',
                'field_src',
                'shape_old',
                'new_date',
                'mod_date',
                'standard_num',
                'testo_renderizzato'
            )
        }),
    )
    actions = [
        clone_car_items,
        refresh_car_items_slug,
    ]
    readonly_fields = ('id', 'testo_renderizzato',)

    def testo_renderizzato(self, obj):
        return obj.get_autocomposed_text()
        
    def get_actions(self, request,*args, **kw):
        actions = super(CarAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        return actions

    def queryset(self, request):
        """
        verifica che venga chiamato /admin/content/car/update/<carmodel_id>/
        modifica il queryset e visualizza solo le versione del modello in url
        """
        path = request.get_full_path()
        elem_path = path.split('/')[1:-1]
        qs = super(CarAdmin, self).queryset(request)
        if len(elem_path) == 5 and (
            elem_path[0] == 'admin' and
            elem_path[1] == 'content' and
            elem_path[2] == 'car' and
            elem_path[3] == 'update'
        ):
            carmodel_id = elem_path[4]
            qs = qs.filter(model_id=carmodel_id)
        return qs

    def formfield_for_dbfield(self, db_field, **kwargs):
        request = kwargs['request']
        path = request.get_full_path()
        elem_path = path.split('/')[1:-1]
        try:
            carmodel_id = int(elem_path[4])
        except Exception:
            carmodel_id = None
        if (
            db_field.choices and
            carmodel_id and
            len(elem_path) == 5 and
            elem_path[0] == 'admin' and
            elem_path[1] == 'content' and
            elem_path[2] == 'car' and
            elem_path[3] == 'update'
        ):
            db_field._choices = tuple([(new_el[0], new_el[0]) for new_el in [list(el) for el in db_field.choices]])
        return super(CarAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    def changelist_view(self, request, extra_context=None, carmodel_id=None):
        """
        modifica list_display e list_editable se url chiamato è
        /admin/content/car/update/<carmodel_id>/
        """
        path = request.get_full_path()
        elem_path = path.split('/')[1:-1]
        extra_context = None

        self.list_editable = ()
        self.list_display = (
            'get_edit_icon', 'id', 'status', 'corporate', 'brand', 'model', 'version',
            'price', 'fuel', 'displ', 'power_cv', 'layout',
        )

        if carmodel_id and len(elem_path) == 5 and (
            elem_path[0] == 'admin' and
            elem_path[1] == 'content' and
            elem_path[2] == 'car' and
            elem_path[3] == 'update'
        ):
            self.list_display = (
                'status', 'id', 'version', 'slug', 'motor', 'price',
                'shape', 'target', 'seat',
                'engine', 'layout', 'turbo', 'fuel', 'traction', 'gear',
                'pollution', 'engine_loc', 'doors', 'luggage',
                'luggage_max', 'tank', 'weight', 'length', 'width',
                'height', 'displ', 'power_cv', 'power_kw', 'power_rpm',
                'torque_max', 'torque_rpm', 'speed', 'acc', 'co2', 'tow',
                'cons_mix',
                'kml_mix', 'get_driving', 'get_eco', 'get_area_c', 'power_spec',
                'power_weight',
                'get_standard_num',
                'turning_circle', 'wheelbase', 'warranty',
                'number_of_engines',
                'combustion_engine_power_cv',
                'combustion_engine_power_kw',
                'combustion_engine_torque_Nm',
                'electric_engine_power_cv',
                'electric_engine_power_kw',
                'electric_engine_torque_Nm',
                'total_power_cv',
                'total_power_kw',
                'total_torque_Nm',
                'batteries_capacity',
                'usable_capacity',
                'on_board_battery_charger',
                'average_electric_consumption',
                'ac_charging_time_at_3_7_kw',
                'ac_charging_time_at_7_5_kw',   
                'ac_charging_time_at_11_kw',
                'ac_charging_time_at_22_kw',
                'dc_charging_time_at_150_kw',
                'electric_autonomy_in_km',
                'total_autonomy_in_km',
                'top_speed_in_EV',
                'mod_date',
            )

            self.list_editable = (
                'status', 'slug', 'motor', 'version', 'price',
                'shape', 'target', 'seat',
                'engine', 'layout', 'turbo', 'fuel', 'traction', 'gear',
                'pollution', 'engine_loc', 'doors', 'luggage',
                'luggage_max', 'tank', 'weight', 'length', 'width',
                'height', 'displ', 'power_cv', 'power_kw', 'power_rpm',
                'torque_max', 'torque_rpm', 'speed', 'acc', 'co2', 'tow',
                'cons_mix',
                'turning_circle', 'wheelbase', 'warranty',
                'number_of_engines',
                'combustion_engine_power_cv',
                'combustion_engine_power_kw',
                'combustion_engine_torque_Nm',
                'electric_engine_power_cv',
                'electric_engine_power_kw',
                'electric_engine_torque_Nm',
                'total_power_cv',
                'total_power_kw',
                'total_torque_Nm',
                'batteries_capacity',
                'usable_capacity',
                'on_board_battery_charger',
                'average_electric_consumption',
                'ac_charging_time_at_3_7_kw',
                'ac_charging_time_at_7_5_kw', 
                'ac_charging_time_at_11_kw',
                'ac_charging_time_at_22_kw',
                'dc_charging_time_at_150_kw',
                'electric_autonomy_in_km',
                'total_autonomy_in_km',
                'top_speed_in_EV',
            )
            corporate = None
            brand = None
            model = None
            cars = self.queryset(request)
            for car in cars:
                corporate = car.corporate.name if (not corporate and car.corporate and car.corporate.name and car.corporate.name != "") else corporate
                brand = car.brand.name if (not brand and car.brand and car.brand.name and car.brand.name != "") else brand
                model = car.model.name if (not model and car.model and car.model.name and car.model.name != "") else model
            if cars.count():
                extra_context = {
                    "update": True,
                    "corporate": corporate,
                    "brand": brand,
                    "model": model,
                }
        return super(CarAdmin, self).changelist_view(request, extra_context=extra_context)

    def get_changelist_formset(self, request, **kwargs):
        from django.forms.models import modelformset_factory
        from functools import partial
        defaults = {
            "formfield_callback": partial(self.formfield_for_dbfield, request=request),
        }
        defaults.update(kwargs)
        return modelformset_factory(
            self.model, form=CarChangeListAdminForm, extra=0,
            fields=self.list_editable, **defaults
        )

    def get_urls(self):
        urls = super(CarAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^update/(?P<carmodel_id>\d+)/$',
                self.changelist_view,
                name='content_car_update'
            ),
            url(
                r'^file_import/$',
                self.admin_site.admin_view(CarImportView.as_view()),
                name='content_car_import'
            ),
        )
        return my_urls + urls

    def response_add(self, request, obj, **kwargs):
        response = super(CarAdmin, self).response_add(request, obj, **kwargs)
        if obj.model:
            response['location'] = reverse("admin:content_car_update", args=(obj.model.id,))
        return response

    def get_area_c(self, obj):
        return obj.area_c
    get_area_c.short_description = _('AreaC')
    get_area_c.boolean = True

    def get_eco(self, obj):
        if obj.eco == 1:
            return True
        return False
    get_eco.short_description = _('Eco')
    get_eco.boolean = True

    def get_driving(self, obj):
        return obj.driving
    get_driving.short_description = _('Neop')
    get_driving.boolean = True

    def get_standard_num(self, obj):
        if obj.standard_num:
            return obj.standard_num
        return ''
    get_standard_num.short_description = _('St. num')
    get_standard_num.allow_tags = True


class CarContentAdmin(ExtendibleModelAdmin):
    list_display = default_list_display


class CorporateBrandLinkAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'id', 'corporate', 'brand', 'kind',
        'start_date', 'end_date'
    )
    advanced_search_fields = (
        ('id',),
        ('corporate__name', 'brand__name',),
        ('kind',),
        ('start_date__range', 'end_date__range',),
    )
    search_fields = ('corporate__name', 'brand__name',)
    list_filter = ('kind',)
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                ('corporate', 'brand',),
                ('kind',),
                ('start_date', 'end_date',),
            )
        })
    )

    def get_actions(self, request,*args, **kw):
        actions = super(CorporateBrandLinkAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        return actions


class HitAdminForm(AdminModelForm):

    class Meta:
        model = models.Hit
        
    def clean_filters(self):
        value = self.cleaned_data['filters']
        if not value:
            return value
        try:
            json.loads(value)
        except:
            value = value.replace("'", '"')
            try:
                json.loads(value)
            except:
                msg = _("Field could not be stored. check syntax for json")
                raise(forms.ValidationError(msg))
        return value

    def clean(self):
        if not self.cleaned_data['filters']:
            return self.cleaned_data
        filters = json.loads(self.cleaned_data['filters'])
        if filters:
            if self.cleaned_data['table'] == 'version':
                try:
                    qs = models.Car.objects.filter(**filters)
                except:
                    msg = _("Check the filters. They don't seem to apply to Cars")
                    raise(forms.ValidationError(msg))
            else:
                try:
                    qs = models.CarModelSale.objects.filter(**filters)
                except:
                    msg = _("Check the filters. They don't seem to apply to CarModelSale")
                    raise(forms.ValidationError(msg))
                self.cleaned_data['field'] = 'sold'
        return self.cleaned_data


class HitAdmin(ExtendibleModelAdmin):
    form = HitAdminForm
    list_display = (
        'get_edit_icon', 'id', 'status',
        'title', 'field', 'filters', 'unit', 'ordering_field', 'limit',
    )
    advanced_search_fields = (
        ('id', 'title__icontains',),
        ('status__in', 'field__icontains'),
    )
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                ('status', 'table'),
                ('title', 'slug',),
                ('description', 'filters'), 
                ('field', 'unit',),
                ('ordering_field', 'operator'),
                ('limit', 'ordering'),
            )
        })
    )


class StreamAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'id', 'status',
        'name', 'language', 'is_filter'
    )
    list_editable = ('is_filter',)
    advanced_search_fields = (
        ('id', 'name',),
        ('status__in',),
    )
    search_fields = ('name',)
    list_filter = ('status',)
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'name', 'is_filter', 'info', 'status', 'language',
            )
        })
    )

    def get_actions(self, request,*args, **kw):
        actions = super(StreamAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        return actions


class LayoutAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'id', 'status', 'name',
    )
    advanced_search_fields = (
        ('id', 'name',),
        ('status__in',),
    )
    search_fields = ('name',)
    list_filter = ('status',)
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'name', 'status', 'contexts',
            )
        })
    )

    def get_actions(self, request,*args, **kw):
        actions = super(LayoutAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        return actions


class ContentIssueInline(admin.TabularInline):
    model = models.Content
    extra = 0
    readonly_fields = ('get_edit_id', 'code', 'page', 'get_title', 'get_photo_html_4_3_inline', 'get_table_html_4_3_inline',)
    fields = ('ordering', 'get_edit_id', 'code', 'page', 'get_title', 'get_photo_html_4_3_inline', 'get_table_html_4_3_inline',)
    ordering = ('ordering', "page", "id")
    can_delete = False

    def get_edit_id(self, obj):
        request = get_request()
        opts = self.model._meta
        app_label = opts.app_label
        layout_name = "content"
        if obj.layout:
            layout_name = obj.layout.name.lower().replace(" ", "")
        dj_reverse_url = reverse('admin:%s_%s_change' % (
             app_label, layout_name
        ), args=(obj.id,))
        return """<a href=%(reverse_url)s>%(obj_id)s</a>""" % {
            'reverse_url': "%s?%s=%s" % (dj_reverse_url, '_return_to', urlquote(request.get_full_path())),
            'obj_id': obj.id,
        }
    get_edit_id.short_description = _("ID")
    get_edit_id.allow_tags = True

    def queryset(self, request):
        queryset = super(ContentIssueInline, self).queryset(request)
        issue_id = request.session.get("admin_publieditor_content_issue_id", "")
        if queryset.count() > 0 and issue_id!="":
            issue = Issue.objects.get(id=issue_id)
            if issue.stream:
                if issue.stream.name == "AutolinkNews":
                    autolinknews_stream = issue.stream
                    autolink_stream = Stream.objects.get(name="Autolink")
                    new_queryset = queryset.filter(Q(stream=autolinknews_stream) | Q(stream=autolink_stream), issue=issue)
                else:
                    new_queryset = queryset.filter(stream=issue.stream, issue=issue)
                return new_queryset
        return queryset

    def get_photo_html_4_3_inline(self, obj):
        html_code = ""
        if obj.get_first_photo():
            html_code = "<img src='%s' alt='%s' width='70px' style='float:left!important;'/>" %(
                obj.get_first_photo_url_dict()['thumb43'],
                obj.name,
            )
        return html_code
    get_photo_html_4_3_inline.allow_tags = True
    get_photo_html_4_3_inline.short_description = _('Photo')

    def get_table_html_4_3_inline(self, obj):
        html_code = ""
        if obj.get_first_table():
            html_code = "<img src='%s' alt='%s' width='70px' style='float:left!important;'/>" %(
                obj.get_first_table_url_dict()['orig'],
                obj.name,
            )
        return html_code
    get_table_html_4_3_inline.allow_tags = True
    get_table_html_4_3_inline.short_description = _('Table')

    def get_title(self, obj):
        return obj.title
    get_title.short_description = _('Titolo')


class IssueAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'get_output', 'get_attachment_link', 'id', 'get_pub_date', 'status',
        'stream', 'get_tooltip_note', 'number', 'content_count'
    )
    advanced_search_fields = (
        ('pub_date__range', 'number',),
        ('stream__in', 'status__in',),
    )
    search_fields = ('number',)
    list_filter = ('status',)
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'status', 'stream', 'number', 'pub_date',
                'note', 'attachment'
            )
        })
    )

    inlines = (ContentIssueInline,)

    ordering = ('-pub_date', 'stream__name')

    def get_actions(self, request,*args, **kw):
        actions = super(IssueAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        return actions

    def get_output(self, obj):
        "Pdf per bosh_en"
        if obj.stream.name == u'Bosch_EN':
            #URL = reverse('content_issue_output', args=(obj.id,))
            URL = "/admin/content/issue/{0}/output".format(obj.id)
            return """
                <a href="{0}" target="_blank">
                    <img src="" alt="PDF">
                </a>
            """.format(URL)

        "HTML per autolink"
        if obj.stream.id == 1:
            #URL = reverse('content_issue_output_html', args=(obj.id,))
            URL = "/admin/content/issue/{0}/output_html".format(obj.id)
            return """
                <a href="{0}" target="_blank">
                    <img src="" alt="HTML">
                </a>""".format(URL)

        else:
            return ''
    get_output.short_description = _('Output')
    get_output.allow_tags = True

    def get_attachment_link(self, obj):
        "HTML per autolink"
        if obj.attachment and obj.attachment.url:
            return """
                <a href="{0}" target="_blank">
                    <img src="" alt="PDF">
                </a>""".format(obj.attachment.url)

        else:
            return ''
    get_attachment_link.short_description = _('attachment')
    get_attachment_link.allow_tags = True

    def get_pub_date(self, obj):
        if obj.pub_date:
            to_zone = tz.gettz('Europe/Rome')
            central = obj.pub_date.astimezone(to_zone)
            return central.date()
        return ""
    get_pub_date.short_description = _('Date')
    get_pub_date.admin_order_field = 'pub_date'

    def get_tooltip_note(self, obj):
        img_pth = "%sadmin/images/view.gif" % settings.STATIC_URL
        code = "<img src='%s' title='%s' />" % (img_pth, obj.note) if obj.note else ""
        return code
    get_tooltip_note.allow_tags = True
    get_tooltip_note.short_description = _('Note')

    def get_urls(self):
        urls = super(IssueAdmin, self).get_urls()
        my_urls = patterns('',
            url(
                r'^(?P<issue_id>\d+)/output/$',
                self.admin_site.admin_view(OutputView.as_view()),
                name='content_issue_output'
            ),
            url(
                r'^(?P<issue_id>\d+)/output_html/$',
                self.admin_site.admin_view(OutputViewHtml.as_view()),
                name='content_issue_output_html'
            ),
        )
        return my_urls + urls

    def change_view(self, request, object_id, form_url='', extra_context=None):
        request.session["admin_publieditor_content_issue_id"] = object_id
        return super(IssueAdmin, self).change_view(request, object_id, form_url, extra_context)


class SectorAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'id', 'status', 'name', 'get_parent_names_string', 'context',
    )
    advanced_search_fields = (
        ('id', 'name',),
        ('status__in',),
    )
    search_fields = ('name',)
    list_filter = ('status',)
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'parents', 'name', 'status', 'context', 'key', 'keys',
                'old_name', 'query_keys'
            )
        })
    )
    filter_horizontal = ('parents',)

    def get_actions(self, request,*args, **kw):
        actions = super(SectorAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        return actions


class KindAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(KindAdminForm, self).__init__(*args, **kwargs)
        self.fields['parents'] = forms.ModelMultipleChoiceField(
            models.Kind.objects,
            required=False,
            widget=autocomplete_light.MultipleChoiceWidget('ParentKindAutocomplete')
        )


class KindAdmin(ExtendibleModelAdmin):
    form = KindAdminForm
    list_display = (
        'get_edit_icon', 'id', 'status', 'name', 'get_parent_unicodes_string', 'context',
    )
    advanced_search_fields = (
        ('id', 'name',),
        ('status__in',),
    )
    search_fields = ('name',)
    list_filter = ('status',)
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'parents', 'name', 'status', 'context', 'key',
                'old_name', 'query_keys'
            )
        })
    )
    filter_horizontal = ('parents',)

    def get_actions(self, request, *args, **kw):
        actions = super(KindAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        return actions


class ContentAdminForm(AdminModelForm):

    date_content = forms.DateTimeField(
        widget=DateTimePicker(attrs={'class': 'hasDateTimePicker', 'stepMinute': 1})
    )

    def __init__(self, *args, **kwargs):
        super(ContentAdminForm, self).__init__(*args, **kwargs)
        self.fields['issue'] = forms.ModelChoiceField(
            models.Issue.objects,
            widget=autocomplete_light.ChoiceWidget('IssueAutocomplete'),
            required=False
        )
        self.fields['parent'] = forms.ModelChoiceField(
            models.Content.objects,
            widget=autocomplete_light.ChoiceWidget('ParentAutocomplete'),
            required=False
        )
        self.fields['kind'].queryset = models.Kind.objects.filter(status=1, context='news')
        self.fields["name"].required = False
        today = datetime.datetime.now()
        fixed_today = today.replace(hour=13, minute=00, second=00)
        self.fields["date_content"].initial = fixed_today

        layouts = empty_layouts = Layout.objects.none()
        if self.instance:
            layouts = Layout.objects.filter(name=self.instance.__class__.__name__)
        self.fields["layout"].empty_label = None
        self.fields["layout"].queryset = layouts if layouts.count() == 1 else empty_layouts
        self.fields["layout"].initial = layouts[0].id if layouts.count() == 1 else None

    class Meta:
        #model = models.Car
        model = models.Content


class OnlineOnlyStreamFilter(SimpleListFilter):
    title = _('Online Stream')
    parameter_name = 'stream'

    def lookups(self, request, model_admin):
        return [(stream.id, stream.name) for stream in models.Stream.objects.filter(is_filter=True).distinct()]

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(stream=self.value())


class ContentAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("content/content/css/admin_content.css", "tag/css/tag.css")}
        js = (
            "content/content/js/admin_dajaxice.js",
            "content/content/js/content_inline.js",
            "tag/js/tag.js",
            "content/content/js/admin_clone_content.js"
        )

    advanced_search_fields = (
        #('id', 'name__icontains', 'translations__title__icontains',),
        ('id', 'translations__title__icontains:Titolo',),
        ('layout__in', 'stream', 'sector:Settore'),
        ('kind:Tipo', 'status',),
        ('car_details__corporate:Azienda', 'car_details__brand:Marchio',),
        ('date_content__gte:Data', 'date_content__lte:Data',),
        ('date_content__range:Data',),
        ('translations__description__icontains:Descrizione',),
        ('translations__abstract__icontains:Astratto',),
        ('translations__body__icontains:Corpo',),
        ('note__icontains',),
        ('issue__number', 'code', 'page',),
        ('photo', 'table', 'parent__id__isnull:Senza genitore',),
    )
    #search_fields = ('name',)
    #list_filter = ('status', 'stream',)
    list_filter = ('status', OnlineOnlyStreamFilter,)
    list_display = (
        'get_edit_icon', 'view_on_site', 'get_id', 'get_date_content',
        'status', 'layout', 'get_stream_issue_code_page',
        'get_tooltip_note', 'get_title', 'get_carcontent_list_admin', 
        'get_photo_html_4_3_admin', 'get_table_html'
    )
    list_display_csv = (
        'id', 'parent', 'get_date_content', 'status', 'layout', 'stream',
        'issue', 'code', 'page', 'note',
        'get_current_language', 'get_title_by_language',
        'get_description_by_language', 'get_abstract_by_language',
        'get_body_by_language', 'get_has_photo', 'get_has_table', 'get_has_video',
        'get_first_photo_url', 'get_first_table_url',
    )
    ordering = ("-id",)
    inlines = (
        TranslationContentInline, CarContentInline,
        MediaPhotoInline, MediaTableInline, MediaVideoInline,
        # PhotoContentGalleryInline, VideoContentGalleryInline,
        ContentInline
    )
    save_on_top = False
    readonly_fields = (
        'id', 'parent', 'creator', 'last_modifier', 'date_create', 'date_last_modify',
    )
    form = ContentAdminForm
    # form = autocomplete_light.modelform_factory(models.Content)

    def __init__(self, *args, **kwargs):
        # define fieldsets
        self.properties_fieldsets = (_('properties'), {
            #'fields': ('id', 'parent', 'status', 'layout', 'name', 'ordering')
            'fields': ('id', 'parent', 'status', 'layout',)
        })
        self.classifications_fieldsets = (_('classifications'), {
            'fields': ('source', 'ref', 'place', 'sector', 'kind',)
        })
        self.issue_fieldsets = (_('issue'), {
            'fields': ('stream', 'date_content', 'issue', 'code', 'page')
        })
        #self.media_fieldsets = (_('media'), {'fields': (('photo', 'table'),)})
        self.note_fieldsets = (_('note'), {'fields': (('note', 'annotation',),)})

        # define tabs
        # self.properties_tabs = (_('properties'), {'items': [_('properties')], 'active': True})
        # self.classifications_tabs = (_('classifications'), {'items': [_('classifications'), CarContentAjaxInline]})
        # self.issue_tabs = (_('issue'), {'items': [_('issue')]})
        # self.news_tabs = (_('news'), {'items': [models.TranslationContent]})
        # self.note_tabs = (_('note'), {'items': [_('note')]})
        # self.logs_tabs = (_('logs'), {'items': [_('logs admin')]})

        super(ContentAdmin, self).__init__(*args, **kwargs)

    def get_urls(self):
        urls = super(ContentAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^photos/news/(?P<news_id>[-\w]+)/$',
                self.admin_site.admin_view(get_photos),
                name='content_%s_get_photos' % self.model.__name__.lower()
            ),
            url(
                r'^videos/news/(?P<news_id>[-\w]+)/$',
                self.admin_site.admin_view(get_videos),
                name='content_%s_get_videos' % self.model.__name__.lower()
            ),
            url(
                r'^tables/news/(?P<news_id>[-\w]+)/$',
                self.admin_site.admin_view(get_tables),
                name='content_%s_get_tables' % self.model.__name__.lower()
            ),
        )
        return my_urls + urls

    # def get_tabs(self, request, obj=None):
    #     tabs = (
    #         self.properties_tabs,
    #         self.classifications_tabs,
    #         self.issue_tabs,
    #         self.news_tabs,
    #         self.note_tabs,
    #         self.logs_tabs,
    #     )
    #     return tabs

    def get_fieldsets(self, request, obj=None):
        return (
            default_fieldsets,
            self.properties_fieldsets,
            self.classifications_fieldsets,
            self.issue_fieldsets,
            #self.media_fieldsets,
            self.note_fieldsets,
        )

    # def get_tabs(self, request, obj=None):
    #     return (
    #         (_('main'), {'items': [
    #             _('properties'), _('classifications'),
    #             _('issue'), _('media'), _('note'), _('logs admin'),
    #             TranslationContentInline,
    #             MediaPhotoInline, MediaTableInline, MediaVideoInline,
    #             PhotoContentGalleryInline, VideoContentGalleryInline,
    #             CarContentInline
    #         ], 'active': True}),
    #         (_('children'), {'items': [ContentAjaxInline]}),
    #     )

    action_form = ContentActionForm
    action_form_fields = {
        'clone_news_item': ['layout', 'stream:required'],
        'change_status': ['status:required'],
    }

    def get_actions(self, request,*args, **kw):
        actions = super(ContentAdmin, self).get_actions(request, *args, **kw)
        del actions['set_disactive']
        del actions['set_active']
        del actions['delete_selected']
        if not self.model == models.Content:
            actions['clone_news_item'] = (self.clone_news_item, 'clone_news_item', _('Clone news item'))
        actions['change_status'] = (self.change_status, 'change_status', _('Change status'))
        return actions

    def change_status(self, modeladmin, request, queryset):
        """
        Action to change the status of a content item
        :param modeladmin:
        :param request:
        :param queryset:
        :return:
        """
        status_to_change_to = request.POST.get("status")
        for content in queryset:
            content.status = status_to_change_to
            content.save()
        return redirect(request.META.get('HTTP_REFERER'))
    change_status.short_description = _('Change status')

    def clone_news_item(self, modeladmin, request, queryset):
        """
        Action to clone a news
        :param modeladmin:
        :param request:
        :param queryset:
        :return:
        """

        if queryset.count() > 1:
            modeladmin.message_user(request, _("You must select just one news item!!!"))
            return redirect(reverse("admin:content_content_changelist"))

        first_element = queryset.all()[0] # solo una notizia per volta!
        layout_id = request.POST.get("layout", "")
        layout = Layout.objects.get(id=layout_id) if layout_id.strip() != "" else first_element.layout
        stream_id = request.POST.get('stream')
        cloned_item = self._clone_item(first_element, stream_id, layout)
        cloned_item.stream_id = stream_id
        cloned_item.layout = layout
        cloned_item.parent = first_element
        cloned_item.status = 'dr'
        cloned_item.save()
        return redirect(reverse("admin:content_%s_change" % layout.name.lower(), args=(cloned_item.id,)))

    clone_news_item.short_description = _('Clone news item')

    def _clone_item(self, orig, stream_id, layout):
        from jmb.core.utils import models_utils
        from django.db.models.loading import get_model

        dest_model = get_model("content", layout.get_table_name())
        kw = {
            'kwargs_excludes': [
                'date_create', 'date_last_modify', 'status', 'ordering', 
                'creator', 'last_modifier', 'parent', 'stream', 'parent', 'content_ptr'
            ]
        }
        done, msg, new_item = models_utils.ModelObjectCopier(
            orig, related_parent_name='content',
            related_managers_excludes=['children', 'preferences', 'contentdetailsettings_set'],
            dest_model=dest_model,
            **kw
        ).bulk_copy()
        return new_item

    def response_add(self, request, obj, **kwargs):
        response = super(ContentAdmin, self).response_add(request, obj, **kwargs)
        if '_save' in request.POST:
            if request.GET.get('_return_to', None):
                response['location'] = request.GET['_return_to']
            else:
                response['location'] = reverse("admin:content_content_changelist")
        return response

    def changelist_view(self, request, extra_context=None):
        layouts = models.Layout.objects.filter(status=1)
        extra_context = {}
        add_urls = []
        for layout in layouts:
            layout_url = {}
            try:
                layout_url['name'] = "Add %s" % layout.name
                reverse_url = reverse("admin:content_%s_add" % layout.name.lower().replace(" ", ""))
                layout_url['url'] = "%s?_return_to=%s" % (
                    reverse_url,
                    urlquote(request.get_full_path())
                )
                add_urls.append(layout_url)
            except:
                continue
        extra_context['add_urls'] = add_urls
        return super(ContentAdmin, self).changelist_view(request, extra_context=extra_context)

    def get_edit_icon(self, obj):
        ## per condividerla con TabularInline
        return get_edit_icon(self, obj)
    get_edit_icon.short_description = _("E")
    get_edit_icon.allow_tags = True

    def get_id(self, obj):
        if obj.parent:
            return "<strong>%s</strong><br><span class='parent_id'>%s</span>" % (obj.id, obj.parent.id)
        return "<strong>%s</strong>" % obj.id
    get_id.short_description = _('ID')
    get_id.admin_order_field = 'id'
    get_id.allow_tags = True

    def get_date_content(self, obj):
        if obj.date_content:
            to_zone = tz.gettz('Europe/Rome')
            central = obj.date_content.astimezone(to_zone)
            return central.date().strftime('%d/%m/%Y')
        return ""
    get_date_content.short_description = _('Date')
    get_date_content.admin_order_field = 'date_content'

    def get_current_language(self, obj):
        return translation.get_language().lower()
    get_current_language.short_description = _('Language')

    def get_translation_by_language(self, obj):
        cur_language = self.get_current_language(obj)
        fallback_language = 'en' if cur_language == 'it' else 'it'
        #we must use fallback, or it's gonna export only news in current admin language
        try:
            cur_translation = obj.translations.get(language__code__in=[cur_language, cur_language.upper()])
        except Exception:
            try:
                cur_translation = obj.translations.get(language__code__in=[fallback_language, fallback_language.upper()])
            except Exception:
                return None
        return cur_translation
    get_translation_by_language.short_description = _('Traduzione')

    def get_title_by_language(self, obj):
        if self.get_translation_by_language(obj):
            return self.get_translation_by_language(obj).title
        return ""
    get_title_by_language.short_description = _('Title')

    def get_description_by_language(self, obj):
        if self.get_translation_by_language(obj):
            return self.get_translation_by_language(obj).description
        return ""
    get_description_by_language.short_description = _('Description')

    def get_abstract_by_language(self, obj):
        if self.get_translation_by_language(obj):
            return self.get_translation_by_language(obj).abstract
        return ""
    get_abstract_by_language.short_description = _('Abstract')

    def get_body_by_language(self, obj):
        if self.get_translation_by_language(obj):
            return fix_text(strip_tags(self.get_translation_by_language(obj).body))
        return ""
    get_body_by_language.short_description = _('Body')

    def get_has_photo(self, obj):
        return _("Yes") if obj.has_photo() else _("No")
    get_has_photo.short_description = _('Has photo')

    def get_has_table(self, obj):
        return _("Yes") if obj.has_table() else _("No")
    get_has_table.short_description = _('Has table')

    def get_has_video(self, obj):
        return _("Yes") if obj.has_video() else _("No")
    get_has_video.short_description = _('Has video')

    def get_first_photo_url(self, obj):
        if obj.has_photo():
            return obj.get_first_photo_url_dict()["orig"]
        return None
    get_first_photo_url.short_description = _('First photo url')

    def get_first_table_url(self, obj):
        if obj.has_table():
            return obj.get_first_table_url_dict()["orig"]
        return None
    get_first_table_url.short_description = _('First table url')

    def get_stream_issue_code_page(self, obj):
        number = ""
        if obj.issue:
            number = "<a href='%s'>%s</a>" % (
                reverse('admin:content_issue_change', args=(obj.issue.id,),), obj.issue.number,
            )
        return "%s<br/>%s <span class='code'>%s</span> <span class='page'>%s</span>" % (
            obj.stream or '', number, obj.code or '', obj.page or ''
        )
    get_stream_issue_code_page.allow_tags = True
    get_stream_issue_code_page.short_description = _('Stream/Issue/Code/Page')

    def get_tooltip_note(self, obj):
        img_pth = "%sadmin/images/view.gif" % settings.STATIC_URL
        code = "<img src='%s' title='%s' />" % (img_pth, obj.note) if obj.note else ""
        return code
    get_tooltip_note.allow_tags = True
    get_tooltip_note.short_description = _('Note')

    def get_title(self, obj):
        if obj.translations.all():
            translation = obj.translations.all()[0]
            title = translation.title
            sector = obj.sector
            return "<b>%s</b><br/>%s <span class='issue'>%s</span>" % (title, sector, obj.kind)
        return obj.name
    get_title.allow_tags = True
    get_title.short_description = _('Title')
    get_title.admin_order_field = 'title'

    def view_on_site(self, obj):
        translations = obj.translations.all()
        if translations:
            translation = translations[0]
            reverse_url = reverse('content_translation_detail', args=(translation.slug,))
            return """
                <a href="%(reverse_url)s" target="_blank">
                    <img src='%(url)sjmb/images/search.png' alt='%(title)s' title='%(title)s'/>
                </a>
            """ % {
                'reverse_url': reverse_url,
                'url': settings.STATIC_URL,
                'title': translation.title
            }
        return ""
    view_on_site.allow_tags = True
    view_on_site.short_description = _('Detail')

    def get_carcontent_list_admin(self, obj):
        html_code = "<ul>"
        for detail in obj.get_carcontent_list():
            html_code += "<li>%s</li>" % detail
        html_code += "</ul>"
        return html_code
    get_carcontent_list_admin.allow_tags = True
    get_carcontent_list_admin.short_description = _('Car Detail')

    def get_carcontent_list_admin(self, obj):
        html_code = ""
        for detail in obj.get_carcontent_list():
            html_code += "- %s" % detail
        html_code += "<br/>"
        return html_code
    get_carcontent_list_admin.allow_tags = True
    get_carcontent_list_admin.short_description = _('Car Detail')

    def get_photo_html_4_3_admin(self, obj):
        html_code = ""
        if obj.get_first_photo():
            html_code = "<img src='%s' alt='%s' width='70px' style='float:left!important;'/>&nbsp;photos: <b>%s</b>" %(
                obj.get_first_photo_url_dict()['thumb43'],
                obj.name,
                obj.media_photos.count()
            )
        return html_code
    get_photo_html_4_3_admin.allow_tags = True
    get_photo_html_4_3_admin.short_description = _('Photo')


class TestDriveAdmin(ContentAdmin):
    pass


class OpenHouseAdminForm(ContentAdminForm):
    def __init__(self, *args, **kwargs):
        super(OpenHouseAdminForm, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field.endswith('point'):
                self.fields[field] = forms.ChoiceField(
                    choices=models.get_openhouse_point_choices(),
                    widget=forms.RadioSelect,
                    required=False
                )
                self.fields[field].empty_label = None

    class Meta:
        model = models.OpenHouse


class OpenHouseAdmin(ContentAdmin):
    class Media:
        js = ("content/content/js/avg_calc.js",)
        
    form = OpenHouseAdminForm

    # def get_tabs(self, request, obj=None):
    #     voting_tabs = (_('voting'), {'items': [_('voting')]})
    #     tabs = (
    #         self.properties_tabs,
    #         self.classifications_tabs,
    #         self.issue_tabs,
    #         self.news_tabs,
    #         voting_tabs,
    #         self.note_tabs,
    #         self.logs_tabs,
    #     )
    #     return tabs

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = list(super(OpenHouseAdmin, self).get_readonly_fields(request, obj))
        readonly_fields.append('get_point_avg')
        return readonly_fields

    def get_fieldsets(self, request, obj=None):
        voting_fieldsets = (_('voting'), {'fields': (
            'get_point_avg',
            'overall_point', 'overall_description', 'waiting_point', 'waiting_description',
            'availability_point', 'availability_description', 'roadtest_point', 'roadtest_description',
            'welcome_point', 'welcome_description', 'quotation_point', 'quotation_description',
            'financial_point', 'financial_description', 'brochure_point', 'brochure_description',
            'offers_point', 'offers_description', 'showroom_point', 'showroom_description',
            'buyback_point', 'buyback_description',
        )})

        return (
            default_fieldsets,
            self.properties_fieldsets,
            self.classifications_fieldsets,
            self.issue_fieldsets,
            voting_fieldsets,
            self.note_fieldsets,
        )


class CrashTestAdmin(ContentAdmin):
    pass


class BrochureAdmin(ContentAdmin):
    pass


class CatalogAdmin(ContentAdmin):
    pass


class VideoAdmin(ContentAdmin):
    pass


class RoadTestAdminForm(ContentAdminForm):
    def __init__(self, *args, **kwargs):
        super(RoadTestAdminForm, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field.endswith('point'):
                self.fields[field] = forms.ChoiceField(
                    choices=models.get_roadtest_point_choices(),
                    widget=forms.RadioSelect,
                    required=False
                )
                self.fields[field].empty_label = None

    class Meta:
        model = models.OpenHouse


class RoadTestAdmin(ContentAdmin):
    class Media:
        js = ("content/content/js/avg_calc.js",)
        
    form = RoadTestAdminForm

    # def get_tabs(self, request, obj=None):
    #     voting_tabs = (_('voting'), {'items': [_('voting')]})
    #     tabs = (
    #         self.properties_tabs,
    #         self.classifications_tabs,
    #         self.issue_tabs,
    #         self.news_tabs,
    #         voting_tabs,
    #         self.note_tabs,
    #         self.logs_tabs,
    #     )
    #     return tabs

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = list(super(RoadTestAdmin, self).get_readonly_fields(request, obj))
        readonly_fields.append('get_point_avg')
        return readonly_fields

    def get_fieldsets(self, request, obj=None):
        voting_fieldsets = (_('voting'), {'fields': (
            ('global_point', 'get_point_avg',),
            'pros', 'cons',
            'howitis_point', 'howitis_description',
            'howitgoes_point', 'howitgoes_description',
            'performance_point', 'performance_description',
            'design_point', 'design_description',
            'comfort_point', 'comfort_description',
            'safety_point', 'safety_description',
            'value_point', 'value_description',
            'toknow', 'url',
        )})

        return (
            default_fieldsets,
            self.properties_fieldsets,
            self.classifications_fieldsets,
            self.issue_fieldsets,
            voting_fieldsets,
            self.note_fieldsets,
        )


class PhotoGalleryAdmin(ContentAdmin):
    pass


class GalleryAdmin(ContentAdmin):
    pass


class ApprofondimentoAdmin(ContentAdmin):
    pass


class BreveAdmin(ContentAdmin):
    pass


class NewsAdmin(ContentAdmin):
    pass


class SalePeriodAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'id', 'status',
        'name', 'date_begin', 'date_end',
    )
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'name',
                ('date_begin', 'date_end',),
            ),
        })
    )
    advanced_search_fields = (
        ('id',),
        ('name__icontains:period',),
        ('date_begin__lte', 'date_begin__gte'),
        ('date_end__lte', 'date_end__gte'),
    )
    ordering = ('-date_begin',)


class CarModelSaleForm(AdminModelForm):
    def __init__(self, *args, **kwargs):
        super(CarModelSaleForm, self).__init__(*args, **kwargs)
        self.fields['carmodels'] = forms.ModelChoiceField(
            models.CarModel.objects.all(),
            required=False,
            widget=autocomplete_light.ChoiceWidget('CarModelAutocompleteDynamic')
        )

    class Meta:
        model = models.CarModelSale


class CarModelSaleAdmin(ExtendibleModelAdmin):
    form = CarModelSaleForm
    list_display = (
        'get_edit_icon', 'id', 'get_brand', 'get_string_models',
        'period', 'sold',
    )
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'period',
                'carmodels',
                'sold',
            ),
        })
    )
    advanced_search_fields = (
        ('period',),
        ('carmodels__id:carmodel id',),
        ('carmodels__name__icontains:models',),
        ('carmodels__brand:brand',),
    )
    ordering = ('-period__date_begin',)

    def queryset(self, request):
        qs = super(CarModelSaleAdmin, self).queryset(request)
        return qs.distinct()


class CarModelEquipmentAdmin(ExtendibleModelAdmin):
    class Media:
        js = ("content/car/js/admin.js",)
        
    form = CarModelEquipmentAdminForm
    list_display = (
        'get_edit_icon', 'id', 'get_brand', 'get_model',
        'description', 'standard', 'ordering'
    )
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                ('carmodel', 'standard',),
                'description',
                'note',
            ),
        })
    )
    
    def get_readonly_fields(self, request, obj = None):
        if obj:
            return ('carmodel', 'standard',) + self.readonly_fields
        return  self.readonly_fields
    ordering = ('carmodel__brand__name', 'carmodel__name', '-standard', 'description')
    advanced_search_fields = (
        ('id', 'standard',),
        ('carmodel__brand__name__icontains:brand', 'carmodel__name__icontains:model',),
        ('description__icontains',),
    )

    def get_model(self, obj):
        return obj.carmodel.name
    get_model.short_description = _('model')
    get_model.admin_order_field = 'carmodel__name'

    def get_brand(self, obj):
        return obj.carmodel.brand.name
    get_brand.short_description = _('brand')
    get_brand.admin_order_field = 'carmodel__brand__name'
    
    def change_view(self, request, object_id, form_url='', extra_context=None):
        if request.method == 'GET' and request.is_ajax():
            qs = super(CarModelEquipmentAdmin, self).queryset(request).none()
            if object_id and ("{}".format(object_id)).isdigit():
                qs = super(CarModelEquipmentAdmin, self).queryset(request).filter(id=object_id)
            data = serializers.serialize("json", qs.all())
            return HttpResponse(data, content_type="application/json")
        return super(CarModelEquipmentAdmin, self).change_view(
            request, object_id, form_url, extra_context)


class CarEquipmentAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'id',
        'get_model', 'get_car', 'get_equipment',
        'ordering'
    )
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'get_model',
                'get_car',
                'get_equipment',
            ),
        })
    )
    readonly_fields = ('get_car', 'carmodel_equipment', 'get_model', 'get_equipment')
    ordering = ('car__model__name', 'car__version', 'carmodel_equipment__description')
    advanced_search_fields = (
        ('id', 'carmodel_equipment__standard',),
        ('car__model__brand__name__icontains:brand',),
        ('car__model__name__icontains:model',),
        ('car__version__icontains:car',),
        ('carmodel_equipment__description__icontains:description',),
    )

    def get_car(self, obj):
        return obj.car.version
    get_car.short_description = _('car')
    get_car.admin_order_field = 'car__version'

    def get_model(self, obj):
        return obj.car.model.name
    get_model.short_description = _('model')
    get_model.admin_order_field = 'car__model__name'

    def get_equipment(self, obj):
        return obj.carmodel_equipment.description
    get_equipment.short_description = _('equipment')
    get_equipment.admin_order_field = 'carmodel_equipment__description'
        

class TooltipAdmin(ExtendibleModelAdmin):
    list_display = (
        'status', 'name', 'language', 'slug', 'description',
    )
    
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': ('status', 'name', 'language', 'slug', 'description',)
        }),
    )


class UNRAECarModelAdminForm(AdminModelForm):
    def __init__(self, *args, **kwargs):
        super(UNRAECarModelAdminForm, self).__init__(*args, **kwargs)
        self.fields['carmodels'] = forms.ModelMultipleChoiceField(
            models.CarModel.objects.all(),
            required=False,
            widget=autocomplete_light.MultipleChoiceWidget('CarModelAutocompleteBase')
        )

    class Meta:
        model = models.UNRAECarModel


class UNRAECarModelAdmin(ExtendibleModelAdmin):
    form = UNRAECarModelAdminForm
    list_display = (
        'get_edit_icon', 'brand', 'carmodel', 'design', 'get_string_models', 'get_is_associated',
    )
    advanced_search_fields = (
        ('brand__icontains: UNRAE brand',),
        ('carmodel__icontains: UNRAE model',),
        ('design__icontains: UNRAE design',),
        ('carmodels__in: PUB models',),
    )
    ordering = ('brand', 'carmodel', 'design')
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': ('brand', 'carmodel', 'design', 'carmodels',)
        }),
    )

    def get_urls(self):
        urls = super(UNRAECarModelAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^import_data/$',
                self.admin_site.admin_view(import_unrae),
                name='content_unraecarmodel_import_data'
            ),
            url(
                r'^unrae_model_sync/$',
                self.admin_site.admin_view(model_sync),
                name='content_unraecarmodel_model_sync'
            ),
            url(
                r'^upload_model_sent_hits/$',
                self.admin_site.admin_view(update_sales_data),
                name='content_unraecarmodel_upload_model_sent_hits'
            ),
        )
        return my_urls + urls

    def queryset(self, request):
        from django.db.models import Count
        return super(UNRAECarModelAdmin, self).queryset(request).annotate(carmodels_count=Count('carmodels'))

    def get_is_associated(self, inst):
        if inst.carmodels_count:
            return True
        return False
    get_is_associated.boolean = True
    get_is_associated.admin_order_field = 'carmodels_count'
    get_is_associated.short_description = _('Is associated')


class UNRAECarModelSaleAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'get_brand', 'get_carmodel', 'get_design', 'get_string_models',
        'month', 'year', 'sold'
    )
    ordering = (
        '-year', '-month', 'unrae_carmodel__brand',
        'unrae_carmodel__carmodel', 'unrae_carmodel__design'
    )
    advanced_search_fields = (
        ('unrae_carmodel__brand__icontains: UNRAE brand',),
        ('unrae_carmodel__carmodel__icontains: UNNRAE model',),
        ('unrae_carmodel__design__icontains: UNRAE design',),
        ('unrae_carmodel__carmodels__in: PUB models',),
        ('year__gte', 'year__lte'),
        ('month__gte', 'month__lte'),
        ('sold__gte', 'sold__lte')
    )
    fieldsets = (
        default_fieldsets,
        (_('general information'), {
            'fields': (
                'get_brand', 'get_carmodel', 'get_design', 'get_string_models',
                ('month', 'year',), 'sold'
            )
        }),
    )
    readonly_fields = (
        'get_brand', 'get_carmodel', 'get_design', 'get_string_models',
    )

    def get_brand(self, obj):
        return obj.unrae_carmodel.brand
    get_brand.short_description = _('brand')
    get_brand.admin_order_field = ('unrae_carmodel__brand',)

    def get_carmodel(self, obj):
        return obj.unrae_carmodel.carmodel
    get_carmodel.short_description = _('model')
    get_carmodel.admin_order_field = ('unrae_carmodel__carmodel',)

    def get_design(self, obj):
        return obj.unrae_carmodel.design
    get_design.short_description = _('design')
    get_design.admin_order_field = ('unrae_carmodel__design',)

    def get_string_models(self, obj):
        return obj.unrae_carmodel.get_string_models()
    get_string_models.short_description = _('models')


def get_edit_icon(self, obj):
    opts = self.model._meta
    can_be_modified = False
    # Se il modello ha la funzione can_be_modified la chiamo per fare
    # i controlli relativi a quel modello
    if hasattr(obj, "can_be_modified"):
        if obj.can_be_modified():
            can_be_modified = True
    # Se non ce l'ha non faccio controlli e visualizzo l'icona
    else:
        can_be_modified = True

    request = get_request()
    if can_be_modified:
        app_label = opts.app_label
        layout_name = "content"
        if obj.layout:
            layout_name = obj.layout.name.lower().replace(" ", "")
        dj_reverse_url = reverse('admin:%s_%s_change' % (
            app_label, layout_name
        ), args=(obj.id,))
        return """
            <a href=%(reverse_url)s>
            <img src='%(url)sjmb/images/edit.gif' alt='%(window_title)s' title='%(window_title)s'/>
            </a>
        """ % {
            'reverse_url': "%s?%s=%s" % (dj_reverse_url, '_return_to', urlquote(request.get_full_path())),
            'url': settings.STATIC_URL,
            'window_title': ugt("Edit %s" % opts.object_name.lower())
        }
    return ""

register_inline(CarContentAjaxInline)
register_inline(PhotoContentGalleryAjaxInline)
register_inline(VideoContentGalleryAjaxInline)

admin.site.register(models.Corporate, CorporateAdmin)
admin.site.register(models.Brand, BrandAdmin)
admin.site.register(models.CarModel, CarModelAdmin)
admin.site.register(models.Car, CarAdmin)
admin.site.register(models.CarContent, CarContentAdmin)
admin.site.register(models.Stream, StreamAdmin)
admin.site.register(models.Layout, LayoutAdmin)
admin.site.register(models.Issue, IssueAdmin)
admin.site.register(models.Sector, SectorAdmin)
admin.site.register(models.Kind, KindAdmin)
admin.site.register(models.Content, ContentAdmin)
admin.site.register(models.TestDrive, TestDriveAdmin)
admin.site.register(models.OpenHouse, OpenHouseAdmin)
admin.site.register(models.CrashTest, CrashTestAdmin)
admin.site.register(models.Brochure, BrochureAdmin)
admin.site.register(models.Catalog, CatalogAdmin)
admin.site.register(models.Video, VideoAdmin)
admin.site.register(models.RoadTest, RoadTestAdmin)
admin.site.register(models.PhotoGallery, PhotoGalleryAdmin)
admin.site.register(models.Gallery, GalleryAdmin)
admin.site.register(models.Approfondimento, ApprofondimentoAdmin)
admin.site.register(models.Breve, BreveAdmin)
admin.site.register(models.News, NewsAdmin)
admin.site.register(models.PhotoContentGallery, PhotoContentGalleryAdmin)
admin.site.register(models.VideoContentGallery, VideoContentGalleryAdmin)
admin.site.register(models.TranslationContent)
admin.site.register(models.CorporateBrandLink, CorporateBrandLinkAdmin)
admin.site.register(models.Hit, HitAdmin)
admin.site.register(models.SalePeriod, SalePeriodAdmin)
admin.site.register(models.CarModelSale, CarModelSaleAdmin)
admin.site.register(models.CarModelEquipment, CarModelEquipmentAdmin)
admin.site.register(models.CarEquipment, CarEquipmentAdmin)
admin.site.register(models.Tooltip, TooltipAdmin)
admin.site.register(models.UNRAECarModel, UNRAECarModelAdmin)
admin.site.register(models.UNRAECarModelSale, UNRAECarModelSaleAdmin)

