# -*- coding: utf-8 -*-
import autocomplete_light
import collections
import copy
import datetime
from django.conf import settings
from django import forms
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext as _

from jmb.core.admin import ConstrainedModelForm
from jmb.core.admin.options import JumboActionForm
from publieditor.content.models import (
    CarModel, CarModelEquipment, CarEquipment, Stream, Layout,
)
from publieditor.content.validators import validate_excel_extensions

autocomplete_light.autodiscover()

class InsertEquipmentForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super(InsertEquipmentForm, self).__init__(*args)
        self.fields['carmodel'].initial = kwargs.get('carmodel_id', False)
        self.total_insert = 0
        self.total_import = 0

    carmodel = forms.ModelChoiceField(
        queryset=CarModel.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('CarModelAutocompleteBase')
    )

    dotazioni = forms.CharField(widget=forms.Textarea, required=True)

    def save(self):
        dotazioni = self.cleaned_data['dotazioni']
        carmodel = self.cleaned_data['carmodel']
        dotazioni = dotazioni.strip()
        righe = dotazioni.splitlines()
        self.total_import = len(righe)
        for riga in righe:
            riga = riga.strip()
            try:
                CarModelEquipment.objects.get(description__iexact=riga, carmodel=carmodel)
            except ObjectDoesNotExist:
                new = CarModelEquipment(description=riga, carmodel=carmodel)
                new.save()
                new.set_standard()
                self.total_insert += 1


class SetEquipmentForm(forms.Form):
    def __init__(self, *args, **kwargs):
        self.carmodel_id = kwargs.get('carmodel_id')
        super(SetEquipmentForm, self).__init__(*args)
        self.carmodel = CarModel.objects.get(id=self.carmodel_id)
        self.equipments = self.carmodel.equipments.all().order_by('id')
        self.cars = self.carmodel.cars.filter(status__in=('ok', 'working',)).order_by('motor', 'version')
        self.add_first_motor()
        self.matrix = self.empty_matrix()
        self.motors = self.create_motors()

    def add_first_motor(self):
        # adds property first_motor to object Car
        motor = None
        for car in self.cars:
            car.first_motor = False
            if car.motor != motor:
                car.first_motor = True
                motor = car.motor

    def load(self):
        self.matrix = self.set_matrix_from_db()

    def save(self, data):
        db_matrix = self.set_matrix_from_db()
        form_matrix = self.set_matrix_from_form(data)

        equipments = self.equipments.all()
        cars = self.cars.all()

        for equipment in equipments:
            for car in cars:
                if (
                    db_matrix[equipment.id]['cars'][car.id]['value'] !=
                    form_matrix[equipment.id]['cars'][car.id]['value']
                ):
                    # se esisteva nel DB allora cancella
                    if db_matrix[equipment.id]['cars'][car.id]['value'] is True:
                        CarEquipment.objects.get(
                            car__id=car.id,
                            carmodel_equipment__id=equipment.id,
                        ).delete()
                    # se non esisteva nel DB allora scrive
                    else:
                        CarEquipment(
                            car_id=car.id,
                            carmodel_equipment_id=equipment.id,
                        ).save()
        self.matrix = form_matrix

    def empty_matrix(self):
        """
        Returns a structured dict with value = False:
        [equipment.id]
        [equipment.id]['equipment'] = equipment object
        [equipment.id]['cars']
        [equipment.id]['cars'][car.id]
        [equipment.id]['cars'][car.id]['car'] = car object
        [equipment.id]['cars'][car.id]['value'] = True or False if car has equipment

        esempio
        -------
        1112703: {
            'equipment': <CarModelEquipment: Abarth 500C: Fari xeno>,
            'cars': {
                3: {
                    'car': <Car: 2010 Abarth - Abarth 500C (online) - 1.4 B 135CV  (500 1.4 Turbo T-Jet 135 CV) ok>,
                    'value': False,
                    'first': True
                },
                7948: {
                    'car': <Car: 2010 Abarth - Abarth 500C (online) - 595 B 160CV  (595 Turismo 1.4 Turbo T-Jet 160 CV) ok>,
                    'value': False,
                    'first': False
                },
                7949: {
                    'car': <Car: 2010 Abarth - Abarth 500C (online) - 595 B 160CV  (595 Competizione 1.4 Turbo T-Jet 160 CV) ok>,
                    'value': False,
                    'first': False
                }
            }
        }

        self.matrix[1112703]
        self.matrix[1112703]['equipment']
        self.matrix[1112703]['cars']
        self.matrix[1112703]['cars'][3]
        self.matrix[1112703]['cars'][3]['car']
        self.matrix[1112703]['cars'][3]['value']
        """
        matrix = collections.OrderedDict()
        cars = collections.OrderedDict()
        motor_car = None
        for car in self.cars:
            cars[car.id] = collections.OrderedDict(value=False, car=car, first=False)
            if motor_car != car.motor:
                motor_car = car.motor
                cars[car.id]['first'] = True

        for equipment in self.equipments:
            new_cars = copy.deepcopy(cars)
            matrix[equipment.id] = collections.OrderedDict(
                equipment=equipment,
                cars=new_cars,
            )
        return matrix

    def set_matrix_from_db(self):
        """
        Set matrix elements to True taking values from db
        DB returns only standard equipments
        """
        car_equipments = CarEquipment.objects.filter(
            carmodel_equipment__carmodel=self.carmodel,
            car__status__in=('ok', 'working',)
        ).select_related()
        matrix = self.empty_matrix()
        for car_equipment in car_equipments:
            equipment_id = car_equipment.carmodel_equipment.id
            car_id = car_equipment.car.id
            matrix[equipment_id]['cars'][car_id]['value'] = True
        return matrix

    def set_matrix_from_form(self, data):
        """
        Set matrix elements to True taking values from form
        Form returns only checked elements
        """



        matrix = self.empty_matrix()
        for key, value in data.items():
            if key.startswith("carequipment_"):
                car_id, equipment_id = key.split('_')[1:]
                car_id = int(car_id)
                equipment_id = int(equipment_id)
                matrix[equipment_id]['cars'][car_id]['value'] = True
        return matrix

    def create_motors(self):
        """
        Set dictionary of motors and memories number of cars for each single motor
        """
        motors = collections.OrderedDict()
        for car in self.cars:
            if car.motor in motors:
                motors[car.motor] += 1
            else:
                motors[car.motor] = 1
        return motors


class ImportUNRAEForm(forms.Form):
    required_css_class = 'required'

    def __init__(self, *args, **kws):
        curr_year = datetime.date.today().year
        super(ImportUNRAEForm, self).__init__(*args, **kws)
        self.fields['import_file'] = forms.FileField(
            label=_('File Data to upload'),
        )
        self.fields['year'] = forms.IntegerField(
            label=_('year'),
            initial=curr_year
        )

    def clean_import_file(self):
        if not self.cleaned_data['import_file']:
            raise forms.ValidationError(_('Upload excel file'))
        return self.cleaned_data['import_file']


class CarModelEquipmentAdminForm(ConstrainedModelForm):
    def __init__(self, *args, **kwargs):
        super(CarModelEquipmentAdminForm, self).__init__(*args, **kwargs)
        if self.fields.get('carmodel', False):
            self.fields['carmodel'] = forms.ModelChoiceField(
                queryset=CarModel.objects.all(), required=True,
                widget=autocomplete_light.ChoiceWidget('CarModelAutocompleteBase')
            )
            self.fields['standard'] = forms.BooleanField(
                widget=forms.HiddenInput(),
                required=False
            )


class ContentActionForm(JumboActionForm):
    stream = forms.ModelChoiceField(
        queryset=Stream.objects.filter(status='online'),
        label=_('Stream'),
        #required=True,
        required=False,
        # This is automatically added if not specified. Used from JS to toggle extra action fields
        widget=forms.Select(attrs={'id': 'stream', 'class': 'extra_action_field'})
    )
    layout = forms.ModelChoiceField(
        queryset=Layout.objects.filter(
            status=1,
            name__in=settings.PUBLIEDITOR_CONTENT_CLONABLE_CONTENT_MODELS
        ),
        label=_('Layout'),
        required=False,
        # This is automatically added if not specified. Used from JS to toggle extra action fields
        widget=forms.Select(attrs={'id': 'layout', 'class': 'extra_action_field'})
    )
    status = forms.ChoiceField(
        choices=settings.PUBLIEDITOR_CONTENT_STATO_CHOICES,
        label=_('Status'),
        required=False,
    )


class GetPhotosForm(forms.Form):
    pass


class GetVideosForm(forms.Form):
    pass


class GetTablesForm(forms.Form):
    pass


class CarModelUpdateStatusForm(JumboActionForm):
    status = forms.ChoiceField(
        choices=settings.PUBLIEDITOR_CARMODEL_STATUS_CHOICES,
        label=_('Status'),
        required=True,
    )


class CarImportDataForm(forms.Form):
    file_to_import = forms.FileField(
        required=True,
        label=_('Select a file'),
        validators=[validate_excel_extensions],
    )
