# -*- coding: utf-8 -*-
import datetime
from django.conf import settings
from django.core.paginator import Paginator
from django.db.models import Q
from django.utils.translation import ugettext as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from jmb.core.utils.functions import import_class

from publieditor.content.models import TranslationContent, Brand, Sector, Kind, UserContentSettings
from publieditor.content.cms.models import (
    ContentSettings, ContentSearchSettings, ContentDetailSettings,
    TodaySettings, ContentAdvancedSearchSettings
)
from publieditor.content.utils import (
    get_first_day_of_week, get_first_day_of_month, get_first_day_of_year,
    get_querystring_dict,
)


class CMSPluginExtended(CMSPluginBase):

    def get_dict_from_querystring(self, request):
        dictionary = {}
        if request.GET:
            for key in request.GET.keys():
                if request.GET.get(key) != "":
                    dictionary[key] = request.GET.get(key)
        return dictionary

    def get_querystring_from_dictionary(self, dictionary):
        query_list = []
        for key, value in dictionary.items():
            if key != 'page' and key != 'obj_page' and value != '':
                query_list.append(key+"="+value)
        return "&".join(query_list)

    def filter_queryset(self, queryset, filter_dict):
        q_objects = Q()
        for key, value in filter_dict.items():
            q_objects |= Q(**{key: value})
        try:
            queryset = queryset.filter(q_objects)
        except Exception, e:
            print e
        return queryset


class TodayPlugin(CMSPluginBase):
    model = TodaySettings
    name = _("Today")
    module = _("Content")
    admin_preview = False

    def render(self, context, instance, placeholder):
        self.render_template = "content/cms_plugins/empty.html"
        if instance.status == 0:
            return context

        today = datetime.datetime.today()
        self.render_template = instance.template_name
        context.update({
            'instance': instance,
            'today': today,
            'placeholder': placeholder})
        return context

plugin_pool.register_plugin(TodayPlugin)


class ContentDetailPlugin(CMSPluginBase):
    model = ContentDetailSettings
    name = _("Content Detail")
    module = _("Content")
    admin_preview = False

    def render(self, context, instance, placeholder):
        self.render_template = "content/cms_plugins/empty.html"
        if instance.status == 0:
            return context

        translations = instance.content.translations.all()
        if not translations:
            return context

        content = translations[0]
        self.render_template = instance.template_name

        context.update({
            'object': content,
            'instance': instance,
            'placeholder': placeholder})
        return context

plugin_pool.register_plugin(ContentDetailPlugin)


class ContentPlugin(CMSPluginExtended):
    model = ContentSettings
    name = _("Content List")
    module = _("Content")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        querystring_dict = get_querystring_dict(request)
        query_string = None
        today = datetime.datetime.today()
        included_objects = Q()
        excluded_objects = Q()

        self.render_template = "content/cms_plugins/empty.html"
        if instance.status == 0:
            return context
        if instance.show_if_authenticated == "authenticated" and not request.user.is_authenticated():
            return context
        elif instance.show_if_authenticated == "not_authenticated" and request.user.is_authenticated():
            return context

        self.render_template = instance.template_name
        ordering = instance.ordering.lower().replace(" ", "").split(",")

        page = 1
        if 'paginator_instance_slug' in request.GET and request.GET['paginator_instance_slug'] == instance.slug:
            page = int(request.GET['page'])

        #objects = TranslationContent.objects.filter(content__date_content__lt=today).filter(content__status='on')
        objects = TranslationContent.objects

        included_objects &= Q(**{'content__date_content__lt': today})
        included_objects &= Q(**{'content__status': 'on'})

        # provvisorio, da rimuovere quando gestiremo le lingue
        #objects = objects.filter(language__code='IT').select_related('language')
        included_objects &= Q(**{'language__code': 'IT'})

        ############################################################################################
        # Filtri elementi impostati su istanza
        ############################################################################################

        # Stream
        if instance.stream:
            # objects = objects.filter(content__stream=instance.stream).select_related('content')
            included_objects &= Q(**{'content__stream': instance.stream})

        # # category_slugs_included
        # if instance.category_slugs_included:
        #     category_slugs = instance.category_slugs_included.lower().replace(" ", "").split(",")
        #     objects = objects.filter(content__categories__slug__in=category_slugs)
        #
        # # category_slugs_excluded
        # if instance.category_slugs_excluded:
        #     category_slugs = instance.category_slugs_excluded.lower().replace(" ", "").split(",")
        #     objects = objects.exclude(content__categories__slug__in=category_slugs)

        # content_slugs_included
        if instance.content_slugs_included:
            slugs = instance.slugs_included.lower().replace(" ", "").split(",")
            # objects = objects.filter(slug__in=slugs)
            included_objects &= Q(**{'slug__in': slugs})

        # content_slugs_excluded
        if instance.content_slugs_excluded:
            slugs = instance.slugs_excluded.lower().replace(" ", "").split(",")
            # objects = objects.exclude(slug__in=slugs)
            excluded_objects &= Q(**{'slug__in': slugs})

        # content_page_slug

        # content_filter
        if instance.content_filter and request.user.is_authenticated():

            favorite_ids = [k['translation_content'] for k in
                            request.user.content_settings.filter(is_favorite=True).values('translation_content')]
            removed_ids = [k['translation_content'] for k in
                           request.user.content_settings.filter(is_removed=True).values('translation_content')]

            if instance.content_filter == 'favorite' and request.user.is_authenticated():
                # objects = objects.filter(id__in=favorite_ids)
                included_objects &= Q(**{'id__in': favorite_ids})

            if instance.content_filter == 'removed' and request.user.is_authenticated():
                # objects = objects.filter(id__in=removed_ids)
                included_objects &= Q(**{'id__in': removed_ids})

            if instance.content_filter == 'no_favorite' and request.user.is_authenticated():
                # objects = objects.exclude(id__in=favorite_ids)
                excluded_objects &= Q(**{'id__in': favorite_ids})

            if instance.content_filter == 'no_removed' and request.user.is_authenticated():
                # objects = objects.exclude(id__in=removed_ids)
                excluded_objects &= Q(**{'id__in': removed_ids})

            if instance.content_filter == 'no_favorite_removed' and request.user.is_authenticated():
                # objects = objects.exclude(id__in=favorite_ids+removed_ids)
                excluded_objects &= Q(**{'id__in': favorite_ids+removed_ids})

        # content_days_included
        if instance.content_days_included:
            begin_date = today - datetime.timedelta(days=instance.content_days_included)
            # objects = objects.filter(content__date_content__gte=begin_date).select_related('content')
            included_objects &= Q(**{'content__date_content__gte': begin_date})

        # content_days_excluded
        if instance.content_days_excluded:
            end_date = today - datetime.timedelta(days=instance.content_days_excluded)
            # objects = objects.filter(content__date_content__lte=end_date).select_related('content')
            included_objects &= Q(**{'content__date_content__lte': end_date})

        # objects_date_range
        if instance.objects_date_range and instance.objects_date_range != "all":
            end_date = today.date()

            if instance.objects_date_range == "day":
                start_date = end_date

            elif instance.objects_date_range == "week":
                start_date = get_first_day_of_week(end_date)

            elif instance.objects_date_range == "month":
                start_date = get_first_day_of_month(end_date)

            elif instance.objects_date_range == "year":
                start_date = get_first_day_of_year(end_date)

            #objects = objects.filter(content__date_content__range=[start_date, end_date]).select_related('content')
            included_objects &= Q(**{'content__date_content__range': [start_date, end_date]})

        # pub_date_begin
        if instance.pub_date_begin:
            # objects = objects.filter(content__date_content__gte=instance.pub_date_begin).select_related('content')
            included_objects &= Q(**{'content__date_content__gte': instance.pub_date_begin})

        # pub_date_end
        if instance.pub_date_end:
            # objects = objects.filter(content__date_content__lte=instance.pub_date_end).select_related('content')
            included_objects &= Q(**{'content__date_content__lte': instance.pub_date_end})

        ############################################################################################
        # Filtri simple e advanced search
        ############################################################################################

        # Simple searched
        # objects = objects.filter(self.get_q_simple_search(request, instance))
        included_objects &= self.get_q_simple_search(request, instance)

        # Advanced search
        # objects = objects.filter(self.get_q_advanced_search(request, instance))
        included_objects &= self.get_q_advanced_search(request, instance)

        ############################################################################################
        # Filtri tag button
        ############################################################################################

        # Sector
        if "sector_tag" in querystring_dict:
            args = querystring_dict["sector_tag"].split("__")
            sector_id, instance_slug = args[0], args[1]
            if instance.slug == instance_slug:
                # objects = objects.filter(content__sector__id=sector_id)
                included_objects &= Q(**{'content__sector__id': sector_id})
        # Kind
        if "kind_tag" in querystring_dict:
            args = querystring_dict["kind_tag"].split("__")
            kind_id, instance_slug = args[0], args[1]
            if instance.slug == instance_slug:
                # objects = objects.filter(content__kind__id=kind_id)
                included_objects &= Q(**{'content__kind__id': kind_id})

        # Corporate
        if "corporate_tag" in querystring_dict:
            args = querystring_dict["corporate_tag"].split("__")
            corporate_id, instance_slug = args[0], args[1]
            if instance.slug == instance_slug:
                # objects = objects.filter(content__car_details__corporate__id=corporate_id)
                included_objects &= Q(**{'content__car_details__corporate__id': corporate_id})
        # Brand
        if "brand_tag" in querystring_dict:
            args = querystring_dict["brand_tag"].split("__")
            brand_id, instance_slug = args[0], args[1]
            if instance.slug == instance_slug:
                # objects = objects.filter(content__car_details__brand__id=brand_id)
                included_objects &= Q(**{'content__car_details__brand__id': brand_id})


        ### FILTER ALL
        objects = objects.filter(included_objects).exclude(excluded_objects).select_related('content')

        ############################################################################################
        # Ordinamento, limite e paginator
        ############################################################################################

        # Ordinamento
        if ordering:
            objects = objects.order_by(*ordering)

        # Limite
        if instance.objects_limit:
            objects = objects[:instance.objects_limit]

        # Paginator
        paginator = Paginator(objects, instance.objects_per_page)
        objects = paginator.page(page)
        adjacent_pages = instance.adjacent_pages
        page_numbers = [n for n in range(page-adjacent_pages, page+adjacent_pages+1) if n>0 and n<=paginator.num_pages]
        show_first = True if 1 not in page_numbers else False
        show_last = True if paginator.num_pages not in page_numbers else False

        context.update({
            'page': page,
            'prev_page': page-1,
            'next_page': page+1,
            'show_first': show_first,
            'show_last': show_last,
            'objects': objects,
            'page_numbers': page_numbers,
            'paginator': paginator,
            'instance': instance,
            'user': request.user,
            'placeholder': placeholder,
            'query_string': query_string,
            'settings': settings,
        })
        return context

    def get_q_simple_search(self, request, content_setting):
        q_objects = Q()
        if (
            request.method == 'GET'
            and 'content_search_submit' in request.GET
            and request.GET.get('content_slug', None) == str(content_setting.slug)
        ):
            # Testo libero
            q_words = [word for word in request.GET['simple-search-input'].split(",")]
            search_lookups = [
                'title__icontains',
                'abstract__icontains',
                'body__icontains',
            ]
            for q_word in q_words:
                for lookup in search_lookups:
                    q_objects |= Q(**{lookup: q_word})
            # Categoria
            if request.GET['categoria'] != "":
                q_objects &= Q(**{'content__kind__name__iexact': request.GET['categoria']})
            # Settore
            if request.GET['settore'] != "":
                q_objects &= Q(**{'content__sector__name__iexact': request.GET['settore']})
            # Marca
            if request.GET['marca'] != "":
                q_objects &= Q(**{'content__car_details__brand__name__iexact': request.GET['marca']})
        return q_objects

    def get_q_advanced_search(self, request, content_setting):
        q_objects = Q()
        if (
            request.method == 'GET'
            and 'advanced_serch_submit' in request.GET
            and request.GET.get('content_slug', None) == str(content_setting.slug)
        ):
            query_dict = dict(
                marca=['content__car_details__brand__name__iexact'],
                settore=['content__sector__name__iexact'],
                categoria=['content__kind__name__iexact'],
                titolo=['title__icontains'],
                id=['content__id'],
                data_inizio=['content__date_content__gte'],
                data_fine=['content__date_content__lte'],
                #allwords=None,
                #onlytheseswords=None,
                #oneword=None,
                sottotitolo=['abstract__icontains'],
            )
            for key in request.GET:
                if key not in ["allwords", "onlytheseswords", "oneword"]:
                    if key in query_dict:
                        for value in request.GET.getlist(key):
                            value = value.strip()
                            if value != "":
                                for lookup in query_dict[key]:
                                    if key in ["data_inizio", "data_fine"]:
                                        formatted_value = datetime.datetime.strptime(value, '%m/%d/%Y %I:%M %p').strftime('%Y-%m-%d %I:%M')
                                        q_objects &= Q(**{lookup: formatted_value})
                                    else:
                                        q_objects &= Q(**{lookup: value})
                    else:
                        if key == "has_foto":
                            # q_objects &= Q(**{'content__photo__isnull': False})
                            q_objects &= Q(**{'content__media_photos__gt': 0})
                        elif key == "has_table":
                            # q_objects &= Q(**{'content__table__isnull': False})
                            q_objects &= Q(**{'content__media_tables__gt': 0})
                        elif key == "has_video":
                            # q_objects &= Q(**{'content__photo__isnull': False})
                            q_objects &= Q(**{'content__media_videos__gt': 0})
                else:
                    search_lookups = [
                        'title__icontains',
                        'abstract__icontains',
                        'body__icontains',
                    ]
                    # Tutte queste parole
                    if key == "allwords":
                        words = request.GET["allwords"].split(",")
                        for word in words:
                            word = word.strip()
                            if word != "":
                                lk_objects = Q()
                                for lookup in search_lookups:
                                    #q_objects &= Q(**{lookup: word})
                                    lk_objects |= Q(**{lookup: word})
                                q_objects &= lk_objects
                    # Questa esatta parola o frase
                    elif key == "onlytheseswords":
                        word = request.GET["onlytheseswords"].strip()
                        if word != "":
                            for lookup in search_lookups:
                                q_objects &= Q(**{lookup: word})
                    # Una qualunque di queste parole
                    elif key == "oneword":
                        words = request.GET["oneword"].split(",")
                        for word in words:
                            word = word.strip()
                            if word != "":
                                for lookup in search_lookups:
                                    q_objects |= Q(**{lookup: word})
        return q_objects

plugin_pool.register_plugin(ContentPlugin)


class ContentSearchPlugin(CMSPluginBase):
    model = ContentSearchSettings
    name = _("Content Search")
    module = _("Content")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        self.render_template = "content/cms_plugins/empty.html"

        if instance.status == 0:
            return context

        self.render_template = instance.template_name
        kinds = Kind.objects.filter(status=1).filter(context="news")
        sectors = Sector.objects.filter(status=1).filter(context="news")
        brands = Brand.objects.filter(status='active').filter(auto=True).filter(kind='product')

        form_class = import_class(instance.form)
        form = form_class()
        if request.method == 'POST' and 'content_search_submit' in request.POST:
            form = form_class(request.POST)

        context.update({
            'instance': instance,
            'placeholder': placeholder,
            'kinds': kinds,
            'sectors': sectors,
            'brands': brands,
            'form_search': form,
        })
        return context

plugin_pool.register_plugin(ContentSearchPlugin)


class ContentAdvancedSearchPlugin(CMSPluginBase):
    model = ContentAdvancedSearchSettings
    name = _("Content advanced Search")
    module = _("Content")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        self.render_template = "content/cms_plugins/empty.html"

        if instance.status == 0:
            return context

        self.render_template = instance.template_name
        kinds = Kind.objects.filter(status=1).filter(context="news")
        sectors = Sector.objects.filter(status=1).filter(context="news")
        brands = Brand.objects.filter(status='active').filter(auto=True).filter(kind='product')

        form_class = import_class(instance.form)
        form = form_class()
        if request.method == 'POST' and 'content_advanced_search_submit' in request.POST:
            form = form_class(request.POST)

        context.update({
            'instance': instance,
            'placeholder': placeholder,
            'kinds': kinds,
            'sectors': sectors,
            'brands': brands,
            'form_search': form,
        })
        return context

plugin_pool.register_plugin(ContentAdvancedSearchPlugin)
