# -*- coding: utf-8 -*-
""" api.py - functions for accessing dam from python.
"""

import requests
import json
from hashlib import sha1
from django.conf import settings



if getattr(settings, 'CONTENT_DAM_API_KEY', None) is None:
    settings.CONTENT_DAM_API_KEY = "77ad6123e4135dbd22a0b11fc2f01a223ccb9678"
    settings.CONTENT_DAM_BASE_URL = "http://localhost:9000/api/"
    settings.CONTENT_DAM_USER = "admin"
    settings.CONTENT_DAM_PWD = "notredam"

API_KEY = settings.CONTENT_DAM_API_KEY
BASE_URL = settings.CONTENT_DAM_BASE_URL



def dam_login(username, password):
    """Get secret needed to access REST API"""

    url = BASE_URL + 'login/'
    data = dict(user_name=username,
                password=password,
                api_key=API_KEY)
    jstring = requests.post(url, data=data,)
    res = json.loads(jstring.text)

    return res


def dam_compute_checksum(secret, **params):
    """Compute checksup as per:
    http://distrettoict.crs4.it/labcontenuti/wiki/public/api

    - concatenate the argument name-value pair, inclunding api_key and user_id, e.g. :api_key=1, user_id=1, foo=1, bar=2 => ['foo1', 'bar2', 'api_key1', 'user_id1']
    - sort the list of arguments obtained into alphabetical order: 
    l = ['foo1', 'bar2', 'api_key1', 'user_id1']
    l.sort()

    -     then concatenate the shared secret and the argument name-value pairs, e.g. 

    s = SECRET + ''.join(l)

    - calculate the sha1() hash of this string 

    from hashlib import sha1
    checksum = sha1(s).hexdigest()

    """

    concat_params = []
    for k,v in params.items():
        if isinstance(v, list):
            concat_params.extend([str(k) + str(s) for s in v])
        else:
            concat_params.append(str(k) + str(v))
        
    #concat_params = [str(k) + str(v) for k,v in params.items()]
    concat_params.sort()
        
    checksum = sha1(secret + ''.join(concat_params)).hexdigest()

    return checksum


def dam_prepare_params(secret, **params):

    """
    - append this value to the argument list with the name 'checksum'. So, the final arguments list will be: 

    api_key=1&user_id=1&foo=1&bar=2&checksum=8b04574ab4686cb024bdb7e46a9c933a511e4299

    """

    checksum = dam_compute_checksum(secret, **params)

    prepared_params = dict(params)
    prepared_params['checksum'] = checksum

    
    return prepared_params


def dam_workspace_get(user_id, secret):
    """list of workspaces of associated user
    method: GET 
    """

    url = 'workspace/get/'
    params = dam_prepare_params(secret, user_id=user_id,
                                api_key=API_KEY)
    jstring = requests.get(BASE_URL + url, params=params,)
    res = json.loads(jstring.text)

    return res


def dam_workspace_get_keywords(user_id, secret, workspace_id):

    url = 'workspace/%s/get_keywords/' % workspace_id

    params = dam_prepare_params(secret, user_id=user_id,
                                api_key=API_KEY)

    jstring = requests.get(BASE_URL + url, params=params,)
    res = json.loads(jstring.text)

    return res


def dam_workspace_get_items(user_id, secret, workspace_id, query=''):
    """Allows to retrieve workspace items.

    Items can be filtered querying by a string contained in metadata,
    media type, keywords, smart folders, creation date and last
    update. It is also possible to retrieve deleted items.

    Items returned are paginated. To improve performance, you can
    choose which item information retrieve: metadata, keywords,
    renditions.

    """

    url = 'workspace/{id}/get_items/'.format(id=workspace_id)

    params = dam_prepare_params(secret,
                                query=query,
                                start=0,
                                limit=100,
                                renditions='["original", "thumbnail"]',
                                user_id=user_id,
                                api_key=API_KEY)

    jstring = requests.get(BASE_URL + url, params=params,)
    
    res = json.loads(jstring.content)

    return res

def dam_workspace_new_item(user_id, secret, workspace_id, media_type='image/jpeg'):
    url = 'item/new/'
    params = dam_prepare_params(secret,
                                user_id=user_id,
                                api_key=API_KEY,
                                workspace_id=workspace_id,
                                media_type=media_type)
    resp = requests.post(BASE_URL + url, data=params)

    return resp

def dam_workspace_upload_item(user_id, secret, workspace_id, item_id, file_path_or_stream):

    url = 'item/{id}/upload/'.format(id=item_id)
    params = dam_prepare_params(secret,
                                user_id=user_id,
                                api_key=API_KEY,
                                workspace_id=1,
                                rendition_id=1)

    if isinstance(file_path_or_stream, file):
        stream = file_path_or_stream
    else: 
        stream = open(file_path_or_stream, 'r')

    resp = requests.post(BASE_URL + url, data=params, files={'files_to_upload': stream})

    stream.close()
    
    return resp

def dam_workspace_get_scripts(user_id, secret, workspace_id):

    url = 'workspace/{id}/get_scripts/'.format(id=workspace_id)

    params = dam_prepare_params(
        secret,
        user_id=user_id,
        api_key=API_KEY,
        )

    resp = requests.get(BASE_URL + url, params=params)

    return resp

def dam_item_add_metadata(user_id, secret, item_id, metatada):

    url = 'item/{id}/set_metadata/'.format(id=item_id)

    params = dam_prepare_params(
        secret,
        user_id=user_id,
        api_key=API_KEY,
        metadata=json.dumps(metadata),
        )

    resp = requests.post(BASE_URL + url, data=params)

    return resp

    
def dam_workspace_run_script(user_id, secret, script_id, item_ids=[]):
    url = 'script/{id}/run/'.format(id=script_id)

    params = dam_prepare_params(secret,
                                user_id=user_id,
                                items=list(item_ids),
                                api_key=API_KEY,
                                )
    
    resp = requests.post(BASE_URL + url, data=params)

    return resp

if __name__ == '__main__':
    
    dam_sess_data = dam_login(settings.CONTENT_DAM_USER, settings.CONTENT_DAM_PWD)
    sess_secret = dam_sess_data['secret']
    user_id = dam_sess_data['user_id']
    
    #prepared_params = dam_prepare_params(sess_secret, **dict(a=32, b=54))

    #print prepared_params

    workspaces = dam_workspace_get(user_id, sess_secret)

    print workspaces
    
    for w in workspaces:
        id = w['id']


    w_id = workspaces[1]['id']

    # create a new item
    n_id = dam_workspace_new_item(user_id,
                           sess_secret,
                           w_id,)



    print "new item id: {id}".format(id=n_id)


    img_path = '/usr/lib/libreoffice/share/gallery/www-back/roses.jpg'

    item_id = n_id.json()['id']
    # upload image in item
    with open(img_path, 'r') as f:
        dam_workspace_upload_item(
            user_id,
            sess_secret,
            w_id,
            item_id,
            f,
        )


    # add metadata
    
    metadata = dict(dc_subject=['test_article', '123456-654321'])
    resp = dam_item_add_metadata(user_id, sess_secret, item_id, metadata)

    if resp != '':
        print resp.text

    scripts = dam_workspace_get_scripts(user_id, sess_secret, w_id).json()



    for x in scripts['scripts']:
        if x['name'] == 'image renditions':
            resp = dam_workspace_run_script(user_id, sess_secret, script_id=unicode(x['id']), item_ids=[unicode(item_id),])
            if resp != '':
                print repr(resp.text)










