# coding: utf-8

from __future__ import division
from __future__ import unicode_literals
import math
from ftfy import fix_text
from decimal import Decimal

import re
import os
import calendar
import datetime

from BeautifulSoup import BeautifulSoup
from babel.dates import format_date
from babel.numbers import format_decimal
from dateutil.relativedelta import relativedelta
from operator import itemgetter
import json
import urllib2
from filer.fields.image import FilerImageField

from django.utils.encoding import force_unicode
from django.template import loader
from django.template.base import Context

from django.conf import settings
from django.contrib.auth.models import User
from django.core.validators import MinValueValidator, MaxValueValidator
from django.core.urlresolvers import reverse
from django.db import models
from django.db.models import Min, Max, Avg, Q
from django.template.defaultfilters import slugify
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from django.utils.html import format_html


from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jmb.core.db.fields import HTMLFieldTinyMCE
from jmb.core.utils.functions import zerofill
from jmb.i18n.models import Language
from jmb.content.models import ContentAbstract, TranslationContentAbstract, SourceAbstract
from jmb.tag.models import Tag
import mimetypes


def media_path(media_id, number=3):
    """
    return relative media path
    example: media_id = 1234567 and number=3 -> 123/456/7/
    """
    path = ''
    index = 0
    media_id = str(media_id)
    while index * number < len(media_id):
        path += media_id[index*number:(index+1)*number] + '/'
        index += 1
    return path


def get_range(instance, field, label, reverse=False, opt_filters={}, opt_exclude={}):
    try:
        items_dict = instance.cars.filter(status__in=['ok', 'working'], **opt_filters).exclude(**opt_exclude).aggregate(
            Min(field), Max(field)
        )
        aggregate = items_dict.values()
        if all(v is None for v in aggregate):
            return ''.join("-")
        else:
            aggregate = sorted(set(aggregate))
            if reverse:
                aggregate = aggregate[::-1]

            aggregate = [
                round(val, 1) if isinstance(val, (float, Decimal))
                else val for val in aggregate
            ]

            aggregate = [
                str(format_decimal(val, locale='it')) if isinstance(val, (int, float, Decimal))
                else val for val in aggregate
            ]
            return " - ".join(u"{}".format(val) for val in aggregate) + ' ' + label
    except:
        return ''.join("-")


class GenericModel(models.Model):
    @classmethod
    def get_fields(cls):
        return [field.attname for field in cls._meta.fields]

    class Meta:
        abstract = True


class Corporate(DateModel, UserModel, OrderedModel, GenericModel):
    name = models.CharField(
        max_length=32, unique=True,
        db_index=True,
        verbose_name=_('name'),
    )
    nick = models.CharField(
        max_length=32,
        blank=True, null=True,
        verbose_name=_('nick')
    )
    kind = models.CharField(
        max_length=10,
        choices=(
            ('producer', _('Producer')),
            ('agency', _('Agency')),
            ('sport', _('Sport')),
            ('tuner', _('Tuner')),
            ('tyres', _('Tyres')),
            ('parts', _('Parts')),
            ('lubricants', _('Lubricants')),
            ('designer', _('Designer')),
            ('holding', _('Holding')),
        ),
        db_index=True,
        blank=True, null=True,
        verbose_name=_('kind'),
    )
    status = models.CharField(
        max_length=8,
        choices=settings.PUBLIEDITOR_CORPORATE_STATO_CHOICES,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('status'),
    )
    alts = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('alts'),
    )
    url = models.URLField(
        blank=True, null=True,
    )
    wiki = models.URLField(
        blank=True, null=True,
        verbose_name=_('wiki'),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_('note'),
    )
    ticker = models.CharField(
        max_length=24,
        blank=True, null=True,
        verbose_name=_('ticker')
    )  # nome quotazione in borsa
    slug = models.SlugField(
        max_length=100,
        unique=True,
        db_index=True,
        verbose_name=_("slug"),
    )

    class Meta:
        verbose_name = _('Corporate')
        verbose_name_plural = _('Corporates')
        ordering = ('ordering', 'name',)
        index_together = (("ordering", "name"),)

    def __unicode__(self):
        return u"%s" % self.name

    def get_last_corporate_translation_contents(self, limit=10):
        contents = TranslationContent.objects.filter(
            content__car_details__corporate=self
        )[:limit]
        return contents


class Brand(DateModel, UserModel, OrderedModel, GenericModel):
    corporates = models.ManyToManyField(
        Corporate,
        through='CorporateBrandLink',
        blank=True, null=True,
        related_name='brands',
        verbose_name=_('corporate')
    )
    name = models.CharField(
        max_length=64,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('name'),
    )  # unique
    nick = models.CharField(
        max_length=64,
        blank=True, null=True,
        verbose_name=_('nick'),
    )
    alts = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('alts'),
    )
    shorts = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('shorts'),
    )
    url = models.CharField(
        max_length=16,
        blank=True, null=True,
        verbose_name=_('url'),
    )
    logo = FilerImageField(
        null=True, blank=True
    )
    logo_small = models.ImageField(
        verbose_name=_('Logo small'),
        upload_to="brands_small",
        null=True, blank=True
    )
    logo_big = models.ImageField(
        verbose_name=_('Logo big'),
        upload_to="brands_big",
        null=True, blank=True
    )
    auto = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('auto'),
    )
    moto = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('moto'),
    )
    truck = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('truck'),
    )
    kind = models.CharField(
        max_length=10,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_BRAND_KIND_CHOICES,
        verbose_name=_('kind'),
    )
    status = models.CharField(
        max_length=7,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_BRAND_STATO_CHOICES,
        verbose_name=_('status'),
    )
    start_date = models.DateTimeField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('start date time'),
    )
    end_date = models.DateTimeField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('end date time'),
    )
    market = models.CharField(
        max_length=32,
        blank=True, null=True,
        verbose_name=_('market')
    )
    popolarity = models.CharField(
        max_length=7,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_BRAND_POPOLARITY_CHOICES,
        verbose_name=_('popolarity'),
    )
    web = models.URLField(
        max_length=64,
        blank=True, null=True,
        verbose_name='web',
    )
    phone = models.CharField(
        max_length=64L,
        blank=True, null=True,
        verbose_name=_('phone'),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_('note'),
    )
    slug = models.SlugField(
        max_length=100,
        unique=True,
        db_index=True,
        verbose_name=_("slug"),
    )

    class Meta:
        verbose_name = _('Brand')
        verbose_name_plural = _('Brands')
        ordering = ('ordering', 'name',)
        index_together = (("ordering", "name"),)

    def __unicode__(self):
        return u"%s" % self.name

    @classmethod
    def get_brand_choices(cls):
        return [
            dict(
                label=brand.name,
                thumbnail=brand.name,
                value=brand.id
            ) for brand in cls.objects.filter(
                status='active',
                cars__status='ok',
                cars__model__status__in=('online', 'runout'),
            ).order_by('name').distinct('name')
        ]

    @classmethod
    def get_brand_choices_list(cls):
        return [
            (brand.id, brand.name) for brand in cls.objects.filter(
                status='active',
                cars__status='ok',
                cars__model__status__in=('online', 'runout'),
            ).order_by('name').distinct('name')
        ]

    def admin_image(self):
        return '<img src="%s"/>' % self.logo

    admin_image.allow_tags = True

    def model_count(self):
        return self.models.filter(status__in=('online', 'runout')).count()

    def car_count(self):
        return self.cars.filter(status='ok', model__status__in=('online', 'runout')).count()

    def get_logo_url(self):
        if self.logo_small:
            return "{0}://{1}{2}".format(
                settings.PUBLIEDITOR_CONTENT_MEDIA_PROTOCOL,
                settings.PUBLIEDITOR_CONTENT_MEDIA_DOMAIN,
                self.logo_small.url
            )
        return "%s/brands/_small-trans/%s.png" % (settings.PUBLIEDITOR_RCS_CDN_URL, self.name)

    def get_logo(self):
        return "<img src='%s' alt=%s>" % (self.get_logo_url(), self.name)
    get_logo.allow_tags = True
    get_logo.short_description = _("Logo")

    def get_logo_small(self):
        return "<img src='{0}' alt='{1}'>".format(self.get_logo_url(), self.name)
    get_logo_small.allow_tags = True
    get_logo_small.short_description = _("Logo S")

    def get_logo_big(self):
        return "<img src='{0}' alt='{1}'>".format(self.get_logo_url_big(), self.name)
    get_logo_big.allow_tags = True
    get_logo_big.short_description = _("Logo B")

    def get_last_brand_translation_contents(self, limit=10):
        contents = TranslationContent.objects.filter(
            content__car_details__brand=self
        )[:limit]
        return contents

    def get_logo_url_big(self):
        if self.logo_big:
            return "{0}://{1}{2}".format(
                settings.PUBLIEDITOR_CONTENT_MEDIA_PROTOCOL,
                settings.PUBLIEDITOR_CONTENT_MEDIA_DOMAIN,
                self.logo_big.url
            )
        return "%s/brands/_trans_med/%s.png" % (settings.PUBLIEDITOR_RCS_CDN_URL, self.name)

    def get_absolute_url(self):
        return reverse('content:brand_detail', args=[self.slug])

    @property
    def last_corporate(self):
        today = datetime.date.today()
        corporates = self.corporates.filter(
            (Q(brand_links__start_date__isnull=True) | Q(brand_links__start_date__lte=today)),
            (Q(brand_links__end_date__isnull=True) | Q(brand_links__end_date__gte=today))
        ).order_by('brand_links__end_date')
        if corporates:
            return corporates[0]
        return None

    @property
    def min_price(self):
        aggregate = self.cars.filter(status="ok", model__status__in=('online', 'runout')).aggregate(Min('price'))
        return aggregate['price__min']

    @property
    def max_price(self):
        aggregate = self.cars.filter(status="ok", model__status__in=('online', 'runout')).aggregate(Max('price'))
        return aggregate['price__max']

    def get_autocomposed_text(self):
        template_txt = settings.PUBLIEDITOR_BRAND_DESCRIPTION
        context_dict = {'brand': self}
        context = Context(context_dict)
        template = loader.get_template_from_string(template_txt)
        ret = template.render(context)
        return ret

    def first_letter(self):
        return self.name.upper()[0]


class CarModel(DateModel, UserModel, OrderedModel, GenericModel):
    status = models.CharField(
        max_length=10,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CARMODEL_STATUS_CHOICES,
        verbose_name=_('status')
    )
    brand = models.ForeignKey(
        Brand,
        related_name="models",
        blank=True, null=True,
        verbose_name=_('brand'),
    )
    brand_info = models.CharField(
        max_length=64,
        blank=True, null=True,
        verbose_name=_('brand info'),
    )
    name = models.CharField(
        max_length=64,
        db_index=True,
        verbose_name=_('name'),
    )
    alts = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('alts'),
    )
    multi = models.IntegerField(
        blank=True, null=True,
        verbose_name=_('multi'),
    )
    descr = models.TextField(
        blank=True, null=True,
        verbose_name=_('description'),
    )
    new_date = models.DateTimeField(
        null=True, blank=True,
        verbose_name=_('new date time'),
    )
    mod_date = models.DateTimeField(
        null=True, blank=True,
        verbose_name=_('modify date time'),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_('note'),
    )
    pmodels_id = models.IntegerField(
        null=True, blank=True,
        verbose_name=_('pmodels id'),
    )
    safety = models.DecimalField(
        max_digits=4, decimal_places=1,
        blank=True, null=True,
        verbose_name=_('safety'),
    )
    euroncap_name = models.CharField(
        max_length=64,
        blank=True, null=True,
        verbose_name=_('euroncap name'),
    )
    shapes = models.CharField(
        max_length=255,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('shapes'),
    )
    driving = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('driving'),
    )
    new_entry = models.CharField(
        max_length=4,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CARMODEL_NEW_ENTRY_CHOICES,
        default='side',
        verbose_name=_('new entry'),
    )
    phase = models.CharField(
        max_length=32,
        blank=True, null=True,
        verbose_name=_('phase'),
    )
    brochure_link = models.URLField(
        blank=True,
        null=True,
        default="",
        verbose_name=_('brochure link')
    )
    listino_link = models.URLField(
        blank=True,
        null=True,
        default="",
        verbose_name=_('listino link')
    )
    slug = models.SlugField(
        max_length=100,
        db_index=True,
        unique=True,
        verbose_name=_("slug"),
    )
    year = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('year'),
    )

    class Meta:
        verbose_name = _('CarModel')
        verbose_name_plural = _('CarModels')
        ordering = ('ordering', 'name')
        index_together = (
            ("date_last_modify", "status", "ordering", "name",),
            ("status", "ordering", "name",),
            ("status", "id",),
        )

    def __unicode__(self):
        return u"%s (%s)" % (self.name, self.status)

    def save(self, *args, **kwargs):
        if self.status == 'runout':
            self.year = self.get_cars_max_year()
        else:
            self.year = None
        super(CarModel, self).save(*args, **kwargs)

    def get_model_name_year(self):
        if self.year:
            return '{name} ({year})'.format(name=self.name, year=self.year)
        else:
            return self.name
    get_model_name_year.short_description = _('name')

    def shapes_version(self):
        versions = self.cars.filter(status='ok')
        if versions.count():
            return u", ".join(list(set([version.get_shape_display() for version in versions])))
        return u"-"
    shapes_version.short_description = _("Shapes")

    @property
    def image_count(self):
        count = self.photos.count()
        return count

    @property
    def video_count(self):
        count = self.videos.count()
        return count

    @property
    def version_count(self):
        return self.cars.count()

    @property
    def min_price(self):
        aggregate = self.cars.aggregate(Min('price'))
        return aggregate['price__min']

    @property
    def avg_price(self):
        aggregate = self.cars.aggregate(Avg('price'))
        return aggregate['price__avg']

    @property
    def max_price(self):
        aggregate = self.cars.aggregate(Max('price'))
        return aggregate['price__max']

    @property
    def min_length(self):
        aggregate = self.cars.aggregate(Min('length'))
        return aggregate['length__min']

    @property
    def avg_length(self):
        aggregate = self.cars.aggregate(Avg('length'))
        return aggregate['length__avg']

    @property
    def avg_power_cv(self):
        aggregate = self.cars.aggregate(Avg('power_cv'))
        return aggregate['power_cv__avg']

    @property
    def max_power_cv(self):
        aggregate = self.cars.aggregate(Max('power_cv'))
        return aggregate['power_cv__max']

    @property
    def descr_len(self):
        return len(self.descr) if self.descr else 0

    # TODO: fixme
    # standard_num = models.IntegerField(blank=True, null=True)  funzione

    def cars_count(self):
        return self.cars.filter(status='ok').count()

    def get_driving(self):
        values_list = set([c.driving for c in self.cars.filter(status='ok')])
        try:
            if len(values_list) == 1:
                if 0 in values_list:
                    return "Nessuna versione"
                elif 1 in values_list:
                    return "Tutte le versioni"
            else:
                if 0 in values_list and 1 in values_list:
                    return "Alcune versioni"
                else:
                    return ', '.join("-")
        except:
            return ', '.join("-")

    def get_area_c(self):
        values_list = set([c.area_c for c in self.cars.filter(status='ok')])
        try:
            if len(values_list) == 1:
                if 0 in values_list:
                    return "Nessuna versione"
                elif 1 in values_list:
                    return "Tutte le versioni"
            else:
                if 0 in values_list and 1 in values_list:
                    return "Alcune versioni"
                else:
                    return ', '.join("-")
        except:
            return ', '.join("-")

    def get_safety(self):
        if not self.safety:
            return "-"
        else:
            return mark_safe("<img src='%s/_imc/safety/%d.png'>" % (settings.PUBLIEDITOR_RCS_CDN_URL, self.safety))

    def get_others_variants(self):
        other_variants = CarModel.objects\
            .filter(status__in=('online', 'runout'), multi=self.multi, multi__isnull=False)\
            .exclude(id=self.id)
        return other_variants

    def check_has_electric_engine_version(self):
        return self.cars.filter(status__in=['ok', 'working'], engine__in=settings.PUBLIEDITOR_ELECTRIC_ENGINE_LIST).exists()

    def check_has_hybrid_engine_version(self):
        return self.cars.filter(status__in=['ok', 'working'], engine__in=settings.PUBLIEDITOR_HYBRID_ENGINE_LIST).exists()

    def check_has_standard_engine_version(self):
        return self.cars.filter(status__in=['ok', 'working'], engine__in=settings.PUBLIEDITOR_STANDARD_ENGINE_LIST).exists()

    def get_range_seats(self):
        return get_range(self, 'seat', 'posti')

    def get_range_doors(self):
        return get_range(self, 'doors', 'porte')

    def get_range_luggage(self):
        return get_range(self, 'luggage', 'litri')

    def get_range_luggage_max(self):
        return get_range(self, 'luggage_max', 'litri')

    def get_range_tank(self):
        return get_range(self, 'tank', 'litri')

    def get_range_length(self):
        return get_range(self, 'length', 'cm')

    def get_range_width(self):
        return get_range(self, 'width', 'cm')

    def get_range_height(self):
        return get_range(self, 'height', 'cm')

    def get_range_weight(self):
        return get_range(self, 'weight', 'kg')

    def get_range_displ(self):
        return get_range(self, 'displ', 'cc')

    def get_range_power_cv(self):
        return get_range(self, 'power_cv', 'CV', reverse=True)

    def get_range_torque_max(self):
        return get_range(self, 'torque_max', 'Nm')

    def get_range_acc(self):
        return get_range(self, 'acc', 'sec', reverse=True)

    def get_range_speed(self):
        return get_range(self, 'speed', 'km/h')

    def get_range_power_weight(self):
        return get_range(self, 'power_weight', 'kg/CV', reverse=True)

    def get_range_power_spec(self):
        return get_range(self, 'power_spec', 'CV/litro', opt_exclude={'engine__in': settings.PUBLIEDITOR_ELECTRIC_ENGINE_LIST})

    def get_range_kml_mix(self):
        return get_range(self, 'kml_mix', 'km/litro')

    def get_range_average_electric_consumption(self):
        return get_range(self, 'average_electric_consumption', 'kWh/100 km')

    def get_range_electric_average_mileage(self):
        return get_range(self, 'electric_average_mileage', 'km/kWh')

    def get_range_cons_mix(self):
        return get_range(self, 'cons_mix', 'litri/100 km', reverse=True)

    def get_range_co2(self):
        return get_range(self, 'co2', 'g/km', reverse=True)

    def get_cars_targets(self):
        values_list = set([c.get_target_display() for c in self.cars.filter(status='ok')])
        return ', '.join(values_list) if values_list else ', '.join("-")

    def get_cars_engines(self):
        values_list = set([c.get_engine_display() for c in self.cars.filter(status='ok')])
        return ', '.join(values_list) if values_list else ', '.join("-")

    def get_cars_number_of_engines(self):
        values_list = set(
            [c.number_of_engines for c in self.cars.filter(
                status='ok'
            ).exclude(
                number_of_engines__isnull=True
            )]
        )
        values_list = [str(value) for value in values_list]
        return ', '.join(values_list) if any(values_list) else ', '.join("-")

    def get_cars_fuels(self):
        values_list = set([c.get_fuel_display() for c in self.cars.filter(status='ok')])
        return ', '.join(values_list) if values_list else ', '.join("-")

    def get_cars_tractions(self):
        values_list = set([c.get_traction_display() for c in self.cars.filter(status='ok')])
        return ', '.join(values_list) if values_list else ', '.join("-")

    def get_cars_layout(self):
        values_list = set([c.get_layout_display() for c in self.cars.filter(status='ok')])
        return ', '.join(values_list) if values_list else ', '.join("-")

    def get_cars_gears(self):
        values_list = set([c.get_gear_display() for c in self.cars.filter(status='ok')])
        return ', '.join(values_list) if values_list else ', '.join("-")

    def get_cars_engine_loc(self):
        values_list = set([c.get_engine_loc_display() for c in self.cars.filter(status='ok')])
        return ', '.join(values_list) if values_list else ', '.join("-")

    def get_cars_pollution(self):
        try:
            values_list = set([c.get_pollution_display() for c in self.cars.filter(status='ok')])
            return ', '.join(values_list) if values_list else ', '.join("-")
        except:
            return ', '.join("-")

    def get_cars_max_year(self):
        """
        :return: max year of published car of this model
        """
        return self.cars.filter(status__exact='ok').aggregate(Max('year'))['year__max']

    def get_cars_range_prices(self):
        return self.cars.filter(status="ok").aggregate(Min('price'), Max('price'))

    def get_absolute_url(self):
        return reverse('content:car_model_detail', args=[self.brand.slug, self.slug])

    def get_photo_url_list(self):
        photos = self.photos.all()
        url_list = []
        if photos:
            for photo in photos:
                url_list.append(photo.get_photo_url_formats())
        return url_list

    def get_first_photo_url(self):
        # return "https://rcs.cdn.publieditor.it/w300/M0%s_01.jpg" % self.id
        photos = self.photos.all()
        if photos:
            return photos[0].photo
        return None

    def is_valid_url(self, url):
        try:
            f = urllib2.urlopen(urllib2.Request(url))
            return True
        except:
            return False

    def get_photo_url_small(self, index='01'):
        # https://rcs.cdn.publieditor.it/w55h35/MXXXX_01.jpg
        zid = zerofill(self.id, 4)
        url = "%s/w55h35/M%s_%s.jpg" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, index)
        if self.is_valid_url(url):
            return url
        return "https://placehold.it/55x35&text=M%s_01.jpg" % zid

    def get_photo_url_medium(self, index='01'):
        # https://rcs.cdn.publieditor.it/w98h60/MXXXX_01.jpg
        zid = zerofill(self.id, 4)
        url = "%s/w98h60/M%s_%s.jpg" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, index)
        if self.is_valid_url(url):
            return url
        return "https://placehold.it/98x60&text=M%s_01.jpg" % zid

    def get_photo_url_medium_9860(self, index='01'):
        # https://rcs.cdn.publieditor.it/pw98h60/MXXXX_01.jpg
        zid = zerofill(self.id, 4)
        url = "%s/pw98h60/M%s_%s.jpg" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, index)
        if self.is_valid_url(url):
            return url
        return "https://placehold.it/98x60&text=M%s_01.jpg" % zid

    def get_photo_url_large(self, index='01'):
        # https://rcs.cdn.publieditor.it/w300/MXXXX_01.jpg
        zid = zerofill(self.id, 4)
        url = "%s/w300/M%s_%s.jpg" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, index)
        if self.is_valid_url(url):
            return url
        return "https://placehold.it/300x225&text=M%s_01.jpg" % zid

    def get_photo_url_wide_large(self, index='00'):
        # https://rcs.cdn.publieditor.it/w300/MXXXX_00.jpg
        zid = zerofill(self.id, 4)
        url = "%s/w300/M%s_%s.jpg" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, index)
        if self.is_valid_url(url):
            return url
        return self.get_photo_url_original()

    def get_photo_url_original(self, index='01'):
        # https://rcs.cdn.publieditor.it/w640/MXXXX_01.jpg
        zid = zerofill(self.id, 4)
        url = "%s/w640/M%s_%s.jpg" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, index)
        if self.is_valid_url(url):
            return url
        return "https://placehold.it/640x480&text=M%s_01.jpg" % zid

    def get_photo_url_wide(self, index='00'):
        # https://rcs.cdn.publieditor.it/w640/MXXXX_00.jpg
        zid = zerofill(self.id, 4)
        url = "%s/w640/M%s_%s.jpg" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, index)
        if self.is_valid_url(url):
            return url
        return self.get_photo_url_original()

    # def get_video_url_list(self):
    #     videos = self.videos.all()
    #     url_list = []
    #     if videos:
    #         for video in videos:
    #             url_list.append(video.video)
    #     return url_list

    def get_video_url_list(self):
        """
        restituisce la lista delle url dei video
        alta risoluzione: https://rcs.cdn.publieditor.it/xv/M%s_%s.mp4
        bassa risoluzione: https://rcs.cdn.publieditor.it/mv/M%s_%s.mp4
        """
        url_list = []
        zid = zerofill(self.id, 4)
        url = "%s/xv/M%s_%s.mp4" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, '01')
        ind = 1
        zind = zerofill(ind, 2)
        f = self.is_valid_url(url)
        while f is True:
            url_list.append({
                'url': "%s/xv/M%s_%s.mp4" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, zind),
                'thumbnail': self.get_photo_url_medium_9860(zind)
            })
            # veerifico se esiste un altro video
            ind += 1
            zind = zerofill(ind, 2)
            url = "%s/xv/M%s_%s.mp4" % (settings.PUBLIEDITOR_RCS_CDN_URL, zid, zind)
            f = self.is_valid_url(url)
        return url_list

    def get_first_video_url(self):
        # return "https://rcs.cdn.publieditor.it/xv/M0%s_01.mp4" % self.id
        videos = self.videos.all()
        if videos:
            return videos[0].video
        return None

    def get_cars_shapes(self):
        """
        return shapes of cars
        """
        shapes = set()
        for car in self.cars.filter(status='ok'):
            shapes.add(car.shape)
        return list(shapes)

    def get_cars_driving(self):
        """
        return driving of cars
        """
        driving = set()
        for car in self.cars.filter(status='ok'):
            driving.add(car.driving)
        return list(driving)

    def is_spider(self):
        """
        if at least a Car is sider CarModel is considered "spider"
        """
        return True if self.cars.filter(status='ok', seat='2') else False

    # def similar_models(self):
    #     shapes = self.get_cars_shapes()
    #     driving = self.get_cars_driving()
    #     model_list = []
    #
    #     if shapes and driving:
    #         similar_models = CarModel.objects.filter(
    #             status__in=('online', 'runout'), cars__status='ok',
    #             cars__shape__in=shapes, cars__driving__in=driving,
    #             cars__price__isnull=False, cars__length__isnull=False
    #         ).exclude(
    #             id=self.id
    #         ).distinct()
    #
    #         for model in similar_models:
    #             try:
    #                 price_calc = (1 - abs(self.min_price - model.min_price) / model.min_price)
    #                 length_calc = (1 - abs(self.min_length - model.min_length) / model.min_length)
    #                 power_cv_calc = (1 - abs(self.max_power_cv - model.max_power_cv) / model.max_power_cv)
    #                 if price_calc >= 0.7 and length_calc >= 0.8 and power_cv_calc >= 0.8:
    #                     xsum = price_calc * length_calc * power_cv_calc
    #                     model_list.append({
    #                         'model': model,
    #                         'xsum': xsum,
    #                         'price_calc': price_calc,
    #                         'length_calc': length_calc,
    #                         'power_cv_calc': power_cv_calc
    #                     })
    #             except:
    #                 continue
    #         model_list = sorted(model_list, key=itemgetter('xsum'), reverse=True)
    #     return model_list

    def similar_models(self):

        shapes = self.get_cars_shapes()
        driving = self.get_cars_driving()
        # since a for loop over all the similar car models filtered by shapes and driving takes too much time  we need
        # a more restrictive filter. we use a cutoff value for min_length.
        # As before, if min length is not defined all the check gives errors, so we return an empty model_list
        min_length_cutoff_value = 15
        model_list = []


        if shapes and driving:
            try:
                #
                similar_models = CarModel.objects.filter(
                    status__in=('online', 'runout'), cars__status='ok',
                    cars__shape__in=shapes, cars__driving__in=driving,
                    cars__price__isnull=False, cars__length__isnull=False
                ).filter(
                    cars__length__gte=self.min_length-min_length_cutoff_value, cars__length__lte=self.min_length+min_length_cutoff_value
                ).exclude(
                    id=self.id
                ).distinct()


                if self.is_spider():
                    similar_models = similar_models.filter(
                        cars__seat='2',
                    )

                for model in similar_models:
                    try:
                        if self.min_price <= model.min_price:
                            price_calc = self.min_price / model.min_price
                        else:
                            price_calc = model.min_price / self.min_price

                        if self.min_length <= model.min_length:
                            length_calc = self.min_length / model.min_length
                        else:
                            length_calc = model.min_length / self.min_length

                        if self.max_power_cv <= model.max_power_cv:
                            power_cv_calc = self.max_power_cv / model.max_power_cv
                        else:
                            power_cv_calc = model.max_power_cv / self.max_power_cv

                        # spider or not determine different threshold
                        if (
                            (self.is_spider() and (price_calc >= 0.7 and length_calc >= 0.8 and power_cv_calc >= 0.8)) or
                            (not self.is_spider() and (price_calc >= 0.6 and length_calc >= 0.7 and power_cv_calc >= 0.5))
                        ):
                            xsum = price_calc * length_calc * power_cv_calc
                            model_list.append({
                                'model': model,
                                'xsum': xsum,
                                'price_calc': price_calc,
                                'length_calc': length_calc,
                                'power_cv_calc': power_cv_calc
                            })
                    except:
                        continue
            except:
                pass
            model_list = sorted(model_list, key=itemgetter('xsum'), reverse=True)

        return model_list

    def validate_url(self, url):
        try:
            f = urllib2.urlopen(urllib2.Request(url))
            return True
        except:
            return False

    def set_photos(self):
        ind = 1
        id = zerofill(self.id, width=4)
        validate = True
        while validate:
            url = "%s/w640/M%s_%s.jpg" % (
                settings.PUBLIEDITOR_RCS_CDN_URL, id, zerofill(ind, width=2)
            )
            validate = self.validate_url(url)
            if validate:
                photo, created = PhotoCarModelGallery.objects.get_or_create(
                    model=self, photo=url
                )
                ind += 1

    def set_videos(self):
        ind = 1
        id = zerofill(self.id, width=4)
        validate = True
        while validate:
            url = "%s/mv/M%s_%s.mp4" % (
                settings.PUBLIEDITOR_RCS_CDN_URL, id, zerofill(ind, width=2)
            )
            validate = self.validate_url(url)
            if validate:
                video, created = VideoCarModelGallery.objects.get_or_create(
                    model=self, video=url
                )
                ind += 1

    def get_all_equipments(self):
        """
        return all equipments
        """
        return self.equipments.all()

    def get_standard_equipments(self):
        """
        return only standard equipments for this model
        """
        return self.equipments.filter(standard=True)

    def get_last_sale(self):
        today = datetime.datetime.today()
        sale = self.sales.filter(
            period__status=1,
            period__date_begin__lte=today,
        ).order_by('-period__date_end')
        if sale:
            return sale[0]
        return None

    def has_multiple_unrae_carmodels(self):
        today = datetime.datetime.today()
        sales = self.sales.filter(
            period__status=1,
            period__date_begin__lte=today,
        ).order_by('-period__date_begin')
        if sales:
            sale = sales[0]
            return bool(sale.carmodels.all().count()>1)
        return False

    def get_unrae_carmodels_group(self):
        today = datetime.datetime.today()
        carmodels = ""
        sales = self.sales.filter(
            period__status=1,
            period__date_begin__lte=today,
        ).order_by('-period__date_begin')
        if sales:
            sale = sales[0]
            for carmodel in sale.carmodels.all():
                carmodels += "%s %s, " % (carmodel.brand, carmodel.name)
            carmodels = carmodels.rstrip(", ")
        return carmodels

    def get_period_sales(self, data):
        try:
            sum = 0
            for sale in UNRAECarModelSale.objects.filter(
                year=data.year, month=data.month,
                unrae_carmodel__carmodels__in=[self],
            ):
                sum += sale.sold
            return sum if sum else None
        except UNRAECarModelSale.DoesNotExist:
            return None

    def get_sales_line_graph(self):
        grafico_dict = {}
        try:
            last_period = self.get_last_sale().period
            start_date = last_period.date_begin
            end_date = last_period.date_end
            dates = [start_date]
            categories = [format_date(start_date, 'MMM Y', locale='it').upper()]
            values = [self.get_period_sales(start_date)]
            while True:
                start_date += relativedelta(months=1)
                start_date_month_begin, start_date_month_end = calendar.monthrange(start_date.year, start_date.month)
                date_check = datetime.date(start_date.year, start_date.month, start_date_month_end)
                if date_check <= end_date:
                    dates.append(start_date)
                    categories.append(format_date(start_date, 'MMM Y', locale='it').upper())
                    values.append(self.get_period_sales(start_date))
                else:
                    break
            grafico_dict = {
                'chart': {
                    'backgroundColor': getattr(settings, 'PUBLIEDITOR_CONTENT_HIT_BACKGROUND_COLOR', '#FFFFFF'),
                    'renderTo': "container-grafico",
                    'type': 'line'
                },
                "title": {
                    "text": "Andamento vendite",
                    "x": -20
                },
                "subtitle": {
                    "text": None,
                    "x": -20
                },
                "xAxis": {
                    "categories": categories,
                },
                "yAxis": {
                    "min": 0,
                    "title": {
                        "text": 'Unità vendute',
                    },
                    "plotLines": [{
                        "value": 0, "width": 1, "color": '#808080',
                    }]
                },
                'plotOptions': {
                    'series': {
                        'showInLegend': False,
                    },
                },
                "series": [{
                    "name": 'Vendite',
                    "data": values,
                    "color": settings.PUBLIEDITOR_CONTENT_HIT_SERIE_COLOR,
                }],
                "exporting": {
                    "enabled": False,
                },
                'credits': {
                    'enabled': False,
                },
            }
            return json.dumps(grafico_dict)
        except Exception, e:
            return json.dumps(grafico_dict)

    def get_brand_id(self):
        if self.brand:
            return u'{0}'.format(self.brand.id)
        else:
            return u''

    def get_brand_name(self):
        if self.brand:
            return u'{0}'.format(self.brand.name)
        else:
            return u''

    def get_new_date(self):
        if self.new_date:
            return self.new_date.strftime("%d-%m-%Y_%H:%M:%S")
        else:
            return u''

    def get_mod_date(self):
        if self.mod_date:
            return self.mod_date.strftime("%d-%m-%Y_%H:%M:%S")
        else:
            return u''

    def get_sales(self):
        sales_list = []
        for sale in self.sales.all():
            sales_list.append(u'{0} n.{1}'.format(sale.period, sale.sold))
        return u', '.join(sales_list)

    def get_latest_sales_label(self):
        if self.get_last_sale():
            return self.get_last_sale().period.name
        else:
            return u''

    def get_latest_sales_quantity(self):
        if self.get_last_sale():
            return self.get_last_sale().sold
        else:
            return u''


class FullCarModelManager(models.Manager):
    def get_query_set(self):
        return super(FullCarModelManager, self).get_query_set().filter(cars__isnull=False).distinct()


class FullCarModel(CarModel):
    """
    gestisce i modelli che hanno almeno un'automobile/versione
    """
    objects = FullCarModelManager()

    class Meta:
        proxy = True


# http://atlk.publieditor.it/model_std_cars.xhtml?model_id=1004
class CarModelEquipment(DateModel, UserModel, OrderedModel, GenericModel):
    carmodel = models.ForeignKey(
        CarModel,
        related_name='equipments',
        verbose_name=_('model'),
    )
    description = models.CharField(
        max_length=255,
        verbose_name=_('description'),
    )
    note = models.TextField(
        null=True, blank=True,
        verbose_name=_('note'),
    )
    standard = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('standard'),
    )

    class Meta:
        verbose_name = _('Car Model Equipment')
        verbose_name_plural = _('Car Model Equipments')

    def __unicode__(self):
        return u"%s: %s" % (self.carmodel.name, self.description)

    def set_standard(self):
        cars = self.carmodel.cars.filter(status="ok")
        standard = True
        # se non ci sono auto con questa dotazione non è di serie
        if not cars:
            standard = False
        for car in cars:
            # se esiste almeno una versione senza dotazione non è di serie
            if not car.equipments.filter(carmodel_equipment=self):
                standard = False
                break
        self.standard = standard
        self.save()


class CarEquipment(DateModel, UserModel, OrderedModel, GenericModel):
    car = models.ForeignKey(
        "Car",
        related_name='equipments',
        verbose_name=_('car'),
    )
    carmodel_equipment = models.ForeignKey(
        CarModelEquipment,
        related_name='cars',
        verbose_name=_('equipment')
    )

    def get_car_id(self):
        if self.car:
            return self.car.id
        else:
            return u''

    def get_carmodel_name(self):
        if self.car:
            return self.car.model.name
        else:
            return u''

    def get_version(self):
        if self.car:
            return self.car.version
        else:
            return u''

    def get_carmodel_equipment_description(self):
        if self.carmodel_equipment:
            return self.carmodel_equipment.description
        else:
            return u''

    def get_carmodel_equipment_id(self):
        if self.carmodel_equipment:
            return u''.format(self.carmodel_equipment.id)
        else:
            return u''

    class Meta:
        verbose_name = _('Car Equipment')
        verbose_name_plural = _('Car Equipments')
        unique_together = (('car', 'carmodel_equipment'),)

    def __unicode__(self):
        return u"%s: %s" % (self.car.version, self.carmodel_equipment.description)

    @property
    def get_description(self):
        if self.carmodel_equipment:
            return u"{0}".format(self.carmodel_equipment.description)
        return u""

    def save(self, *args, **kw):
        carmodel_equipment = self.carmodel_equipment
        # se la dotazione è comune a tutte le versioni diventa di serie
        if self.car.status == "ok":
            carmodel_equipment.set_standard()
        super(CarEquipment, self).save(*args, **kw)

    def delete(self, *args, **kw):
        carmodel_equipment = self.carmodel_equipment
        # se la dotazione è comune a tutte le versioni diventa di serie
        if self.car.status == "ok":
            carmodel_equipment.set_standard()
        super(CarEquipment, self).delete(*args, **kw)


class SalePeriod(DateModel, UserModel, StatusModel, GenericModel):
    name = models.CharField(
        max_length=255,
        verbose_name=_('name')
    )
    date_begin = models.DateField(
        db_index=True,
        verbose_name=_('date begin')
    )
    date_end = models.DateField(
        db_index=True,
        verbose_name=_('date end')
    )

    class Meta:
        verbose_name = _('Sale Period')
        verbose_name_plural = _('Sale Periods')
        ordering = ('-date_begin',)
        index_together = (('date_begin', 'date_end'),)

    def __unicode__(self):
        return u"%s" % self.name


class CarModelSale(DateModel, UserModel, OrderedModel, GenericModel):
    # Unrae Sales
    carmodels = models.ManyToManyField(
        CarModel,
        blank=True, null=True,
        related_name='sales',
        verbose_name=_('models'),
        limit_choices_to={'status__in': ("online", "runout")}
    )
    period = models.ForeignKey(
        SalePeriod,
        related_name='sales',
        verbose_name=_('period')
    )
    sold = models.IntegerField(
        verbose_name=_('sold')
    )

    class Meta:
        verbose_name = _('Model Sale Data')
        verbose_name_plural = _('Model Sale Data')

    def __unicode__(self):
        return u"%s: %s" % (self.period, self.sold)

    @property
    def get_period(self):
        if self.period:
            return u"da: %s a: %s" % (
                self.period.date_begin.strftime("%d %b %Y"), self.period.date_end.strftime("%d %b %Y")
            )
        return u""

    def get_model_list(self):
        model_list = []
        for model in self.carmodels.all():
            model_list.append(model)
        return model_list
    get_model_list.short_description = _('models')

    def get_string_models(self):
        model_list = []
        for model in self.carmodels.all():
            model_list.append(model.name)
        return ", ".join(model_list)
    get_string_models.short_description = _('models')

    def get_brand(self):
        models = self.carmodels.all()
        if models:
            return models[0].brand
        return None
    get_brand.short_description = _('brand')
    get_brand.admin_order_field = 'carmodels__brand__name'

    def get_brand_id(self):
        if self.get_brand():
            return u"{0}".format(self.get_brand().id)
        else:
            return u''

    def get_brand_name(self):
        if self.get_brand():
            return u"{0}".format(self.get_brand().name)
        else:
            return u''

    def get_model_list_to_string(self):
        models_list = []
        for carmodel in self.get_model_list():
            models_list.append(u'[id:{0} name:{1}]'.format(carmodel.id, carmodel.name))
        return u' '.join(models_list)


class UNRAECarModel(DateModel, UserModel, OrderedModel, GenericModel):
    carmodels = models.ManyToManyField(
        CarModel,
        blank=True, null=True,
        related_name='unrae_carmodels',
        verbose_name=_('publieditor car models')
    )
    brand = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_('brand')
    )
    carmodel = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_('model')
    )
    design = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_('design')
    )

    class Meta:
        verbose_name = _('UNRAE Model')
        verbose_name_plural = _('UNRAE Models')
        index_together = (
            ('brand', 'carmodel', 'design'),
        )

    def __unicode__(self):
        return u"%s %s" % (self.brand, self.carmodel)

    def is_associated(self):
        if self.carmodels.all():
            return True
        return False
    is_associated.boolean = True

    def get_model_list(self):
        model_list = []
        for model in self.carmodels.all():
            model_list.append(model)
        return model_list
    get_model_list.short_description = _('models')

    def get_string_models(self):
        model_list = []
        for model in self.carmodels.all():
            model_list.append(model.name)
        return ", ".join(model_list)
    get_string_models.short_description = _('models')


class UNRAECarModelSale(DateModel, UserModel, OrderedModel, GenericModel):
    unrae_carmodel = models.ForeignKey(
        UNRAECarModel,
        related_name='unrare_sales',
        verbose_name=_('unrae car model')
    )
    year = models.IntegerField(
        db_index=True,
        verbose_name=_('year')
    )
    month = models.IntegerField(
        db_index=True,
        verbose_name=_('month')
    )
    sold = models.IntegerField(
        db_index=True,
        verbose_name=_('sold')
    )

    class Meta:
        verbose_name = _('UNRAE Model Sale Data')
        verbose_name_plural = _('UNRAE Model Sale Data')
        unique_together = (('unrae_carmodel', 'year', 'month'),)
        index_together = (
            ('year', 'month'),
            ('year', 'month', 'sold'),
            ('unrae_carmodel', 'sold'),
            ('unrae_carmodel', 'year', 'month'),
        )

    def __unicode__(self):
        return u"%s %s %s - %s/%s: %s" % (
            self.unrae_carmodel.brand,
            self.unrae_carmodel.carmodel,
            self.unrae_carmodel.design,
            self.month, self.year, self.sold
        )


class CorporateBrandLink(DateModel, UserModel, OrderedModel, GenericModel):
    brand = models.ForeignKey(
        Brand,
        related_name='corporate_links',
        verbose_name=_('brand'),
    )
    corporate = models.ForeignKey(
        Corporate,
        related_name='brand_links',
        verbose_name=_('corporate')
    )
    kind = models.CharField(
        max_length=3,
        db_index=True,
        choices=settings.PUBLIEDITOR_CORPORATEBRANDLINKS_CHOICES,
        verbose_name=_('kind'),
    )
    start_date = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('start date'),
    )
    end_date = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('end date'),
    )

    class Meta:
        verbose_name = _('Corporate Brand Link')
        verbose_name_plural = _('Corporate Brand Links')
        ordering = ('ordering', 'corporate__name', 'brand__name')
        index_together = (('ordering', 'corporate', 'brand'),)
        unique_together = (('corporate', 'brand'),)

    def __unicode__(self):
        return u"%s" % self.kind


class Car(DateModel, UserModel, OrderedModel, GenericModel):
    """
    versioni
    """

    status = models.CharField(
        max_length=7,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_STATUS_CHOICES,
        verbose_name=_('status'),
    )
    infocar = models.IntegerField(
        blank=True, null=True,
        verbose_name=_('info car'),
    )
    corporate = models.ForeignKey(
        Corporate,
        related_name='cars',
        blank=True, null=True,
        verbose_name=_('corporate'),
    )
    brand = models.ForeignKey(
        Brand,
        related_name='cars',
        blank=True, null=True,
        verbose_name=_('brand'),
    )
    model = models.ForeignKey(
        CarModel,
        related_name="cars",
        null=True, blank=True,
        verbose_name=_('model'),
    )
    motor = models.CharField(
        max_length=64,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('motor'),
    )
    version = models.CharField(
        max_length=64,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('version'),
    )
    price = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('price'),
    )
    shape = models.CharField(
        max_length=13,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_SHAPE_CHOICES,
        verbose_name=_('shape'),
    )
    target = models.CharField(
        max_length=9,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_TARGET_CHOICES,
        verbose_name=_('target'),
    )
    seat = models.CharField(
        max_length=3,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_SEAT_CHOICES,
        verbose_name=_('seat'),
    )
    engine = models.CharField(
        max_length=4,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_ENGINE_CHOICES,
        verbose_name=_('engine'),
    )
    layout = models.CharField(
        max_length=32,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_LAYOUT_CHOICES,
        blank=True, null=True,
        verbose_name=_('layout'),
    )
    turbo = models.CharField(
        max_length=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('turbo'),
    )
    fuel = models.CharField(
        max_length=8,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_FUEL_CHOICES,
        blank=True, null=True,
        verbose_name=_('fuel'),
    )
    traction = models.CharField(
        max_length=3,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_TRACTION_CHOICES,
        verbose_name=_('traction'),
    )
    gear = models.CharField(
        max_length=5,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_GEAR_CHOICES,
        verbose_name=_('gear'),
    )
    agear = models.CharField(
        max_length=1,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_AGEAR_CHOICES,
        verbose_name=_('agear'),
    )
    pollution = models.CharField(
        max_length=3,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_POLLUTION_CHOICES,
        verbose_name=_('pollution'),
    )
    engine_loc = models.CharField(
        max_length=3,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_ENGINE_LOC_CHOICES,
        verbose_name=_('engine loc'),
    )
    doors = models.CharField(
        max_length=1,
        blank=True, null=True,
        db_index=True,
        choices=settings.PUBLIEDITOR_CAR_DOORS_CHOICES,
        verbose_name=_('doors'),
    )
    year_first = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('first year restyling'),
    )
    year = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('year'),
    )
    month = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('month'),
    )
    luggage = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('luggage'),
    )
    luggage_max = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('luggage max'),
    )
    tank = models.DecimalField(
        max_digits=8, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('tank'),
    )
    weight = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('weight'),
    )
    length = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('length'),
    )
    width = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('width'),
    )
    height = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('height'),
    )
    displ = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('displacement'),
    )
    power_cv = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('power cv'),
    )
    power_kw = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('power kw'),
    )
    power_rpm = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('power rpm'),
    )
    torque_max = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('torque max'),
    )
    torque_rpm = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('torque rpm'),
    )
    speed = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('speed'),
    )
    acc = models.DecimalField(
        max_digits=6, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('acceleration'),
    )
    co2 = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('co2'),
    )
    tow = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('tow'),
    )
    cons_urban = models.DecimalField(
        max_digits=8, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('urban consumption'),
    )
    cons_mix = models.DecimalField(
        max_digits=8, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('mix consumption'),
    )
    cons_extra = models.DecimalField(
        max_digits=8, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('extra consumption'),
    )
    kml_mix = models.DecimalField(
        max_digits=8, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('kml mix'),
    )
    driving = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('driving'),
    )
    # TODO: deve diventare boolean true se co2 <= 120 else false
    eco = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('eco'),
    )
    area_c = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('area c'),
    )
    power_spec = models.DecimalField(
        max_digits=8, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('power spec'),
    )
    electric_average_mileage = models.DecimalField(
        max_digits=8, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('electric average mileage (km/kWh)'),
    )
    power_weight = models.DecimalField(
        max_digits=8, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('power weight'),
    )
    proxymity = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('proximity'),
    )
    price_update = models.DateField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('price update'),
    )
    field_src = models.CharField(
        max_length=1,
        db_index=True,
        blank=True, null=True,
        choices=settings.PUBLIEDITOR_CAR_FIELD_SRC_CHOICES,
        verbose_name=_('field src'),
    )
    shape_old = models.CharField(
        max_length=22,
        blank=True, null=True,
        choices=settings.PUBLIEDITOR_CAR_FIELD_SHAPE_OLD_CHOICES,
        verbose_name=_('old shape'),
    )
    new_date = models.DateTimeField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('new date time'),
    )
    mod_date = models.DateTimeField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('date time modify'),
    )
    standard_num = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('standard num'),
    )
    off_year = models.IntegerField(
        null=True, blank=True,
        verbose_name=_('off year'),
    )
    off_month = models.IntegerField(
        null=True, blank=True,
        verbose_name=_('off month'),
    )
    slug = models.SlugField(
        max_length=100,
        unique=True,
        db_index=True,
        verbose_name=_("slug"),
    )
    turning_circle = models.DecimalField(
        max_digits=8, decimal_places=1,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('turning circle'),
    )
    wheelbase = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('whellbase'),
    )
    warranty = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('warranty (years)'),
    )
    ecobonus = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('ecobonus'),
    )
    ecobonus_perc = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('ecobonus (percentuale)'),
    )
    superbollo = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('superbollo'),
    )
    superbollo_perc = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('superbollo (percentuale)'),
    )
    number_of_engines = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('number of engines'),
    )
    combustion_engine_power_cv = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('combustion engine power in CV'),
        help_text=_('CV'),
    )
    combustion_engine_power_kw = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('combustion engine power in kW'),
        help_text=_('kW'),
    )
    combustion_engine_torque_Nm = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('combustion engine torque in Nm'),
        help_text=_('Nm'),
    )
    electric_engine_power_cv = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('electric engine power in CV'),
        help_text=_('CV'),
    )
    electric_engine_power_kw = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('electric engine power in kW'),
        help_text=_('kW'),
    )
    electric_engine_torque_Nm = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('electric engine torque in Nm'),
        help_text=_('Nm'),
    )
    total_power_cv = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('total power in CV'),
        help_text=_('CV'),
    )
    total_power_kw = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('total power in kW'),
        help_text=_('kW'),
    )
    total_torque_Nm = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('total torque in Nm'),
        help_text=_('Nm'),
    )
    batteries_capacity = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('batteries capacity'),
        help_text=_('kWh'),
    )
    usable_capacity = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('usable capacity'),
        help_text=_('kWh'),
    )
    on_board_battery_charger = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('on board battery charger'),
        help_text=_('kW'),
    )
    average_electric_consumption = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('average electric consumption'),
        help_text=_('kWh/100 km'),
    )
    ac_charging_time_at_3_7_kw = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('ac charging time at 3.7 kW'),
        help_text=_('hours, minutes'),
    )
    ac_charging_time_at_7_5_kw = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('ac charging time at 7.5 kW'),
        help_text=_('hours, minutes'),
    )
    ac_charging_time_at_11_kw = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('ac charging time at 11 kW'),
        help_text=_('hours, minutes'),
    )
    ac_charging_time_at_22_kw = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('ac charging time at 22 kW'),
        help_text=_('hours, minutes'),
    )
    dc_charging_time_at_150_kw = models.DecimalField(
        max_digits=8, decimal_places=2,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('dc charging time at 150 kW'),
        help_text=_('hours, minutes'),
    )
    electric_autonomy_in_km = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('electric autonomy in km'),
        help_text=_('km'),
    )
    total_autonomy_in_km = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('total autonomy in km'),
        help_text=_('km'),
    )
    top_speed_in_EV = models.IntegerField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('top speed in EV'),
        help_text=_('km/h'),
    )

    class Meta:
        verbose_name = _('Car')
        verbose_name_plural = _('Cars')
        ordering = ('ordering', 'version')
        index_together = (
            ("ordering", "version"),
            # usati per le classifiche
            ("status", "acc"),
            ("status", "co2", "shape", "length"),
            ("status", "co2", "target"),
            ("status", "co2", "shape"),
            ("status", "cons_mix", "fuel"),
            ("status", "power_cv", "target"),
            ("status", "power_spec", "target", "engine"),
            ("status", "power_weight"),
        )

    def __unicode__(self):
        return u"%s %s - %s - %s  (%s) %s" % (
            self.year_first, self.model.brand,
            self.model, self.motor, self.version, self.status
        )

    def save(self, *args, **kwargs):
        self.set_driving()
        self.set_area_c()
        self.set_kml_mix()
        self.set_eco()
        self.set_power_spec()
        self.set_electric_average_mileage()
        self.set_power_weight()
        self.set_agear()
        self.set_ecobonus()
        self.set_ecobonus_perc()
        self.set_superbollo()
        self.set_superbollo_perc()
        super(Car, self).save(*args, **kwargs)

    # Class methods used by the create_config_service Command

    @classmethod
    def get_active_cars(cls):
        return cls.objects.filter(
            status='ok',
            model__status__in=('online', 'runout'),
            brand__status='active'
        )

    @classmethod
    def get_price_range(cls):
        price_range = cls.get_active_cars().aggregate(Max('price'), Min('price'))
        return [price_range['price__min'], price_range['price__max']]

    @classmethod
    def get_year_range(cls):
        year_range = cls.get_active_cars().aggregate(Max('year'), Min('year'))
        return [year_range['year__min'], year_range['year__max']]

    @classmethod
    def get_luggage_range(cls):
        luggage_range = cls.get_active_cars().aggregate(Max('luggage'), Min('luggage'))
        return [luggage_range['luggage__min'], luggage_range['luggage__max']]

    @classmethod
    def get_tank_range(cls):
        tank_range = cls.get_active_cars().aggregate(Max('tank'), Min('tank'))
        return [math.trunc(tank_range['tank__min']), int(math.ceil(tank_range['tank__max']))]

    @classmethod
    def get_weight_range(cls):
        weight_range = cls.get_active_cars().aggregate(Max('weight'), Min('weight'))
        return [weight_range['weight__min'], weight_range['weight__max']]

    @classmethod
    def get_width_range(cls):
        width_range = cls.get_active_cars().aggregate(Max('width'), Min('width'))
        return [width_range['width__min'], width_range['width__max']]

    @classmethod
    def get_height_range(cls):
        height_range = cls.get_active_cars().aggregate(Max('height'), Min('height'))
        return [height_range['height__min'], height_range['height__max']]

    @classmethod
    def get_length_range(cls):
        length_range = cls.get_active_cars().aggregate(Max('length'), Min('length'))
        return [length_range['length__min'], length_range['length__max']]

    @classmethod
    def get_displ_range(cls):
        displ_range = cls.get_active_cars().aggregate(Max('displ'), Min('displ'))
        return [displ_range['displ__min'], displ_range['displ__max']]

    @classmethod
    def get_power_cv_range(cls):
        power_cv_range = cls.get_active_cars().aggregate(Max('power_cv'), Min('power_cv'))
        return [power_cv_range['power_cv__min'], power_cv_range['power_cv__max']]

    @classmethod
    def get_total_power_cv_range(cls):
        total_power_cv_range = cls.get_active_cars().aggregate(Max('total_power_cv'), Min('total_power_cv'))
        return [total_power_cv_range['total_power_cv__min'], total_power_cv_range['total_power_cv__max']]

    @classmethod
    def get_torque_max_range(cls):
        torque_max_range = cls.get_active_cars().aggregate(Max('torque_max'), Min('torque_max'))
        return [torque_max_range['torque_max__min'], torque_max_range['torque_max__max']]

    @classmethod
    def get_total_torque_Nm_range(cls):
        total_torque_Nm_range = cls.get_active_cars().aggregate(Max('total_torque_Nm'), Min('total_torque_Nm'))
        return [total_torque_Nm_range['total_torque_Nm__min'], total_torque_Nm_range['total_torque_Nm__max']]

    @classmethod
    def get_speed_range(cls):
        speed_range = cls.get_active_cars().aggregate(Max('speed'), Min('speed'))
        return [speed_range['speed__min'], speed_range['speed__max']]

    @classmethod
    def get_acc_range(cls):
        acc_range = cls.get_active_cars().aggregate(Max('acc'), Min('acc'))
        return [math.trunc((acc_range['acc__min'])), int(math.ceil(acc_range['acc__max']))]

    @classmethod
    def get_co2_range(cls):
        co2_range = cls.get_active_cars().aggregate(Max('co2'), Min('co2'))
        return [co2_range['co2__min'], co2_range['co2__max']]

    @classmethod
    def get_cons_mix_range(cls):
        cons_mix_range = cls.get_active_cars().aggregate(Max('cons_mix'), Min('cons_mix'))
        return [math.trunc(cons_mix_range['cons_mix__min']), int(math.ceil(cons_mix_range['cons_mix__max']))]

    @classmethod
    def get_average_electric_consumption_range(cls):
        average_electric_consumption_range = cls.get_active_cars().aggregate(Max('average_electric_consumption'), Min('average_electric_consumption'))
        return [
            math.trunc(average_electric_consumption_range['average_electric_consumption__min'] or 0),
            int(math.ceil(average_electric_consumption_range['average_electric_consumption__max'] or 0))
        ]

    @classmethod
    def get_electric_autonomy_in_km_range(cls):
        electric_autonomy_in_km_range = cls.get_active_cars().aggregate(Max('electric_autonomy_in_km'), Min('electric_autonomy_in_km'))
        return [
            math.trunc(electric_autonomy_in_km_range['electric_autonomy_in_km__min'] or 0),
            int(math.ceil(electric_autonomy_in_km_range['electric_autonomy_in_km__max'] or 0))
        ]

    @classmethod
    def get_kml_mix_range(cls):
        kml_mix_range = cls.get_active_cars().aggregate(Max('kml_mix'), Min('kml_mix'))
        return [math.trunc((kml_mix_range['kml_mix__min'])), int(math.ceil(kml_mix_range['kml_mix__max']))]

    @classmethod
    def get_electric_engine_power_kw_range(cls):
        electric_engine_power_kw_range = cls.get_active_cars().aggregate(Max('electric_engine_power_kw'), Min('electric_engine_power_kw'))
        return [
            int(electric_engine_power_kw_range['electric_engine_power_kw__min'] or 0),
            int(electric_engine_power_kw_range['electric_engine_power_kw__max'] or 0)
        ]

    @classmethod
    def get_shape_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('shape', flat=True).distinct('shape').order_by('shape')
        choices_settings = settings.PUBLIEDITOR_CAR_SHAPE_CHOICES
        choices = []
        thumb_dict = settings.PUBLIEDITOR_CAR_SHAPE_CHOICES_THUMB_DICT
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=label,
                    thumbnail=thumb_dict[label]
                )
                choices.append(choice_dict)
        return choices

    @classmethod
    def get_target_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('target', flat=True).distinct('target').order_by('target')
        choices_settings = settings.PUBLIEDITOR_CAR_TARGET_CHOICES
        choices = []
        thumb_dict = settings.PUBLIEDITOR_CAR_TARGET_CHOICES_THUMB_DICT
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=label,
                    thumbnail=thumb_dict[label]
                )
                choices.append(choice_dict)
        return choices

    @classmethod
    def get_seat_choices(cls):
        seat_choices_in_db = cls.get_active_cars().values_list('seat', flat=True).distinct('seat').order_by('seat')
        seat_settings = settings.PUBLIEDITOR_CAR_SEAT_CHOICES
        seat_choices = []
        for value, label in seat_settings:
            if value in seat_choices_in_db:
                seat_dict = dict(
                    value=value,
                    label=value,
                )
                seat_choices.append(seat_dict)
        return seat_choices

    @classmethod
    def get_engine_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('engine', flat=True).distinct('engine').order_by('engine')
        choices_settings = settings.PUBLIEDITOR_CAR_ENGINE_CHOICES
        choices = []
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=label,
                )
                choices.append(choice_dict)
        return choices

    @classmethod
    def get_layout_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('layout', flat=True).distinct('layout').order_by('layout')
        choices_settings = settings.PUBLIEDITOR_CAR_LAYOUT_CHOICES
        choices = []
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=label,
                )
                choices.append(choice_dict)
        return choices

    @classmethod
    def get_fuel_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('fuel', flat=True).distinct('fuel').order_by('fuel')
        choices_settings = settings.PUBLIEDITOR_CAR_FUEL_CHOICES
        choices = []
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=label,
                )
                choices.append(choice_dict)
        return choices

    @classmethod
    def get_traction_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('traction', flat=True).distinct('traction').order_by('traction')
        choices_settings = settings.PUBLIEDITOR_CAR_TRACTION_CHOICES
        choices = []
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=label,
                )
                choices.append(choice_dict)
        return choices

    @classmethod
    def get_agear_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('agear', flat=True).distinct('agear').order_by('agear')
        choices_settings = settings.PUBLIEDITOR_CAR_AGEAR_CHOICES
        choices = []
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=label,
                )
                choices.append(choice_dict)
        return choices

    @classmethod
    def get_pollution_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('pollution', flat=True).distinct('pollution').order_by('pollution')
        choices_settings = settings.PUBLIEDITOR_CAR_POLLUTION_CHOICES
        choices = []
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=label,
                )
                choices.append(choice_dict)
        return choices

    @classmethod
    def get_engine_loc_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('engine_loc', flat=True).distinct('engine_loc').order_by('engine_loc')
        choices_settings = settings.PUBLIEDITOR_CAR_ENGINE_LOC_CHOICES_LABELS
        choices = []
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=label,
                )
            choices.append(choice_dict)
        return choices

    @classmethod
    def get_door_choices(cls):
        choices_in_db = cls.get_active_cars().values_list('doors', flat=True).distinct('doors').order_by('doors')
        choices_settings = settings.PUBLIEDITOR_CAR_DOORS_CHOICES
        choices = []
        for value, label in choices_settings:
            if value in choices_in_db:
                choice_dict = dict(
                    value=value,
                    label=value,
                )
                choices.append(choice_dict)
        return choices

    def get_average_electric_consumption(self):
        average_electric_consumption = None
        if self.average_electric_consumption != None:
            average_electric_consumption = self.average_electric_consumption
        elif self.batteries_capacity and self.electric_autonomy_in_km:
            average_electric_consumption = self.batteries_capacity/self.electric_autonomy_in_km*100
        return average_electric_consumption

    def get_total_autonomy_in_km(self):
        total_autonomy_in_km = None
        if self.total_autonomy_in_km != None:
            total_autonomy_in_km = self.total_autonomy_in_km
        elif self.tank and self.cons_mix:
            total_autonomy_in_km = self.tank/self.cons_mix*100
        return total_autonomy_in_km

    def set_driving(self):
        try:
            if (
                (self.power_kw * 1000) / (self.weight - 75) <= 55
                and self.power_kw <= 70
            ):
                self.driving = True
            else:
                self.driving = False
        except:
            self.driving = False

    def set_area_c(self):
        try:
            if (
                self.engine == "E" or self.engine == "ET" or self.engine == "TE" or
                (self.engine == "T" and (self.fuel == "B/GPL" or self.fuel == "B/M"))
            ):
                self.area_c = True
            else:
                self.area_c = False
        except:
            self.area_c = False

    def set_kml_mix(self):
        try:
            if self.cons_mix and self.cons_mix > 0:
                self.kml_mix = 100 / self.cons_mix
        except:
            pass

    def set_eco(self):
        try:
            if self.co2 and self.co2 <= 120:
                self.eco = 1
            else:
                self.eco = 0
        except:
            pass

    def set_power_spec(self):
        try:
            if self.displ and (self.displ / 1000) > 0:
                if self.engine in settings.PUBLIEDITOR_ELECTRIC_ENGINE_LIST:
                    self.power_spec = None
                else:
                    self.power_spec = self.power_cv / (self.displ / 1000)
        except:
            pass

    def set_electric_average_mileage(self):
        try:
            if self.engine in settings.PUBLIEDITOR_ELECTRIC_ENGINE_LIST and self.get_average_electric_consumption():
                self.electric_average_mileage = 100 / self.get_average_electric_consumption()
            else:
                self.electric_average_mileage = None
        except:
            pass

    def set_power_weight(self):
        try:
            if self.power_cv and self.power_cv > 0:
                self.power_weight = self.weight / self.power_cv
        except:
            pass

    def set_agear(self):
        if self.gear and self.gear.lower().startswith("m"):
            self.agear = "M"
        self.agear = "A"

    def similar_cars(self):
        """
        restituisce la lista delle auto simili con parametro di ugualianza
        """
        similar_cars = Car.objects.filter(
            status='ok', model__status__in=('online', 'runout'),
            shape=self.shape,
            driving=self.driving, price__isnull=False, length__isnull=False
        ).exclude(id=self.id)
        car_list = []
        for car in similar_cars:
            try:
                price_calc = (1 - abs(self.price - car.price) / car.price)
                length_calc = (1 - abs(self.length - car.length) / car.length)
                power_cv_calc = (1 - abs(self.power_cv - car.power_cv) / car.power_cv)
                if price_calc >= 0.7 and length_calc >= 0.8 and power_cv_calc >= 0.8:
                    xsum = price_calc * length_calc * power_cv_calc
                    car_list.append({
                        'car': car,
                        'xsum': xsum,
                        'price_calc': price_calc,
                        'length_calc': length_calc,
                        'power_cv_calc': power_cv_calc
                    })
            except:
                continue
            car_list = sorted(car_list, key=itemgetter('xsum'), reverse=True)
        return car_list

    def similar_cars_same_model(self):
        """
        restituisce la lista delle auto simili dello stesso modello con parametro di ugualianza
        """
        similar_cars = Car.objects.filter(
            status='ok', model__status__in=('online', 'runout'), shape=self.shape,
            driving=self.driving, price__isnull=False, length__isnull=False,
            model=self.model
        ).exclude(id=self.id)
        car_list = []
        for car in similar_cars:
            try:
                price_calc = (1 - abs(self.price - car.price) / car.price)
                length_calc = (1 - abs(self.length - car.length) / car.length)
                power_cv_calc = (1 - abs(self.power_cv - car.power_cv) / car.power_cv)
                if price_calc >= 0.7 and length_calc >= 0.8 and power_cv_calc >= 0.8:
                    xsum = price_calc * length_calc * power_cv_calc
                    car_list.append({
                        'car': car,
                        'xsum': xsum,
                        'price_calc': price_calc,
                        'length_calc': length_calc,
                        'power_cv_calc': power_cv_calc
                    })
            except:
                continue
        car_list = sorted(car_list, key=itemgetter('xsum'), reverse=True)
        return car_list

    def similar_cars_other_models(self):
        """
        restituisce la lista delle auto simili di altri modelli con parametro di ugualianza
        """
        # since a for loop over all the similar car models filtered by shapes and driving takes too much time  we need
        # a more restrictive filter. we use a cutoff value for min_length.
        # As before, if min length is not defined all the check gives errors, so we return an empty model_list
        min_length_cutoff_value = 15
        car_list = []

        try:
            similar_cars = Car.objects.filter(
                status='ok', model__status__in=('online', 'runout'), shape=self.shape,
                driving=self.driving, price__isnull=False, length__isnull=False
            ).filter(
                length__gte=self.length-min_length_cutoff_value, length__lte=self.length+min_length_cutoff_value
            ).exclude(
                model=self.model
            )

            for car in similar_cars:
                try:
                    price_calc = (1 - abs(self.price - car.price) / car.price)
                    length_calc = (1 - abs(self.length - car.length) / car.length)
                    power_cv_calc = (1 - abs(self.power_cv - car.power_cv) / car.power_cv)
                    if price_calc >= 0.7 and length_calc >= 0.8 and power_cv_calc >= 0.8:
                        xsum = price_calc * length_calc * power_cv_calc
                        car_list.append({
                            'car': car,
                            'xsum': xsum,
                            'price_calc': price_calc,
                            'length_calc': length_calc,
                            'power_cv_calc': power_cv_calc
                        })
                except:
                    continue
        except:
            pass
        car_list = sorted(car_list, key=itemgetter('xsum'), reverse=True)
        return car_list

    def get_ac_charging_time_at_3_7_kw_display(self):
        try:
            separated_num = '{0:.2f}'.format(self.ac_charging_time_at_3_7_kw).split('.')
            return '{}h{}m'.format(separated_num[0], separated_num[1])
        except:
            return ''

    def get_ac_charging_time_at_7_5_kw_display(self):
        try:
            separated_num = '{0:.2f}'.format(self.ac_charging_time_at_7_5_kw).split('.')
            return '{}h{}m'.format(separated_num[0], separated_num[1])
        except:
            return ''

    def get_ac_charging_time_at_11_kw_display(self):
        try:
            separated_num = '{0:.2f}'.format(self.ac_charging_time_at_11_kw).split('.')
            return '{}h{}m'.format(separated_num[0], separated_num[1])
        except:
            return ''
        
    def get_ac_charging_time_at_22_kw_display(self):
        try:
            separated_num = '{0:.2f}'.format(self.ac_charging_time_at_22_kw).split('.')
            return '{}h{}m'.format(separated_num[0], separated_num[1])
        except:
            return ''
        
    def get_dc_charging_time_at_150_kw_display(self):
        try:
            separated_num = '{0:.2f}'.format(self.dc_charging_time_at_150_kw).split('.')
            return '{}h{}m'.format(separated_num[0], separated_num[1])
        except:
            return ''

    def get_autocomposed_text(self):
        template_txt = settings.PUBLIEDITOR_CAR_BASE_DESCRIPTION
        context_dict = {'car': self}
        context = Context(context_dict)
        template = loader.get_template_from_string(template_txt)
        ret = template.render(context)
        return ret

    def render_text(self):
        return self.get_autocomposed_text()

    def check_has_electric_engine(self):
        if self.engine in settings.PUBLIEDITOR_ELECTRIC_ENGINE_LIST:
            return True
        return False

    def check_has_hybrid_engine(self):
        if self.engine in settings.PUBLIEDITOR_HYBRID_ENGINE_LIST:
            return True
        return False

    def check_has_standard_engine(self):
        if self.engine in settings.PUBLIEDITOR_STANDARD_ENGINE_LIST:
            return True
        return False

    def get_standard_equipments(self):
        return self.equipments.all()

    def get_absolute_url(self):
        return reverse('content:car_detail', args=[self.brand.slug, self.model.slug, self.slug])

    def get_model_name(self):
        if self.model:
            return u'{0}'.format(self.model.name)
        else:
            return u''

    def get_model_status(self):
        if self.model:
            return u'{0}'.format(self.model.status)
        else:
            return u''

    def get_model_id(self):
        if self.model:
            return u'{0}'.format(self.model.id)
        else:
            return u''

    def get_brand_id(self):
        if self.brand:
            return u'{0}'.format(self.brand.id)
        else:
            return u''

    def get_brand_name(self):
        if self.brand:
            return u'{0}'.format(self.brand.name)
        else:
            return u''

    def get_corporate_id(self):
        if self.corporate:
            return u'{0}'.format(self.corporate.id)
        else:
            return u''

    def get_corporate_name(self):
        if self.corporate:
            return u'{0}'.format(self.corporate.name)
        else:
            return u''

    def get_neopatentati(self):
        if self.driving:
            return u'Yes'
        else:
            return u'No'

    def get_low_emission(self):
        if self.eco == 0:
            return u'No'
        else:
            return u'Yes'

    def get_area_c(self):
        if self.area_c:
            return u'Yes'
        else:
            return u'No'

    def get_price_update(self):
        if self.price_update:
            return self.price_update.strftime("%d-%m-%Y")
        else:
            return u''

    def get_new_date(self):
        if self.new_date:
            return self.new_date.strftime("%d-%m-%Y_%H:%M:%S")
        else:
            return u''

    def get_mod_date(self):
        if self.mod_date:
            return self.mod_date.strftime("%d-%m-%Y_%H:%M:%S")
        else:
            return u''

    def get_ecobonus(self):
        if self.co2 != None and self.price != None:
            if int(self.price) <= 42700 and int(self.co2) <= 20:
                return -5000
            if int(self.price) <= 54900 and 20 < int(self.co2) <= 60:
                return -4000
            if int(self.price) <= 42700 and 60 < int(self.co2) <= 135:
                return -2000
            if 160 < int(self.co2) <= 175:
                return 1100
            if 175 < int(self.co2) <= 200:
                return 1600
            if 200 < int(self.co2) <= 250:
                return 2000
            if int(self.co2) > 250:
                return 2500
        return 0

    def set_ecobonus(self):
        self.ecobonus = self.get_ecobonus()

    def get_ecobonus_perc(self):
        if self.price > 0:
            return self.get_ecobonus()/self.price*100
        return 0

    def set_ecobonus_perc(self):
        self.ecobonus_perc = self.get_ecobonus_perc()

    def get_superbollo(self):
        if self.engine in settings.PUBLIEDITOR_ELECTRIC_ENGINE_LIST:
            return 0
        elif self.combustion_engine_power_kw:
            if int(self.combustion_engine_power_kw) > 185:
                return (int(self.combustion_engine_power_kw) - 185) * 20
            else:
                return 0
        elif self.engine in settings.PUBLIEDITOR_STANDARD_ENGINE_LIST and self.total_power_kw:
            if int(self.total_power_kw) > 185:
                return (int(self.total_power_kw) - 185) * 20
            else:
                return 0
        else:
            return None

    def set_superbollo(self):
        self.superbollo = self.get_superbollo()

    def get_superbollo_perc(self):
        superbollo = self.get_superbollo()
        if self.price > 0 and superbollo is not None:
            return self.get_superbollo()/self.price*100
        return 0

    def set_superbollo_perc(self):
        self.superbollo_perc = self.get_superbollo_perc()

    @staticmethod
    def create_object_from_import(row, kw, j):
        from decimal import Decimal
        car_id = row.id
        cons_extra = row.cons_extra
        turning_circle = row.turning_circle
        wheelbase = row.wheelbase
        warranty = row.warranty
        try:
            car = Car.objects.get(id=car_id)
            if cons_extra:
                car.cons_extra = Decimal(cons_extra)
            if turning_circle:
                car.turning_circle = Decimal(turning_circle)
            if wheelbase:
                car.wheelbase = Decimal(wheelbase)
            if warranty:
                car.warranty = Decimal(warranty)
            car.save()
            return car, False
        except Car.DoesNotExist:
            print car_id


class Hit(DateModel, UserModel, OrderedModel, GenericModel):
    status = models.IntegerField(
        db_index=True,
        choices=settings.PUBLIEDITOR_HIT_STATUS_CHOICES,
        default=0,
        verbose_name=_('status'),
    )
    title = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_('title'),
    )
    slug = models.SlugField(
        max_length=255,
        db_index=True,
        unique=True,
        verbose_name=_('slug'),
    )
    unit = models.CharField(
        max_length=15,
        db_index=True,
        choices=settings.PUBLIEDITOR_HIT_UNIT_CHOICES,
        blank=True, null=True,
        verbose_name=_("unit"),
    )
    description = models.TextField(
        verbose_name=_('description'),
    )
    field = models.CharField(
        max_length=255,
        verbose_name=_('field'),
    )
    ordering_field = models.CharField(
        max_length=1,
        choices=settings.PUBLIEDITOR_HIT_ORDERING_FIELD_CHOICES,
        default='d',
        verbose_name=_('ordering field'),
    )
    limit = models.IntegerField(
        default=50,
        verbose_name=_('limit'),
    )
    filters = models.TextField(
        null=True, blank=True,
        verbose_name=_('filters'),
    )
    operator = models.CharField(
        max_length=25,
        choices=settings.PUBLIEDITOR_HIT_OPERATOR_CHOICES,
        verbose_name=_('operator'),
    )
    table = models.CharField(
        max_length=25,
        choices=settings.PUBLIEDITOR_HIT_CLASS,
        verbose_name=_('table'),
    )

    def get_first_photo_url(self):
        try:
            model = self.get_first_item_stat()['model']
        except KeyError:
            return ''
        return model.get_first_photo_url()

    def get_first_photo_url_medium(self):
        try:
            model = self.get_first_item_stat()['model']
        except KeyError:
            return ''
        return model.get_photo_url_medium()

    def get_first_photo_url_wide_large(self):
        try:
            model = self.get_first_item_stat()['model']
        except KeyError:
            return ''
        return model.get_photo_url_wide_large()

    def get_first_video_url(self):
        try:
            model = self.get_first_item_stat()['model']
        except KeyError:
            return ''
        return model.get_first_video_url()

    def get_stat(self, limit=None):
        """
        Return a list of lists
        """
        # se il valore da restituire è il primo non occorre prendere i successivi
        filters = json.loads(self.filters) if self.filters else {}

        if self.table == "version":
            return self.get_stat_from_version(filters, limit)
        elif self.table == "sales":
            return self.get_stat_from_sales(filters, limit)
            
    def get_stat_from_version(self, filters, limit):

        from . import utils

        coeff = 1
        limit = limit or self.limit
        if limit > 1:
            coeff = 10
        filters.update({'%s__isnull' % self.field: False})
        # ascending / descendng
        order_by_field = "%s" % self.field if self.ordering_field == 'a' else "-%s" % self.field
        qs = Car.objects.filter(status='ok', **filters).exclude(model__status="retired").order_by(
            order_by_field, 'brand__name', 'model__name'
        ).select_related(
            'model', 'model__brand'
        )[0:limit*coeff]
        output = utils.get_stat(
            qs=qs, limit=limit, op=self.operator,
            attr_name=self.field, order=self.ordering_field,
        )
        return output

    def get_stat_from_sales(self, filters, limit=10):
        """
        Return sales' stats 
        
        :arg filters: filter from CarModelSale class. i.e.: {'carmodel__shape' : 'SUV'}
        :arg 
        """
        from . import utils

        if SalePeriod.objects.all().count():
            last_period = SalePeriod.objects.all().order_by('-date_end')[0]
            order_by_field = 'sold' if self.ordering_field == 'a' else '-sold'
            qs = CarModelSale.objects.filter(**filters).filter(
                carmodels__status__in=("online", "runout"),
                period=last_period
            ).order_by(
                order_by_field, 'id', 'carmodels__brand__name', 'carmodels__name'
            ).distinct('sold', 'id').select_related(
                'carmodels', 'carmodels__brand'
            )[0:limit]
            output = utils.get_stat_sold(
                qs=qs, limit=limit,
            )
        else:
            output = []
        return output

    def get_first_item_stat(self):
        stats = self.get_stat(limit=1)
        if stats:
            first = stats[0]
            return {'model': first['models'][0], 'value': first['value']}
        else:
            return {}

    def get_top_three_items_stat(self):
        return self.get_stat(limit=3)

    def get_last_sale_period(self):
        return SalePeriod.objects.filter(status=1).order_by('-date_end')[0]

    def get_absolute_url(self):
        return reverse('content:hit_detail', args=[self.slug])

    class Meta:
        verbose_name = _('Hit')
        verbose_name_plural = _('Hits')
        ordering = ('ordering', 'title')
        index_together = (("ordering", "title"),)

    def __unicode__(self):
        return u"%s - %s %s %s" % (
            self.get_status_display(), self.title, self.field,
            self.get_ordering_field_display()
        )


class Stream(DateModel, UserModel, OrderedModel, GenericModel):
    is_filter = models.BooleanField(
        default=True,
        verbose_name=_("is filter")
    )
    name = models.CharField(
        max_length=32, unique=True,
        db_index=True,
        verbose_name=_('name'),
    )
    info = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('info'),
    )
    status = models.CharField(
        max_length=16,
        db_index=True,
        blank=True, null=True,
        choices=settings.PUBLIEDITOR_STREAM_STATUS_CHOICES,
        verbose_name=_('status')
    )
    language = models.ForeignKey(
        Language,
        related_name="streams",
        verbose_name=_('language'),
    )

    class Meta:
        verbose_name = _('Stream')
        verbose_name_plural = _('Streams')
        ordering = ('ordering', 'name',)
        index_together = (("ordering", "name"),)

    def __unicode__(self):
        return u"%s" % self.name


class Layout(DateModel, UserModel, StatusModel, OrderedModel, GenericModel):
    name = models.CharField(
        max_length=32, unique=True,
        db_index=True,
        verbose_name=_('name'),
    )
    contexts = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('contexts')
    )  # laStampa, Mercedes-app, ecc.

    class Meta:
        verbose_name = _('Layout')
        verbose_name_plural = _('Layouts')
        ordering = ('ordering', 'name',)
        index_together = (("ordering", "name"),)

    def __unicode__(self):
        return u"%s" % self.name

    def get_table_name(self):
        return self.name.lower()


class Issue(DateModel, UserModel, OrderedModel, GenericModel):
    """
    Aggregatore di News
    """
    def get_attachment_storage_path(self, filename):
        filename = force_unicode(filename)
        path = os.path.join(settings.MEDIA_URL, settings.PUBLIEDITOR_CONTENT_ISSUE_ATTACHMENT, filename)
        return path[1:]

    status = models.IntegerField(
        choices=settings.PUBLIEDITOR_ISSUE_STATUS_CHOICES,
        db_index=True,
        verbose_name=_('status'),
    )
    stream = models.ForeignKey(
        Stream, related_name='issues',
        null=True, blank=True,
        verbose_name=_('stream')
    )
    # issue nel vecchio modello - numerazione progressiva
    # non annuale - unique
    # creare funzione per numerazione automatica
    number = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('number')
    )
    pub_date = models.DateTimeField(
        null=True, blank=True,
        db_index=True,
        verbose_name=_('pub date')
    )
    note = models.TextField(null=True, blank=True)
    # deve essere una funzione che ritorna True/False se ci sono attach
    # creare funzione che ritorna il nome del pdf
    # has_attach = models.IntegerField()

    attachment = models.FileField(
        _('File to attach'),
        null=True, blank=True,
        max_length=255,
        upload_to=get_attachment_storage_path

    )

    class Meta:
        verbose_name = _('Issue')
        verbose_name_plural = _('Issues')
        ordering = ('ordering', '-number',)
        index_together = (("stream", "number"),)

    def __unicode__(self):
        return u"id: %s - number: %s" % (self.id, self.number)

    def date_content_ordinal(self, ):
        from . import utils 
        return utils.format_date_with_ordinal(self.pub_date)
        
    def content_count(self):
        if self.stream.name == "AutolinkNews":
            autolinknews_stream = self.stream
            autolink_stream = Stream.objects.get(name="Autolink")
            return self.contents.filter(Q(stream=autolinknews_stream) | Q(stream=autolink_stream)).count()
        return self.contents.all().count()


class Sector(DateModel, UserModel, StatusModel, OrderedModel, GenericModel):
    parents = models.ManyToManyField(
        'self',
        related_name='children',
        blank=True, null=True,
        verbose_name=_('parent'),
    )
    name = models.CharField(
        max_length=64, unique=True,
        db_index=True,
        verbose_name=_('name'),
    )
    context = models.CharField(
        max_length=16,
        blank=True, null=True,
        verbose_name=_('context'),
    )  # choices: news
    key = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('key'),
    )
    # chiavi di ricerca
    keys = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('keys'),
    )
    old_name = models.CharField(
        max_length=64,
        blank=True, null=True,
        verbose_name=_('old name'),
    )
    query_keys = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('query keys'),
    )  # interi serve per ordinamento

    class Meta:
        verbose_name = _('Sector')
        verbose_name_plural = _('Sectors')
        ordering = ('ordering', 'name',)
        index_together = (("ordering", "name"),)

    def __unicode__(self):
        return u"%s" % self.name

    def get_parent_ids_list(self):
        id_list = []
        for parent in self.parents.all():
            id_list.append(parent.id)
        return id_list

    def get_parent_ids_string(self):
        id_list = self.get_parent_ids_list()
        return ','.join(id_list)

    def get_parent_names_list(self):
        name_list = []
        for parent in self.parents.all():
            name_list.append(parent.name)
        return name_list

    def get_parent_names_string(self):
        name_list = self.get_parent_names_list()
        return ','.join(name_list)

    get_parent_names_string.short_description = _('parents')


class Kind(DateModel, UserModel, StatusModel, OrderedModel, GenericModel):
    parents = models.ManyToManyField(
        'self',
        related_name='children',
        blank=True, null=True,
        verbose_name=_('parent'),
    )
    name = models.CharField(
        max_length=32,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('name'),
    )  # unique
    context = models.CharField(
        max_length=64,
        blank=True, null=True,
        verbose_name=_('context'),
    )  # usa il template news per generare admin
    key = models.CharField(
        max_length=64,
        blank=True, null=True,
        verbose_name=_('key'),
    )  # slug corrispondente su sito autolinknews
    old_name = models.CharField(
        max_length=32,
        blank=True, null=True,
        verbose_name=_('old name'),
    )
    query_keys = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('query keys'),
    )  # stream: mercedes-app - aggrega sotto un kind gli altri kind

    class Meta:
        verbose_name = _('Kind')
        verbose_name_plural = _('Kinds')
        ordering = ('ordering', 'name',)
        index_together = (("ordering", "name"),)

    def __unicode__(self):
        return u"%s (%s)" % (self.name, self.context)

    def get_parent_ids_list(self):
        id_list = []
        for parent in self.parents.all():
            id_list.append(parent.id)
        return id_list

    def get_parent_ids_string(self):
        id_list = self.get_parent_ids_list()
        return ','.join(id_list)

    def get_parent_names_list(self):
        name_list = []
        for parent in self.parents.all():
            name_list.append(parent.name)
        return name_list

    def get_parent_names_string(self):
        name_list = self.get_parent_names_list()
        return ','.join(name_list)

    get_parent_names_string.short_description = _('parents')

    def get_parent_unicodes_list(self):
        uni_list = []
        for parent in self.parents.all():
            uni_list.append(parent.__unicode__())
        return uni_list

    def get_parent_unicodes_string(self):
        uni_list = self.get_parent_unicodes_list()
        return ','.join(uni_list)

    get_parent_names_string.short_description = _('parents')


class Content(ContentAbstract, GenericModel):
    """
    stream
    layout
    issue
    code
    page
    pos
    source
    ref
    place
    has_photo
    has_table
    sector
    kind
    brand1
    brand2
    corp
    model
    version
    abstract
    annotation
    """
    parent = models.ForeignKey(
        'self',
        blank=True, null=True,
        related_name='children',
        verbose_name=_('parent'),
    )
    status = models.CharField(
        max_length=2,
        db_index=True,
        choices=settings.PUBLIEDITOR_CONTENT_STATO_CHOICES,
        verbose_name=_('stato')
    )
    source = models.CharField(
        max_length=20,
        choices=settings.PUBLIEDITOR_CONTENT_SOURCE_CHOICES,
        verbose_name=_('source'),
    )
    stream = models.ForeignKey(
        Stream,
        related_name='contents',
        blank=True, null=True,
        verbose_name=_('stream'),
    )
    layout = models.ForeignKey(
        Layout,
        related_name='contents',
        verbose_name=_('layout'),
    )
    issue = models.ForeignKey(
        Issue,
        related_name='contents',
        blank=True, null=True,
        verbose_name=_("issue"),
    )
    sector = models.ForeignKey(
        Sector,
        related_name='contents',
        blank=True, null=True,
        verbose_name=_('sector'),
    )
    kind = models.ForeignKey(
        Kind,
        related_name='contents',
        blank=True, null=True,
        verbose_name=_('kind'),
    )
    code = models.CharField(
        max_length=16,
        db_index=True,
        blank=True, null=True,
        verbose_name=_('code'),
    )
    page = models.IntegerField(
        blank=True, null=True,
        verbose_name=_('page'),
    )  # numero progressivo - da generare da funzione

    ref = models.CharField(max_length=64, blank=True, null=True)
    place = models.CharField(max_length=32, blank=True, null=True)
    annotation = models.TextField(blank=True, null=True)
    photo = models.URLField(
        db_index=True,
        blank=True, null=True,
        verbose_name=_('photo')
    )
    table = models.URLField(
        db_index=True,
        blank=True, null=True,
        verbose_name=_('table')
    )

    class Meta:
        verbose_name = _('Content')
        verbose_name_plural = _('Contents')

    def __unicode__(self):
        return u"%s: %s" % (self.id, self.name)

    def create_tags(self):
        # set tag kind
        if self.kind:
            value = self.kind.name.lower()
            translations = self.translations.all()
            for translation in translations:
                tag, created = Tag.objects.get_or_create(
                    slug=slugify(value), defaults={'name': value}
                )
                translation.tags.add(tag)

        # set tag source
        if self.sector:
            value = self.source
            translations = self.translations.all()
            for translation in translations:
                tag, created = Tag.objects.get_or_create(
                    slug=slugify(value), defaults={'name': value}
                )
                translation.tags.add(tag)

    ##############################
    # NEW MEDIA METHODS - STARTS #
    ##############################

    ## PHOTOS ##
    def get_media_photos(self):
        """
        Returns queryset of photos
        """
        return self.media_photos.all().order_by('ordering', 'id')

    def get_first_photo(self):
        """
        Returns first media photo object
        """
        if self.get_media_photos():
            return self.get_media_photos()[0]
        return None

    def get_first_photo_16_9(self):
        media_photos = self.media_photos.filter(has16_9=True).order_by('ordering', 'id')
        if media_photos.count():
            return media_photos[0]
        return None

    def get_photo_url_list(self):
        """
        Returns list of dictionary photo urls
        """
        list = []
        for photo in self.get_media_photos():
            list.append(photo.get_photo_url_dict())
        return list

    def has_photo(self):

        #return bool(self.get_first_photo)
        # Just to see that template works correctly till web setup media server
        return bool(self.get_first_photo())

    def get_first_photo_url_dict(self):
        """
        Returns dictionary photo urls
        """
        list = self.get_photo_url_list()
        if list:
            return list[0]
        return None

    def get_photo_html(self):
        if self.get_first_photo():
            return "<img src='%s' alt='%s' width='50px' />" % (
                self.get_first_photo_url_dict()['thumb'],
                self.name
            )
        return ""

    get_photo_html.short_description = _('photo')
    get_photo_html.allow_tags = True

    def get_photo_appy(self):
        "Return a local path suitable to be used in appypod"
        from urllib2 import urlopen

        urlf = urlopen(self.get_first_photo().get_photo_url_dict()['bosch'])
        file_name = '/tmp/appy_%s.jpg' % self.id
        with open(file_name, 'w') as f:
            f.write(urlf.read())
        return file_name

    ## VIDEOS ##
    def get_media_videos(self):
        """
        Returns queryset of videos
        """
        return self.media_videos.all().order_by('ordering', 'id')

    def get_first_video(self):
        """
        Returns first media video object
        """
        if self.get_media_videos():
            return self.get_media_videos()[0]
        return None

    def get_video_url_list(self):
        """
        Returns list of dictionary video urls
        """
        list = []
        for video in self.get_media_videos():
            list.append(video.get_video_url_dict())
        return list

    def get_first_video_url_dict(self):
        """
        Returns dictionary video urls
        """
        list = self.get_video_url_list()
        if list:
            return list[0]
        return None

    def has_video(self):
        # Just to see that template works correctly till web setup media server
        return bool(self.get_first_video())

    ## TABLES ##
    def get_media_tables(self):
        """
        Returns queryset of tables
        """
        return self.media_tables.all().order_by('ordering', 'id')

    def get_first_table(self):
        """
        Returns first media tables object
        """
        if self.get_media_tables():
            return self.get_media_tables()[0]
        return None

    def get_table_url_list(self):
        """
        Returns list of dictionary table urls
        """
        list = []
        for table in self.get_media_tables():
            list.append(table.get_table_url_dict())
        return list

    def get_first_table_url_dict(self):
        """
        Returns dictionary table urls
        """
        list = self.get_table_url_list()
        if list:
            return list[0]
        return None

    def get_table_html(self):
        if self.get_first_table():
            return "<img src='%s' alt='%s' width='100px' style='float:left!important;'/>&nbsp;tables: <b>%s</b>" % (
                self.get_first_table_url_dict()['orig'],
                self.name,
                self.media_tables.count()
            )
        return ""

    get_table_html.short_description = _('table')
    get_table_html.allow_tags = True

    def has_table(self):
        return bool(self.get_first_table())


    # NEW MEDIA METHODS - ENDS

    def get_photo_url(self):
        return "%s%s" % (
            settings.PUBLIEDITOR_CONTENT_MEDIA_URL,
            self.get_photo(),
        )

    def get_photo(self):
        return "gallery/%s00/photo_large/%sF01.jpg" % (str(self.id)[0], self.id)

    def get_width_photo(self):
        ## TODO
        return 1200

    def get_height_photo(self):
        ## TODO
        return 900

    def has_table_url(self):
        return "%s%s" % (
            settings.PUBLIEDITOR_CONTENT_MEDIA_URL,
            self.get_table(),
        )

    def get_table(self):
        return "gallery/%s00/photo_large/%sF01.jpg" % (str(self.id)[0], self.id)

    # def get_photo_html(self):
    #     if self.has_photo():
    #         return "<img src='%s' alt='%s' width='50px' />" % (
    #             self.get_photo_url(),
    #             self.name
    #         )
    #     return ""
    #
    # get_photo_html.allow_tags = True
    # get_photo_html.short_description = _('photo')

    def get_thumb_url(self):
        return "%s%s" % (
            settings.PUBLIEDITOR_CONTENT_MEDIA_URL,
            self.get_thumb(),
        )

    def get_thumb(self):
        """
        """
        return "gallery/%s00/photo_thumb/%sF01.jpg" % (str(self.id)[0], self.id)

    def get_thumb_html(self):
        return "<img src='%s' alt='%s' width='100px' />" % (
            self.get_thumb_url(),
            self.name)

    get_photo.short_description = _('photo')
    get_photo.allow_tags = True

    def get_table_url(self):
        return "%sgallery/300/photo_thumb/%sT01.jpg" % (
            settings.PUBLIEDITOR_CONTENT_MEDIA_URL,
            self.id,
        )

    # def has_table(self):
    #     url = self.get_table_url()
    #     try:
    #         f = urllib2.urlopen(urllib2.Request(url))
    #         return True
    #     except:
    #         return False

    # def get_table(self):
    #     if self.has_table():
    #         return "<img src='%s' alt='%s' width='100px'>" % (
    #             self.get_table_url(),
    #             self.name
    #         )
    #     return ""
    #
    # get_table.short_description = _('table')
    # get_table.allow_tags = True

    def get_corporate_names_list(self):
        details = self.car_details.all()
        detail_list = []
        for detail in details:
            if detail.corporate:
                detail_list.append(detail.corporate.name)
        return detail_list

    get_corporate_names_list.short_description = _('corporates')

    def get_corporate_names_string(self):
        return ", ".join(self.get_corporate_names_list())

    get_corporate_names_string.short_description = _('corporates')

    def get_brand_names_list(self):
        details = self.car_details.all()
        detail_list = []
        for detail in details:
            if detail.brand:
                detail_list.append(detail.brand.name)
        return detail_list

    get_brand_names_list.short_description = _('brands')

    def get_brand_names_string(self):
        return ", ".join(self.get_brand_names_list())

    get_brand_names_string.short_description = _('brands')

    def get_model_names_list(self):
        details = self.car_details.all()
        detail_list = []
        for detail in details:
            if detail.model:
                detail_list.append(detail.model.name)
        return detail_list

    get_model_names_list.short_description = _('models')

    def get_model_names_string(self):
        return ", ".join(self.get_model_names_list())

    get_model_names_string.short_description = _('models')

    def get_carcontent_list(self):
        details = self.car_details.all()
        detail_list = []
        for detail in details:
            carcontent = []
            if detail.corporate:
                carcontent.append(detail.corporate.name)
            if detail.brand:
                carcontent.append(detail.brand.name)
            if detail.model:
                carcontent.append(detail.model.name)
            detail_list.append(" -> ".join(carcontent))
        return detail_list

    get_carcontent_list.short_description = _('carcontents')

    # TODO: ritorna il kind corretto in funzione dello stream
    def get_kind(self, universe):
        return self.kind

    def get_other_sector_contents(self, limit=10):
        contents = Content.objects.none()
        if self.sector:
            contents = Content.objects.filter(sector=self.sector)[:limit]
        return contents

    def get_other_brand_contents(self, limit=10):
        contents = Content.objects.none()
        details = self.car_details.filter(brand__isnull=False)
        if details:
            contents = Content.objects.filter(car_details__brand__in=details.brand)[:limit]
        return contents

    def get_other_kind_contents(self, limit=10):
        contents = Content.objects.none()
        if self.kind:
            contents = Content.objects.filter(kind=self.kind)[:limit]
        return contents


    @property
    def trans(self):
        # essendo previsto uso nel template 
        # suppongo che esista solo 1 traduzione 
        # Già fatta join con lingua dello stream
        try:
            return self.translations.all()[0]
        except:
            return ''

    @property
    def title(self):
        return self.trans.title


    @property
    def body(self):
        txt = re.sub('</?body>', '', self.trans.body)
        return txt.encode('utf-8')

    def date_content_ordinal(self, ):
        from . import utils 
        return utils.format_date_with_ordinal(self.date_content)
        
    @property
    def subtitle(self):
        
        text = ("%s | " % self.ref) if self.ref else ''
        text += self.date_content_ordinal()
        return text

    def get_photo_dict(self):
        "Return a handler suitable for content in appypod"
        from urllib2 import urlopen
        ext = os.path.splitext(self.get_photo_url())[1].lstrip('.')
        return {
            'content' : urlopen(self.get_photo_url()),
            'format'  : ext,
        }

    def save(self, *args, **kwargs):
        if not self.name:
            self.name = "---"
        super(Content, self).save(*args, **kwargs)


class TranslationContent(TranslationContentAbstract, GenericModel):
    content = models.ForeignKey(
        Content,
        related_name='translations',
        verbose_name=_("content")
    )
    abstract = HTMLFieldTinyMCE(
        blank=True, null=True,
        verbose_name=_('abstract')
    )
    body = HTMLFieldTinyMCE(
        blank=True, null=True,
        verbose_name=_('body')
    )

    def get_other_sector_translation_contents(self, limit=10):
        contents = TranslationContent.objects.none()
        if self.content.sector:
            contents = TranslationContent.objects.filter(
                content__sector=self.content.sector).exclude(id=self.id)[:limit]
        return contents

    def get_other_kind_translation_contents(self, limit=10):
        contents = TranslationContent.objects.none()
        if self.content.kind:
            contents = TranslationContent.objects.filter(
                content__kind=self.content.kind).exclude(id=self.id)[:limit]
        return contents

    def has_photo_content_gallery(self):
        return self.content.photos.all().count()

    def get_photo_content_gallery_elements(self):
        return self.content.photos.all()

    def get_detail_url(self):
        return settings.PUBLIEDITOR_CONTENT_NEWS_URL + reverse(
            'content_translation_detail', args=(self.slug,)
        )

    ## Metodo utilizzato per restituire i link del dettaglio
    ## relativi ad autolinknews.com
    def get_atlk_detail_url(self):
        return settings.PUBLIEDITOR_ATLK_CONTENT_NEWS_URL + "/news/" + self.slug

    @property
    def stripped_body(self):
        """
        returns body field with no 'body' html tag
        """
        txt = ""
        if self.body:
            txt = re.sub('</?body>', '', fix_text(self.body))
            soup = BeautifulSoup(txt, convertEntities=BeautifulSoup.HTML_ENTITIES)
            return unicode(soup).replace('&', '&amp;')
        return txt

    @property
    def stripped_abstract(self):
        """
        returns abstract field with no 'body' html tag
        """
        txt = ""
        if self.abstract:
            txt = re.sub('</?body>', '', fix_text(self.abstract))
            soup = BeautifulSoup(txt, convertEntities=BeautifulSoup.HTML_ENTITIES)
            return unicode(soup)
        return txt


class MediaPhoto(DateModel, UserModel, StatusModel, OrderedModel):

    content = models.ForeignKey(
        Content,
        related_name='media_photos',
        verbose_name=_("content")
    )
    photo_id = models.PositiveIntegerField(
        db_index=True,
        verbose_name=_('photo id')
    )
    gallery_id = models.PositiveIntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('gallery id')
    )
    title = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('title')
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_('description')
    )
    width = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('width')
    )
    height = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('height')
    )
    weigth = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('weigth')
    )
    mime_type = models.CharField(
        blank=True, null=True,
        max_length=25,
        verbose_name=_('mime_type')
    )
    has4_3 = models.BooleanField(
        default=True,
        verbose_name=_('4:3 format')
    )
    has16_9 = models.BooleanField(
        default=False,
        verbose_name=_('16:9 format')
    )

    def __unicode__(self):
        return u"%s" % self.photo_id

    class Meta:
        verbose_name = _('media photo')
        verbose_name_plural = _('media photos')
        ordering = ('ordering', 'id',)
        index_together = (("ordering", "id"),)
        unique_together = (("photo_id", "content"),)

    def get_photo_dict(self):
        """
        return a dictionary of relative paths to get photos from media server
        thumbnail: http://publieditor.s3.amazonaws.com/media/xxx/yyy/xxxyyy-thumb.jpg
        image:	http://publieditor.s3.amazonaws.com/media/xxx/yyy/xxxyyy-4_3.jpg
        """
        path = media_path(self.photo_id, 3)
        photo_dict = dict(
            thumb='%s%s-thumb.jpg' % (path, self.photo_id),
            thumb43='%s%s-4_3-thumb.jpg' % (path, self.photo_id),
            orig='%s%s-4_3.jpg' % (path, self.photo_id),
            bosch='%s%s-2_1-small.jpg' % (path, self.photo_id),
            orig16_9="",
            thumb16_9="",
        )
        if self.has16_9:
            photo_dict.update({
                "orig16_9": '%s%s-16_9.jpg' % (path, self.photo_id),
                "thumb16_9": '%s%s-16_9.jpg' % (path, self.photo_id),
            })
        return photo_dict

    def get_photo_url_dict(self):
        """return a dictionary of absolute urls to get photos from media server"""
        photo_url_dict = dict(
            thumb=settings.PUBLIEDITOR_CONTENT_PHOTO_URL + self.get_photo_dict()['thumb'],
            thumb43=settings.PUBLIEDITOR_CONTENT_PHOTO_URL + self.get_photo_dict()['thumb43'],
            orig=settings.PUBLIEDITOR_CONTENT_PHOTO_URL + self.get_photo_dict()['orig'],
            bosch=settings.PUBLIEDITOR_CONTENT_PHOTO_URL + self.get_photo_dict()['bosch'],
            orig16_9="",
            thumb16_9="",
        )
        if self.has16_9:
            photo_url_dict.update({
                "orig16_9": settings.PUBLIEDITOR_CONTENT_PHOTO_URL + self.get_photo_dict()['orig16_9'],
                "thumb16_9": settings.PUBLIEDITOR_CONTENT_PHOTO_URL + self.get_photo_dict()['thumb16_9'],
            })
        return photo_url_dict

    def get_photo_mime_type_dict(self):
        """
        Returns a dictionary of mime type from url dict
        """
        thumb = mimetypes.guess_type(self.get_photo_url_dict()['thumb'])[0]
        orig = mimetypes.guess_type(self.get_photo_url_dict()['orig'])[0]
        return {
            'thumb': thumb,
            'orig': orig
        }

    def get_photo_type_dict(self):
        """
        Returns a dictionary of type from url mime type
        """
        thumb = orig = None
        if self.get_photo_mime_type_dict()['thumb']:
            thumb = self.get_photo_mime_type_dict()['thumb'].split('/')[1]
        if self.get_photo_mime_type_dict()['orig']:
            orig = self.get_photo_mime_type_dict()['orig'].split('/')[1]
        return {
            'thumb': thumb,
            'orig': orig
        }


class MediaTable(DateModel, UserModel, StatusModel, OrderedModel):
    content = models.ForeignKey(
        Content,
        related_name='media_tables',
        verbose_name=_("content")
    )
    table_id = models.PositiveIntegerField(
        db_index=True,
        verbose_name=_('table id')
    )
    title = models.CharField(
        blank=True, null=True,
        max_length=100,
        verbose_name=_('title')
    )
    description = models.CharField(
        blank=True, null=True,
        max_length=100,
        verbose_name=_('description')
    )
    width = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('width')
    )
    height = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('height')
    )
    weigth = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('weigth')
    )
    mime_type = models.CharField(
        blank=True, null=True,
        max_length=25,
        verbose_name=_('mime_type')
    )

    def __unicode__(self):
        return u"%s" % self.table_id

    class Meta:
        verbose_name = _('media table')
        verbose_name_plural = _('media table')
        ordering = ('ordering', 'id',)
        index_together = (("ordering", "id"),)
        unique_together = (("table_id", "content"),)

    def get_table_dict(self):
        """
        return a dictionary of relative paths to get tables from media server
        thumbnail: http://publieditor.s3.amazonaws.com/media/xxx/yyy/xxxyyy-thumb.jpg
        image:	http://publieditor.s3.amazonaws.com/media/xxx/yyy/xxxyyy-4_3.jpg
        """
        path = media_path(self.table_id, 3)

        return {
            'thumb': '%s%s.jpg' % (path, self.table_id),
            'orig': '%s%s.jpg' % (path, self.table_id)
        }

    def get_table_url_dict(self):
        """return a dictionary of absolute urls to get tables from media server"""
        return {
            'thumb': settings.PUBLIEDITOR_CONTENT_PHOTO_URL + self.get_table_dict()['thumb'],
            'orig': settings.PUBLIEDITOR_CONTENT_PHOTO_URL + self.get_table_dict()['orig']
        }

    def get_table_mime_type_dict(self):
        """
        Returns a dictionary of mime type from url dict
        """
        return {
            'thumb': mimetypes.guess_type(self.get_table_url_dict()['thumb'])[0],
            'orig': mimetypes.guess_type(self.get_table_url_dict()['orig'])[0]
        }

    def get_table_type_dict(self):
        """
        Returns a dictionary of type from url mime type
        """
        return {
            'thumb': self.get_table_mime_type_dict()['thumb'].split('/')[1],
            'orig': self.get_table_mime_type_dict()['orig'].split('/')[1]
        }


class MediaVideo(DateModel, UserModel, StatusModel, OrderedModel):
    content = models.ForeignKey(
        Content,
        related_name='media_videos',
        verbose_name=_("content")
    )
    video_id = models.PositiveIntegerField(
        db_index=True,
        verbose_name=_('video id')
    )
    title = models.CharField(
        blank=True, null=True,
        max_length=100,
        verbose_name=_('title')
    )
    description = models.CharField(
        blank=True, null=True,
        max_length=100,
        verbose_name=_('description')
    )
    width = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('width')
    )
    height = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('height')
    )
    weigth = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('weigth')
    )
    mime_type = models.CharField(
        blank=True, null=True,
        max_length=25,
        verbose_name=_('mime_type')
    )
    video_url = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('video url')
    )

    def __unicode__(self):
        return u"%s" % self.video_id

    class Meta:
        verbose_name = _('media video')
        verbose_name_plural = _('media video')
        ordering = ('ordering', 'id',)
        index_together = (("ordering", "id"),)
        unique_together = (("video_id", "content"),)

    def get_video_dict(self):
        """
        return a dictionary of relative paths to get videos from media server
        thumbnail: http://publieditor.s3.amazonaws.com/media/xxx/yyy/xxxyyy-thumb.jpg
        image:	http://publieditor.s3.amazonaws.com/media/xxx/yyy/xxxyyy-4_3.jpg
        """
        path = media_path(self.video_id, 3)

        return {
            'thumb': '%s%s-4_3-thumb.jpg' % (path, self.video_id),
            #'orig': '%s%s.mp4' % (path, self.video_id) # modifica causa url vimeo
            'orig': self.video_url
        }

    def get_video_url_dict(self):
        """return a dictionary of absolute urls to get videos from media server"""
        return {
            'thumb': settings.PUBLIEDITOR_CONTENT_MEDIA_URL + self.get_video_dict()['thumb'],
            'orig': self.get_video_dict()['orig']
        }

    def get_video_mime_type_dict(self):
        """
        Returns a dictionary of mime type from url dict
        """
        mimetype = ""
        if self.get_video_url_dict()['orig']:
            mimetype = self.get_video_url_dict()['orig'].split("?")[0]
        return {
            'thumb': mimetypes.guess_type(self.get_video_url_dict()['thumb'])[0],
            'orig': mimetypes.guess_type(mimetype)[0]
        }

    def get_video_type_dict(self):
        """
        Returns a dictionary of type from url mime type
        """
        thumb = orig = None
        if self.get_video_mime_type_dict()['thumb']:
            thumb = self.get_video_mime_type_dict()['thumb'].split('/')[1]
        if self.get_video_mime_type_dict()['orig']:
            orig = self.get_video_mime_type_dict()['orig'].split('/')[1]
        return {
            'thumb': thumb,
            'orig': orig
        }


class PhotoContentGallery(DateModel, UserModel, StatusModel, OrderedModel):
    content = models.ForeignKey(
        Content,
        related_name='photos',
        verbose_name=_("content")
    )
    photo = models.URLField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('photo')
    )

    def __unicode__(self):
        return u"%s" % self.photo

    def get_photo_url(self):
        if self.photo and len(self.photo.split('.')) == 2:
            return "%sfiles/300/%s/x/%s" % (
                settings.PUBLIEDITOR_CONTENT_MEDIA_URL,
                self.content_id,
                self.photo,
            )
        else:
            return self.photo

    def get_photo(self):
        if self.photo:
            return "<img src='%s' alt='%s' width='100px'>" % (
                self.get_photo_url(),
                self.photo
            )
        else:
            return ""
        
    get_photo.short_description = _('photo')
    get_photo.allow_tags = True

    def get_photo_image_prop(self):
        return format_html(self.get_photo())
    photo_image = property(get_photo_image_prop)

    def get_photo_type(self):
        return self.get_photo_mime_type().partition('/')[2]

    def get_photo_mime_type(self):
        photo_mime_type = mimetypes.guess_type(self.photo)[0]
        return photo_mime_type

    class Meta:
        verbose_name = _('photo gallery')
        verbose_name_plural = _('photo galleries')
        ordering = ('ordering', 'id',)
        index_together = (("ordering", "id"),)


class VideoContentGallery(DateModel, UserModel, StatusModel, OrderedModel):
    content = models.ForeignKey(
        Content,
        related_name='videos',
        verbose_name=_("content")
    )
    video = models.URLField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('video')
    )

    def __unicode__(self):
        return u"%s" % self.video

    def get_video_url(self):
        if self.video and len(self.video.split('.')) == 2:
            return "%sfiles/300/%s/x/%s" % (
                settings.PUBLIEDITOR_CONTENT_MEDIA_URL,
                self.content_id,
                self.video,
            )
        else:
            return self.video

    def get_video(self):
        return "%s %s" % (
            self.get_video_url(),
            self.video
        )
    get_video.short_description = _('video')
    get_video.allow_tags = True

    def get_video_image_prop(self):
        return format_html(self.get_video())
    video_image = property(get_video_image_prop)

    def get_video_type(self):
        return self.get_video_mime_type().partition('/')[2]

    def get_video_mime_type(self):
        video_mime_type = mimetypes.guess_type(self.video)[0]
        if video_mime_type is None:
            video_mime_type = 'video/mp4'
        return video_mime_type

    class Meta:
        verbose_name = _('video gallery')
        verbose_name_plural = _('video galleries')
        ordering = ('ordering', 'id',)
        index_together = (("ordering", "id"),)


class PhotoCarModelGallery(DateModel, UserModel, StatusModel, OrderedModel):
    model = models.ForeignKey(
        CarModel,
        related_name='photos',
        verbose_name=_("car model")
    )
    photo = models.URLField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('photo')
    )

    def __unicode__(self):
        return u"%s" % self.photo

    class Meta:
        verbose_name = _('photo gallery')
        verbose_name_plural = _('photo galleries')
        ordering = ('ordering', 'id',)
        index_together = (("ordering", "id"),)

    def get_photo_url(self):
        return self.photo        
        
    def get_photo(self):
        if self.photo:
            return "<img src='%s' alt='%s' width='100px'>" % (
                self.get_photo_url(),
                self.get_photo_url()
            )
        else:
            return ""
    get_photo.short_description = _('photo')
    get_photo.allow_tags = True

    def get_photo_image_prop(self):
        return format_html(self.get_photo())
    photo_image = property(get_photo_image_prop)

    def get_photo_url_formats(self):
        photo_dict = dict()
        for key, dim in settings.PUBLIEDITOR_CONTENT_PHOTO_FORMATS.items():
            photo_dict[key] = self.photo.replace('/w640/', '/%s/' % dim)
        return photo_dict

    def get_absolute_url(self):
        return self.photo

    def get_size(self):
        import urllib, cStringIO
        from PIL import Image
        file = cStringIO.StringIO(urllib.urlopen(self.photo).read())
        photo=Image.open(file)
        width, height = photo.size
        return {'width': width, 'height': height}


class VideoCarModelGallery(DateModel, UserModel, StatusModel, OrderedModel):
    model = models.ForeignKey(
        CarModel,
        related_name='videos',
        verbose_name=_("car model")
    )
    video = models.URLField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_('video')
    )

    def __unicode__(self):
        return u"%s" % self.video

    class Meta:
        verbose_name = _('video car model gallery')
        verbose_name_plural = _('video car model galleries')
        ordering = ('ordering', 'id',)
        index_together = (("ordering", "id"),)

    def get_video_url(self):
        return self.video        
        
    def get_video(self):
        return "<img src='%s' alt='%s' width='100px'>" % (
            self.get_video_url(),
            self.get_video_url()
        )        
    get_video.short_description = _('video')
    get_video.allow_tags = True

    def get_video_image_prop(self):
        return format_html(self.get_video())
    video_image = property(get_video_image_prop)


class Preference(DateModel, UserModel, StatusModel, OrderedModel):
    content = models.ForeignKey(
        Content,
        related_name='preferences',
        verbose_name=_("content")
    )
    reader = models.ForeignKey(
        User,
        related_name="content_preferences",
        verbose_name=_("reader")
    )
    last_ip_address = models.GenericIPAddressField(
        protocol='both',
        unpack_ipv4=True,
        verbose_name=_('Last IP address')
    )
    read = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('read')
    )
    saved = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('saved')
    )
    deleted = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('deleted')
    )

    class Meta:
        verbose_name = _('Preference')
        verbose_name_plural = _('Preferences')
        ordering = ('ordering',)
        unique_together = ('reader', 'content')
        index_together = (('reader', 'content'),)


class CarContent(DateModel, UserModel, StatusModel, OrderedModel, GenericModel):
    # da sistemare - ho modificato il modello per permettere di
    # aggiungere diversi modelli per ogni content
    content = models.ForeignKey(
        Content,
        related_name='car_details',
    )
    corporate = models.ForeignKey(
        Corporate,
        related_name='corporate_contents',
        blank=True, null=True
    )
    brand = models.ForeignKey(
        Brand, related_name='brand_contents',
        blank=True, null=True
    )
    model = models.ForeignKey(
        CarModel, related_name='model_contents',
        blank=True, null=True
    )
    car = models.ForeignKey(
        Car, related_name='car_contents',
        blank=True, null=True
    )

    class Meta:
        verbose_name = _('Car Content')
        verbose_name_plural = _('Car Contents')
        ordering = ('ordering',)
        index_together = (('ordering',),)

    def __unicode__(self):
        return u"%s: %s %s %s %s" % (
            self.content,
            self.corporate or "", self.brand or "",
            self.model or "", self.car or ""
        )

    def save(self, *args, **kwargs):
        super(CarContent, self).save(*args, **kwargs)
        self.create_tags()

    def create_tags(self):
        # set tag corporate
        if self.corporate:
            value = self.corporate.name.lower()
            translations = self.content.translations.all()
            for translation in translations:
                tag, created = Tag.objects.get_or_create(
                    slug=slugify(value), defaults={'name': value}
                )
                translation.tags.add(tag)

        # set tag brand
        if self.brand:
            value = self.brand.name.lower()
            translations = self.content.translations.all()
            for translation in translations:
                tag, created = Tag.objects.get_or_create(
                    slug=slugify(value), defaults={'name': value}
                )
                translation.tags.add(tag)

        # set tag model
        if self.model:
            value = self.model.name.lower()
            translations = self.content.translations.all()
            for translation in translations:
                tag, created = Tag.objects.get_or_create(
                    slug=slugify(value), defaults={'name': value}
                )
                translation.tags.add(tag)


class TestDrive(Content):
    """
    stream
    layout
    issue
    code
    page
    pos
    source
    ref
    place
    has_photo
    has_table
    sector
    kind
    brand1
    brand2
    corp
    model
    version
    abstract
    annotation
    """

    class Meta:
        verbose_name = _('Test Drive')
        verbose_name_plural = _('Tests Drive')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


def get_openhouse_point_choices():
    choices = []
    for point in range(
            settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT,
            settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT + 1
    ):
        choices.append((point, point))
    return choices


class OpenHouse(Content):
    """
    stream
    layout
    issue
    code
    page
    pos
    source
    ref
    place
    has_photo
    has_table
    sector
    kind
    brand1
    brand2
    corp
    model
    version
    abstract
    annotation
    """
    overall_description = models.TextField(blank=True, null=True)
    waiting_description = models.TextField(blank=True, null=True)
    availability_description = models.TextField(blank=True, null=True)
    roadtest_description = models.TextField(blank=True, null=True)
    welcome_description = models.TextField(blank=True, null=True)
    quotation_description = models.TextField(blank=True, null=True)
    financial_description = models.TextField(blank=True, null=True)
    brochure_description = models.TextField(blank=True, null=True)
    offers_description = models.TextField(blank=True, null=True)
    showroom_description = models.TextField(blank=True, null=True)
    buyback_description = models.TextField(blank=True, null=True)
    overall_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    waiting_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    availability_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    roadtest_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    welcome_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    quotation_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    financial_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    brochure_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    offers_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    showroom_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])
    buyback_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_openhouse_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_OPENHOUSE_MAX_POINT)])

    def get_point_avg(self):
        total = 0
        cont = 0
        for key in [
            "overall_point", 'waiting_point', 'availability_point', 'roadtest_point', 'welcome_point',
            'quotation_point', 'financial_point', 'brochure_point', 'offers_point', 'showroom_point', 'buyback_point',
        ]:
            value = getattr(self, key) or 0
            if value:
                total += value
                cont += 1
        if cont > 0:
            return round(total / cont, 1)
        return None

    class Meta:
        verbose_name = _('Open House')
        verbose_name_plural = _('Open House')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class CrashTest(Content):
    class Meta:
        verbose_name = _('Crash Test')
        verbose_name_plural = _('Crash Tests')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class Brochure(Content):
    class Meta:
        verbose_name = _('Brochure')
        verbose_name_plural = _('Brochures')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class Catalog(Content):
    class Meta:
        verbose_name = _('Catalog')
        verbose_name_plural = _('Catalogs')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class Video(Content):
    class Meta:
        verbose_name = _('Video')
        verbose_name_plural = _('Video')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


def get_roadtest_point_choices():
    choices = []
    for point in range(
            settings.PUBLIEDITOR_CONTENT_ROADTEST_MIN_POINT,
            settings.PUBLIEDITOR_CONTENT_ROADTEST_MAX_POINT + 1
    ):
        choices.append((point, point))
    return choices


class RoadTest(Content):
    global_point = models.IntegerField(null=True, blank=True)
    pros = models.TextField(
        blank=True, null=True,
        verbose_name=_('pro'),
    )
    cons = models.TextField(
        blank=True, null=True,
        verbose_name=_('cons'),
    )
    howitis_description = models.TextField(
        blank=True, null=True,
        verbose_name=_('how it is'),
    )
    howitgoes_description = models.TextField(
        blank=True, null=True,
        verbose_name=_('how it goes'),
    )
    performance_description = models.TextField(
        blank=True, null=True,
        verbose_name=_('performance'),
    )
    design_description = models.TextField(
        blank=True, null=True,
        verbose_name=_('design'),
    )
    comfort_description = models.TextField(
        blank=True, null=True,
        verbose_name=_('comfort'),
    )
    safety_description = models.TextField(
        blank=True, null=True,
        verbose_name=_('safety'),
    )
    value_description = models.TextField(
        blank=True, null=True,
        verbose_name=_('value'),
    )
    howitis_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_roadtest_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MAX_POINT)],
        verbose_name=_('how it is point'),
    )
    howitgoes_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_roadtest_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MAX_POINT)],
        verbose_name=_('how it goes point'),
    )
    performance_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_roadtest_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MAX_POINT)],
        verbose_name=_('performance point'),
    )
    design_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_roadtest_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MAX_POINT)],
        verbose_name=_('design point'),
    )
    comfort_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_roadtest_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MAX_POINT)],
        verbose_name=_('comfort point'),
    )
    safety_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_roadtest_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MAX_POINT)],
        verbose_name=_('safety point'),
    )
    value_point = models.PositiveIntegerField(
        null=True, blank=True,
        choices=get_roadtest_point_choices(),
        validators=[
            MinValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MIN_POINT),
            MaxValueValidator(settings.PUBLIEDITOR_CONTENT_ROADTEST_MAX_POINT)],
        verbose_name=_('value point'),
    )
    toknow = models.TextField(
        blank=True, null=True,
        verbose_name=_('to know'),
    )
    url = models.TextField(
        blank=True, null=True,
    )

    def get_point_avg(self):
        total = 0
        cont = 0
        for key in [
            "howitis_point", 'howitgoes_point', 'performance_point',
            'design_point', 'comfort_point', 'safety_point', 'value_point',
        ]:
            value = getattr(self, key) or 0
            if value:
                total += value
                cont += 1
        if cont > 1:
            return round(total / cont, 1)
        return None

    class Meta:
        verbose_name = _('Road Test')
        verbose_name_plural = _('Road Tests')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class PhotoGallery(Content):
    class Meta:
        verbose_name = _('Photo Gallery')
        verbose_name_plural = _('Photo Galleries')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class Gallery(Content):
    class Meta:
        verbose_name = _('Gallery')
        verbose_name_plural = _('Galleries')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class Approfondimento(Content):
    class Meta:
        verbose_name = _('Approfondimento')
        verbose_name_plural = _('Approfondimenti')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class Breve(Content):
    class Meta:
        verbose_name = _('Breve')
        verbose_name_plural = _('Brevi')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class News(Content):
    class Meta:
        verbose_name = _('News')
        verbose_name_plural = _('News')
        ordering = ('ordering', '-date_content', 'name')

    def __unicode__(self):
        return u"%s" % self.name


class UserContentSettings(DateModel, UserModel):
    user = models.ForeignKey(
        User,
        related_name='content_settings',
    )
    translation_content = models.ForeignKey(
        TranslationContent,
        related_name='user_settings',
    )
    is_read = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('read'),
    )
    is_favorite = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('favorite'),
    )
    is_removed = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('removed'),
    )

    class Meta:
        verbose_name = _('User content setting')
        verbose_name_plural = _('Users content settings')
        unique_together = ('user', 'translation_content')
        index_together = (
            ('user', 'translation_content'),
            ('user', 'translation_content', 'is_read'),
            ('user', 'translation_content', 'is_favorite'),
            ('user', 'translation_content', 'is_removed'),
            ('user', 'translation_content', 'is_favorite', 'is_removed'),
        )

    def __unicode__(self):
        return u"%s: %s" % (self.translation_content, self.user)


class TooltipAbstract(StatusModel, DateModel, UserModel):
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
        db_index=True
    )    
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    slug = models.SlugField(
        max_length=255,
        verbose_name=_("slug"),
        db_index=True
    )
    description = models.CharField(
        max_length=255,
        verbose_name=_('description'),
    )
        
    class Meta:
        verbose_name = _('Tooltip')
        verbose_name_plural = _('Tooltips')
        abstract = True
        unique_together = (('language', 'slug'),)
        index_together = (('language', 'slug'),)


class Tooltip(TooltipAbstract):
    pass


#import cms stuffs
if settings.CMS_ENABLED:
    import publieditor.content.cms.models
