# -*- coding: utf-8 -*-
"""
helper module for publieditor.content
"""
import calendar
import datetime
from dateutil import tz
from operator import itemgetter
from collections import OrderedDict

from django.db import models
from django.db.models import Max
from django.contrib.admin.util import lookup_field, display_for_field
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt

from jmb.core.admin.ajax_inlines import ConfigurationError
from publieditor.content.models import (
    UserContentSettings, TranslationContent, CarModel,
    UNRAECarModel, UNRAECarModelSale, CarModelSale, SalePeriod
)
from publieditor.oldapp.models import UnraeMap


def update_user_content_setting(translation_content_id, user, field):
    translation_content = TranslationContent.objects.get(pk=translation_content_id)
    user_content_settings = UserContentSettings.objects.filter(
        user=user,
        translation_content=translation_content
    )
    if user_content_settings:
        user_content_setting = user_content_settings[0]
    else:
        user_content_setting = UserContentSettings(
            user=user,
            translation_content=translation_content
        )
    new_value = not getattr(user_content_setting, field)
    setattr(user_content_setting, field, new_value)
    user_content_setting.save()
    return new_value


def get_first_day_of_week(day):
    day_of_week = day.weekday()
    beginning_of_week = day - datetime.timedelta(days=day_of_week)
    return beginning_of_week


def get_first_day_of_month(day):
    return datetime.date(day.year, day.month, 1)


def get_first_day_of_year(day):
    return datetime.date(day.year, 1, 1)


def get_querystring_dict(request):
    query_dict = {}
    for term in request.META['QUERY_STRING'].split("&"):
        try:
            query_dict[term.split("=")[0]] = term.split("=")[1]
        except IndexError:
            continue
    return query_dict


class Stat(OrderedDict):
    def add(self, car):
        # la chiave e' l'id del modello
        # ogni valore e' tupla (modello [car1, car2, car3, ...])
        if car.model_id in self:
            self[car.model.id][1].append(car)
        else:
            self[car.model.id] = (car.model, [car])

    def show(self, key):
        # solo per mostrare i risultati
        model, cars = self[key]
        return [model.brand.name, model.name, [c.price for c in cars]]

    def get_min(self, key, attr_name='price'):
        # estrae il minimo dell'attribute_name nella lista di versioni (cars)
        model_id, cars = self[key]
        cars.sort(key=lambda x: getattr(x, attr_name))
        return getattr(cars[0], attr_name)

    def get_max(self, key, attr_name):
        model_id, cars = self[key]
        cars.sort(key=lambda x: getattr(x, attr_name), reverse=True)
        return getattr(cars[0], attr_name)

    def get_avg(self, key, attr_name):

        model_id, cars = self[key]
        return sum(getattr(x, attr_name) for x in cars)/len(cars)


def get_stat(qs=None, limit=50, op='min', attr_name='price', order='a'):
    """
    Return a list of tuples: (model, operation)
    Operation can be any operation defined in Stat Ordered
    """
    from .models import Car
    # Passando il qs
    order_by_field = "%s" % attr_name if order == 'a' else "-%s" % attr_name
    qs = qs or Car.objects.filter(
        **{'%s__isnull' % attr_name: False}
    ).select_related('model', 'model__brand').order_by(
        order_by_field, 'brand__name', 'model__name'
    )
    car_stat = Stat()
    for car in qs:
        car_stat.add(car)

    output = []
    operation = getattr(car_stat, 'get_%s' % op)

    for n, key in enumerate(car_stat):
        if n == (limit + 5):  # in case of average you cant' rely on OrderedDict
            break
        output += [(car_stat[key][0],  operation(key, attr_name))]

    out = []
    for key, val in sorted(
            output, key=itemgetter(1), reverse=not(order == 'a'))[:limit]:
        out += [{'models': [key], 'value': val}]
    return out


def get_stat_sold(qs=None, limit=10, ):
    """
    Return a list of tuples: (model, operation)
    Operation can be any operation defined in Stat Ordered
    """
    from .models import CarModelSale
    # Passando il qs
    if qs is None:
        qs = CarModelSale.objects.filter(status='ok', **filters).order_by(
            '-sold', 'models__brand__name', 'models__model__name'
        ).select_related(
            'models', 'models__brand'
        )[0:limit]

    # La situazione è più semplice di get_stat.

    output = []
    for x in qs:
        output += [{'models': x.carmodels.all(), 'value': x.sold}]
    return output


def set_unraecarmodel_carmodels():
    # Get and loop over all UNRAE sales objects
    unraemaps = UnraeMap.objects.all()
    design_list = (
        ("fuoristrada", "Fuoristrada"),
        ("cabrio_e_spider", "CabrioeSpider"),
        ("coupe", "Coup"),
        ("berlina_2_volumi", "Berlina2Vol"),
        ("berlina_3_volumi", "Berlina3Vol"),
        ("multispazio", "Multispazio"),
        ("monovolume", "Monovolume"),
        ("station_wagon", "StationWagon"),
        ("crossover", "Crossover"),
    )
    for unraemap in unraemaps:
        for oldapp_design, pub_design in design_list:
            carmodel_id = getattr(unraemap, oldapp_design)
            if carmodel_id:
                try:
                    carmodel = CarModel.objects.get(id=carmodel_id)
                    unraecarmodel, created = UNRAECarModel.objects.get_or_create(
                        brand=unraemap.marca, carmodel=unraemap.modello, design=pub_design
                    )
                    unraecarmodel.carmodels.add(carmodel)
                except CarModel.DoesNotExist:
                    pass
                    # log.debug("Carmodel with given id %s not found" % carmodel_id)


def set_model_sale_list():
    months = {
        1: "gennaio", 2: "febbraio",  3: "marzo", 4: "aprile", 5: "maggio", 6: "giugno",
        7: "luglio", 8: "agosto", 9: "settembre", 10: "ottobre", 11: "novembre", 12: "dicembre",
    }
    if UNRAECarModelSale.objects.all().count():

        # Creazione nuova voce sale_period (SalePeriod)
        max_year = UNRAECarModelSale.objects.all().aggregate(Max('year'))['year__max']
        max_month = UNRAECarModelSale.objects.filter(year=max_year).aggregate(Max('month'))['month__max']
        start_day_month, end_day_month = calendar.monthrange(max_year, max_month)

        # 1 year range
        date_begin = datetime.date(max_year + (max_month / 12) - 1, ((max_month % 12) + 1), 1)
        date_end = datetime.date(max_year, max_month, end_day_month)

        prep = "ad" if months[max_month] in ["aprile", "agosto", "ottobre"] else "a"

        # se data inizio e data fine nello stesso anno
        if date_begin.year == max_year:
            description = "Negli ultimi 12 mesi, da %s %s %s %s" % (
                months[date_begin.month], prep, months[max_month], max_year
            )
        else:
            description = "Negli ultimi 12 mesi, da %s %s %s %s %s" % (
                months[date_begin.month], date_begin.year, prep, months[max_month], max_year
            )

        # get or create sale period
        sale_period, sale_period_created = SalePeriod.objects.get_or_create(
            date_begin=date_begin, date_end=date_end, defaults={'name': description}
        )

        if not sale_period_created:
            sale_period.sales.all().delete()

        # Ciclo UNRAECarModelSale e li raggruppo tramite dizionario per carmodels sommando le unita vendute
        carmodels_sold_dict = dict()

        # prendo i dati di vendita di tutti i modelli relativi al periodo scelto
        for unrae_carmodel_sale in UNRAECarModelSale.objects.all():
            start_day_month, end_day_month = calendar.monthrange(unrae_carmodel_sale.year, unrae_carmodel_sale.month)
            unrae_carmodel_sale_date = datetime.date(unrae_carmodel_sale.year, unrae_carmodel_sale.month, end_day_month)
            if date_begin <= unrae_carmodel_sale_date <= date_end:
                carmodel_ids = "_".join([
                    str(carmodel.id) for carmodel in unrae_carmodel_sale.unrae_carmodel.carmodels.all().order_by('id')
                ])
                sold = unrae_carmodel_sale.sold
                if carmodel_ids not in carmodels_sold_dict:
                    carmodels_sold_dict[carmodel_ids] = sold
                else:
                    carmodels_sold_dict[carmodel_ids] += sold

        for carmodel_ids, sold in carmodels_sold_dict.iteritems():
            carmodel_sale = CarModelSale(period=sale_period, sold=sold)
            carmodel_sale.save()
            for carmodel_id in carmodel_ids.split("_"):
                if carmodel_id.isdigit():
                    carmodel = CarModel.objects.get(id=carmodel_id)
                    carmodel_sale.carmodels.add(carmodel)


def format_date_with_ordinal(d, format_string='{S} %B, %Y'):
    """Return a date with 1th, 2nd, 3rd end the like...

    :arg d: a datetime.date instance
    :arg format_string: default: '{S} %B, %Y'
    """

    # date_issue requires 1th, 2nd, 3rd end the like...
    def suffix(d):
        return 'th' if 11 <= d <= 13 else {1: 'st', 2: 'nd', 3: 'rd'}.get(d % 10, 'th')

    def custom_strftime(format, t):
        to_zone = tz.gettz('Europe/Rome')
        t = t.astimezone(to_zone)
        return t.strftime(format).replace('{S}', str(t.day) + suffix(t.day))

    return custom_strftime(format_string, d)


def _export_data(modeladmin, fields_list,  request=None, queryset=None):
    """
    ritorna un generatore di liste contenenti i valori degli item del queryset
    :arg modeladmin: the modeladmin.
    :arg request: the request, not used
    :arg queryset: the queryset or a list of
    """



    display_fields_list = []
    for field in fields_list:
        try:
            display_fields_list.append(getattr(modeladmin, field).short_description)
        except Exception, e:
            display_fields_list.append(field)

    title = list(display_fields_list)
    title.append('\n')
    if 'action_checkbox' in title:
        title.remove('action_checkbox')
    yield title

    for obj in queryset:
        row = []
        for field_name in fields_list:

            if field_name == 'action_checkbox':
                continue
            field, attr, value = lookup_field(field_name, obj, modeladmin)
            if not field:
                # try to interpret it as a modeladmin's function
                if hasattr(modeladmin, field_name):
                    result_repr = getattr(modeladmin, field_name)(obj)
                elif hasattr(obj, field_name):
                    result_repr = getattr(obj, field_name)()
                else:
                    ConfigurationError(
                        ugt("field name %(field_name)s does not appear to be valid") % {'field_name': field_name}
                    )
            elif value is None:
                if isinstance(field, models.NullBooleanField):
                    result_repr = ugt("None")
                else:
                    result_repr = ''
            else:
                if isinstance(field, (models.NullBooleanField, models.BooleanField)):
                    result_repr = value and _('Yes') or _('No')
                else:
                    result_repr = display_for_field(value, field)

            # Tolgo eventuali 'a capo'
            row.append(unicode(result_repr))
        yield row
