# -*- coding: utf-8 -*-

import json

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db.models import Min, Max
from django.http import HttpResponse, HttpResponseRedirect
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404
from django.utils.translation import ugettext as _
from django.views.generic import ListView, DetailView
from django import forms

from publieditor.content.utils import update_user_content_setting
from publieditor.content.models import (
    Brand, CarModel, Car, Hit, CarModelEquipment, Issue, Content, TranslationContent
)
from django.views.decorators.cache import never_cache


def ajax_favorite_content(request):
    data = dict()
    translation_content_id = request.GET['translation_content_id']
    is_favorite = update_user_content_setting(translation_content_id, request.user, 'is_favorite')
    data['content_favorite'] = is_favorite
    return HttpResponse(json.dumps(data), content_type="application/json")


def ajax_remove_content(request):
    data = dict()
    translation_content_id = request.GET['translation_content_id']
    is_removed = update_user_content_setting(translation_content_id, request.user, 'is_removed')
    data['content_removed'] = is_removed
    return HttpResponse(json.dumps(data), content_type="application/json")


# ADD AND REMOVE ITEMS
@never_cache
def add_all_to_favourites(request):
    id_list = map(int, request.GET.getlist('idList[]'))
    request.session['favourites_cars'].extend(id_list)
    data = list(set(request.session['favourites_cars']))
    return HttpResponse(json.dumps(data), content_type="application/json")


@never_cache
def remove_all_from_favourites(request):
    id_list = map(int, request.GET.getlist('idList[]'))
    request.session['favourites_cars'] = list(set([x for x in request.session['favourites_cars'] if x not in id_list]))
    return HttpResponse(json.dumps(request.session['favourites_cars']), content_type="application/json")


@never_cache
def add_all_to_compare(request):
    id_list = map(int, request.GET.getlist('idList[]'))
    try:
        request.session['tocompare_cars'].extend(id_list)
    except (KeyError, AttributeError):
        request.session['tocompare_cars'] = id_list
    data = list(set(request.session['tocompare_cars']))
    return HttpResponse(json.dumps(data), content_type="application/json")


@never_cache
def remove_all_from_compare(request):
    id_list = map(int, request.GET.getlist('idList[]'))
    try:
        request.session['tocompare_cars'] = list(set([x for x in request.session['tocompare_cars'] if x not in id_list]))
    except (KeyError, AttributeError):
        request.session['tocompare_cars'] = []
    return HttpResponse(json.dumps(request.session['tocompare_cars']), content_type="application/json")


@never_cache
def add_to_favourites(request):
    car_id = int(request.GET['car_id'])
    try:
        request.session['favourites_cars'].append(car_id)
    except KeyError:
        request.session['favourites_cars'] = [car_id]
    data = request.session['favourites_cars']
    return HttpResponse(json.dumps(data), content_type="application/json")


@never_cache
def remove_from_favourites(request):
    car_id = int(request.GET['car_id'])
    request.session['favourites_cars'] = request.session.get('favourites_cars', [])
    if car_id in request.session['favourites_cars']:
        request.session['favourites_cars'].remove(car_id)
    data = request.session['favourites_cars']
    return HttpResponse(json.dumps(data), content_type="application/json")


@never_cache
def add_to_compare(request):
    car_id = int(request.GET['car_id'])
    try:
        request.session['tocompare_cars'].append(car_id)
    except (KeyError, ValueError, AttributeError):
        request.session['tocompare_cars'] = [car_id]
    data = request.session['tocompare_cars']
    return HttpResponse(json.dumps(data), content_type="application/json")


@never_cache
def remove_from_compare(request):
    car_id = int(request.GET['car_id'])
    try:
        request.session['tocompare_cars'].remove(car_id)
    except (KeyError, ValueError, AttributeError):
        request.session['tocompare_cars'] = []
    data = request.session['tocompare_cars']
    return HttpResponse(json.dumps(data), content_type="application/json")


# GET ITEMS
@never_cache
def get_favourites(request):
    if 'favourites_cars' not in request.session:
        request.session['favourites_cars'] = []
        return HttpResponse(json.dumps([]))
    else:
        data = list(set(request.session['favourites_cars']))
        return HttpResponse(json.dumps(data), content_type="application/json")


@never_cache
def get_compare(request):
    if 'tocompare_cars' not in request.session:
        request.session['tocompare_cars'] = []
        return HttpResponse(json.dumps([]))
    else:
        data = list(set(request.session['tocompare_cars']))
        return HttpResponse(json.dumps(data), content_type="application/json")


class ModelsInHomeMixin(object):

    @staticmethod
    def get_models_in_home():
        return CarModel.objects.filter(
            status__in=('online', 'runout'),
            new_entry__in=('side', 'home'),
        ).order_by('-id')[:3]


class FavouritesListView(ListView, ModelsInHomeMixin):
    template_name = "content/favourites.html"
    context_object_name = 'favourites_cars'

    def get_queryset(self):
        queryset = Car.objects.none()
        if 'favourites_cars' in self.request.session:
            queryset = Car.objects.filter(
                id__in=self.request.session['favourites_cars']
            ).order_by('model')
        else:
            self.request.session['favourites_cars'] = {}
        return queryset

    def get_context_data(self, **kwargs):
        ctx = super(FavouritesListView, self).get_context_data(**kwargs)
        ctx['models_in_home'] = self.get_models_in_home()
        ctx['hits'] = Hit.objects.filter(status=1).order_by('ordering')[:3]
        breadcrumbs = settings.PUBLIEDITOR_CONTENT_BREADCRUMBS + [{'title': _('le tue auto preferite'), 'url': None}]
        ctx['breadcrumbs'] = breadcrumbs
        return ctx


class CompareListView(ListView, ModelsInHomeMixin):
    template_name = "content/compare.html"
    context_object_name = 'tocompare_cars'

    def get_queryset(self):
        queryset = Car.objects.none()
        if 'tocompare_cars' in self.request.session:
            queryset = Car.objects.filter(
                id__in=self.request.session['tocompare_cars']
            ).order_by('model')
        else:
            self.request.session['tocompare_cars'] = []
        return queryset

    def get_context_data(self, **kwargs):
        ctx = super(CompareListView, self).get_context_data(**kwargs)
        ctx['models_in_home'] = self.get_models_in_home()
        ctx['hits'] = Hit.objects.filter(status=1).order_by('ordering')[:3]
        breadcrumbs = settings.PUBLIEDITOR_CONTENT_BREADCRUMBS + [{'title': _('Confronta'), 'url': None}]
        ctx['breadcrumbs'] = breadcrumbs
        return ctx


class PrezziautoListView(ListView, ModelsInHomeMixin):

    template_name = 'content/prezziauto.html'
    queryset = Brand.objects.filter(models__status="online").order_by('name').distinct()

    def get_context_data(self, **kwargs):
        ctx = super(PrezziautoListView, self).get_context_data(**kwargs)
        ctx['models_in_home'] = self.get_models_in_home()
        ctx['hits'] = Hit.objects.filter(status=1).order_by('ordering')[:3]
        breadcrumbs = settings.PUBLIEDITOR_CONTENT_BREADCRUMBS[:]
        breadcrumbs.pop()
        breadcrumbs.append({'title': _('Listino Prezzi Auto'), 'url': None},)
        ctx['breadcrumbs'] = breadcrumbs
        return ctx


class HitListView(ListView, ModelsInHomeMixin):
    model = Hit
    template_name = 'content/hit_list.html'

    def get_context_data(self, **kwargs):
        ctx = super(HitListView, self).get_context_data(**kwargs)
        ctx['models_in_home'] = self.get_models_in_home()
        ctx['hits'] = Hit.objects.filter(status=1).order_by('ordering', '-id')
        breadcrumbs = settings.PUBLIEDITOR_CONTENT_BREADCRUMBS + [
            {'title': _('Classifiche'), 'url': None}
        ]
        ctx['breadcrumbs'] = breadcrumbs
        return ctx


class HitDetailView(DetailView, ModelsInHomeMixin):
    model = Hit
    template_name = 'content/hit_detail.html'

    def get_context_data(self, **kwargs):
        ctx = super(HitDetailView, self).get_context_data(**kwargs)
        ctx['models_in_home'] = self.get_models_in_home()
        try:
            ctx['stats'] = self.object.get_stat()
        except ValueError:
            ctx['stats'] = self.object.get_stat(limit=50)
        ctx['models_in_aside'] = CarModel.objects.filter(
            status__in=('online', 'runout'),
            new_entry__in=('side', 'home'),
        ).order_by('-id')[:3]
        ctx['hits'] = Hit.objects.filter(status=1).order_by('ordering')[:3]
        breadcrumbs = settings.PUBLIEDITOR_CONTENT_BREADCRUMBS + [
            {'title': _('Classifiche'), 'url': reverse('content:hit_list')},
            {'title': _(self.object.title), 'url': None}
        ]
        ctx['breadcrumbs'] = breadcrumbs
        return ctx


class BrandDetailView(DetailView, ModelsInHomeMixin):

    model = Brand

    def get(self, request, *args, **kwargs):
        if settings.PUBLIEDITOR_REPLACE_UNDERSCORE:
            old_full_path = request.get_full_path()
            new_full_path = old_full_path.lower().replace('_', '-').replace('.', '')
            if old_full_path != new_full_path:
                return HttpResponseRedirect(new_full_path)
        return super(BrandDetailView, self).get(request, *args, **kwargs)

    def get_object(self):
        return super(BrandDetailView, self).get_object(
            queryset=Brand.objects.filter(status='active')
        )

    def get_context_data(self, **kwargs):
        ctx = super(BrandDetailView, self).get_context_data(**kwargs)
        ctx['models_in_home'] = self.get_models_in_home()
        ctx['models'] = self.object.models.filter(status__in=('online', 'runout'))
        ctx['models_in_aside'] = CarModel.objects.filter(
            status__in=('online', 'runout'),
            new_entry__in=('side', 'home'),
        ).order_by('-id')[:3]
        ctx['hits'] = Hit.objects.filter(status=1).order_by('ordering')[:3]
        breadcrumbs = settings.PUBLIEDITOR_CONTENT_BREADCRUMBS + [{'title': _(self.object.name), 'url': None}]
        ctx['breadcrumbs'] = breadcrumbs
        return ctx


class CarModelDetailView(DetailView):

    model = CarModel
    template_name = "content/carmodel_detail.html"

    def get(self, request, *args, **kwargs):
        if settings.PUBLIEDITOR_REPLACE_UNDERSCORE:
            old_full_path = request.get_full_path()
            new_full_path = old_full_path.lower().replace('_', '-').replace('.', '')
            if old_full_path != new_full_path:
                return HttpResponseRedirect(new_full_path)
        return super(CarModelDetailView, self).get(request, *args, **kwargs)

    def get_object(self):
        return super(CarModelDetailView, self).get_object(
            queryset=CarModel.objects.filter(
                status__in=('online', 'runout'),
                brand__status='active'
            )
        )

    def get_context_data(self, **kwargs):
        ctx = super(CarModelDetailView, self).get_context_data(**kwargs)
        ctx['name'] = "%s %s" % (self.object.brand.name, self.object.name)
        ctx['versions'] = self.object.cars.filter(status='ok', model__status__in=('online', 'runout'))
        ctx['similar_models'] = self.object.similar_models()[:5]
        ctx['variant_models'] = self.object.get_others_variants()
        standard_equipments = self.object.get_standard_equipments().order_by('description')
        if standard_equipments:
            count_equipments = standard_equipments.count()
            half_count = count_equipments - (count_equipments / 2)
            ctx['equipments_1'] = standard_equipments[:half_count]
            ctx['equipments_2'] = standard_equipments[half_count:count_equipments]

        ctx['domain'] = settings.PUBLIEDITOR_CONTENT_BASEURL
        breadcrumbs = settings.PUBLIEDITOR_CONTENT_BREADCRUMBS + [
            {'title': self.object.brand.name, 'url': self.object.brand.get_absolute_url()},
            {'title': self.object.name, 'url': None}
        ]
        ctx['breadcrumbs'] = breadcrumbs
        return ctx


class CarDetailView(DetailView):

    model = Car
    template_name = "content/car_detail.html"

    def get(self, request, *args, **kwargs):
        if settings.PUBLIEDITOR_REPLACE_UNDERSCORE:
            old_full_path = request.get_full_path()
            new_full_path = old_full_path.lower().replace('_', '-').replace('.', '')
            if old_full_path != new_full_path:
                return HttpResponseRedirect(new_full_path)
        return super(CarDetailView, self).get(request, *args, **kwargs)

    def get_object(self):
        return super(CarDetailView, self).get_object(
            queryset=Car.objects.filter(
                status='ok',
                model__status__in=('online', 'runout'),
                brand__status='active'
            )
        )

    def get_context_data(self, **kwargs):
        ctx = super(CarDetailView, self).get_context_data(**kwargs)
        ctx['name'] = "%s %s %s" % (self.object.model.brand.name, self.object.model.name, self.object.version)
        ctx['similar_models'] = self.object.model.similar_models()[:5]
        ctx['similar_cars_same_model'] = self.object.similar_cars_same_model()[:5]
        ctx['similar_cars_other_models'] = self.object.similar_cars_other_models()[:5]
        standard_equipments = self.object.get_standard_equipments().order_by('carmodel_equipment__description')
        if standard_equipments:
            count_equipments = standard_equipments.count()
            half_count = count_equipments - (count_equipments / 2)
            ctx['equipments_1'] = standard_equipments[:half_count]
            ctx['equipments_2'] = standard_equipments[half_count:count_equipments]

        ctx['domain'] = settings.PUBLIEDITOR_CONTENT_BASEURL
        breadcrumbs = settings.PUBLIEDITOR_CONTENT_BREADCRUMBS + [
            {'title': self.object.model.brand.name, 'url': self.object.model.brand.get_absolute_url()},
            {'title': self.object.model.name, 'url': self.object.model.get_absolute_url()},
            {'title': _(self.object.version), 'url': None}
        ]
        ctx['breadcrumbs'] = breadcrumbs
        return ctx


class EmbedView(ListView):

    template_name = 'content/embed.html'
    queryset = Brand.objects.filter(models__status="online").order_by('name').distinct()

    def get_context_data(self, **kwargs):
        ctx = super(EmbedView, self).get_context_data(**kwargs)
        ctx['models_in_home'] = CarModel.objects.filter(
            status__in=('online', 'runout'),
            new_entry__in=('side', 'home'),
        ).order_by('-id')[:3]
        ctx['hits'] = Hit.objects.filter(status=1).order_by('ordering')[:3]
        return ctx


class EmbeddxView(ListView):

    template_name = 'content/embeddx.html'
    queryset = Brand.objects.filter(models__status="online").order_by('name').distinct()

    def get_context_data(self, **kwargs):
        ctx = super(EmbeddxView, self).get_context_data(**kwargs)
        ctx['hits'] = Hit.objects.filter(status=1).order_by('ordering')[:3]
        return ctx


class CompareListViewNew(CompareListView):
    template_name = "content/compare_new.html"


class CarModelDetailViewNew(CarModelDetailView):
    template_name = "content/carmodel_detail_new.html"
    
    
class CarDetailViewNew(CarDetailView):
    template_name = "content/car_detail_new.html"


class PrezziautoListViewNew(PrezziautoListView):
    template_name = 'content/prezziauto_new.html'


class HitListViewNew(HitListView):
    template_name = 'content/hit_list_new.html'


class HitDetailViewNew(HitDetailView):
    template_name = 'content/hit_detail_new.html'


class FavouritesListViewNew(FavouritesListView):
    template_name = "content/favourites_new.html"


class BrandDetailViewNew(BrandDetailView):
    template_name = "content/brand_detail_new.html"